@@ -61,6 +61,8 @@
 #include "udm_uniconv.h"
 #include "udm_unidata.h"
 
+extern int new_version;
+
 /* Some constants */
 #define UDM_LANGPERDOC				16		/* FIXME */
 #define UDM_USER_AGENT				"MnoGoSearch/" VERSION
@@ -92,14 +94,17 @@
 #define UDM_OPEN_MODE_READ	0
 #define UDM_OPEN_MODE_WRITE	1
 
-
-/* search modes */
+/* TODO: remove deprecated constants*/
 #define UDM_ORD_RATE		0
 #define UDM_ORD_DATE		1
-#define UDM_MODE_ALL		0
-#define UDM_MODE_ANY		1
-#define UDM_MODE_BOOL		2
-#define UDM_MODE_PHRASE		3
+
+/* search modes */
+#define UDM_MODE_ALL             0
+#define UDM_MODE_ANY             1
+#define UDM_MODE_BOOL            2
+#define UDM_MODE_PHRASE          3
+#define UDM_MODE_ALL_MINUS       4
+#define UDM_MODE_ALL_MINUS_HALF  5
 
 /* word match type */
 #define UDM_MATCH_FULL		0
@@ -164,12 +169,21 @@
 #define UDM_TAGSIZE	4
 
 /* Words origins */
-#define UDM_WORD_ORIGIN_QUERY   1
-#define UDM_WORD_ORIGIN_SPELL   2
-#define UDM_WORD_ORIGIN_SYNONYM 4
-#define UDM_WORD_ORIGIN_STOP    8
-#define UDM_WORD_ORIGIN_SUGGEST 16
-#define UDM_WORD_ORIGIN_COLLATION 32
+#define UDM_WORD_ORIGIN_QUERY           1
+#define UDM_WORD_ORIGIN_SPELL           2
+#define UDM_WORD_ORIGIN_SYNONYM         3
+#define UDM_WORD_ORIGIN_SYNONYM_FINAL   4
+#define UDM_WORD_ORIGIN_STOP            5
+#define UDM_WORD_ORIGIN_SUGGEST         6
+#define UDM_WORD_ORIGIN_COLLATION       7
+
+/* URL data flags */
+#define UDM_URLDATA_URL	        1
+#define UDM_URLDATA_SITE        2
+#define UDM_URLDATA_POP         4
+#define UDM_URLDATA_LM          8 
+#define UDM_URLDATA_SU         16
+#define UDM_URLDATA_CACHE      32
 
 /* Locking mutex numbers */
 #define UDM_LOCK_DUMMY          0
@@ -315,33 +329,28 @@ typedef struct
 } UDM_URLCRDLIST;
 
 
-typedef struct
-{
-  int                freeme;
-  size_t             nlists;
-  UDM_URLCRDLIST     *List;
-} UDM_URLCRDLISTLIST;
-
 /***************************/
 
 typedef struct udm_coord2_st
 {
-  udm_pos_t pos;
+  udm_pos_t pos:24;
+  udm_wordnum_t order;
 } UDM_COORD2;
 
 
 typedef struct udm_section_st
 {
-  UDM_COORD2 *Coord;
-  urlid_t url_id;
-  uint4 ncoords;
-  udm_pos_t seclen;
-  udm_pos_t minpos;
-  udm_pos_t maxpos;
-  udm_secno_t secno;
-  udm_wordnum_t wordnum;
-  udm_wordnum_t order;
-} UDM_SECTION;
+  UDM_COORD2 *Coord;                 /* 4/8 bytes */
+  const unsigned char *PackedCoord;  /* 4/8 bytes */
+  urlid_t url_id;                    /* 4   bytes */
+  udm_pos_t ncoords;                 /* 4   bytes */
+  udm_pos_t seclen;                  /* 4   bytes */
+  udm_pos_t minpos;                  /* 4   bytes */
+  udm_pos_t maxpos;                  /* 4   bytes */
+  udm_secno_t secno;                 /* 1   byte  */
+  udm_wordnum_t wordnum;             /* 1   byte  */
+  udm_wordnum_t order;               /* 1   byte  */
+} UDM_SECTION;                       /* 32 bytes (i386), 40 bytes (64bit) */
 
 
 typedef struct udm_sectionlist_st
@@ -641,7 +650,7 @@ typedef struct {
 	UDM_CHARSET		*lcs;		/**< LocalCharser */
 	UDM_SPIDERPARAM		Spider;		/**< Spider prms  */
 	UDM_CONN		connp;		/**< For FTP      */
-	
+	UDM_CONN		connp2;		/**< For FTP      */
 } UDM_DOCUMENT;
 
 /********************************************************/
@@ -756,11 +765,14 @@ typedef struct{
 
 typedef struct {
 	size_t		order;
+        size_t          order_extra_width; /* For multi-word synonyms, see below */
 	size_t		count;
+	size_t          doccount; /* Number of documents this word appears in */
 	char		*word;
 	size_t		len;
         int     	origin;
-        int             weight; /* origin-dependent weight */
+        int             weight; /* origin-dependent weight   */
+        int             user_weight; /* User-supplied weight */
         int		match;
         size_t		secno;  /* Which section to search in */
         size_t		phrpos; /* 0 means "not in phrase"    */
@@ -768,6 +780,16 @@ typedef struct {
         size_t          phrwidth; /* How many additional parts in a multi-word */
 } UDM_WIDEWORD;
 
+/*
+  order_extra_width - use in case of many-to-one and many-to-many synonyms.
+  It represents the number of query words this synonym covers.
+  For example, if we have synonym:
+  "aaaa bbbb" -> "cccc"
+  then origin_extra_width for the words "cccc" will be 2, because
+  it is a replacement for two words.
+  0 means "not a multi-word replacement".
+*/
+
 typedef struct {
 	int		wm;              /* Search mode: wrd, sub, beg, end */
 	int             strip_noaccents; /* If accent insensitive comparison*/
@@ -780,6 +802,7 @@ typedef struct {
 typedef struct {
 	char	*p;
 	char	*s;
+	int     origin; /* SYNONYM or SYNONYM_FINAL */
 } UDM_SYNONYM;
 
 #define UDM_SYNONYM_LANGLEN 32
@@ -795,6 +818,7 @@ typedef struct {
         char  lang[UDM_SYNONYM_LANGLEN];
         char  cset[UDM_SYNONYM_CSETLEN];
         char  fname[UDM_SYNONYM_FILELEN];
+        size_t max_phrase_length; /* Many-to-many, many-to-one, one-to-many */
 } UDM_SYNONYMLIST;
 
 
@@ -889,6 +913,15 @@ typedef struct {
 	UDM_DB		*db;
 } UDM_DBLIST;
 
+
+typedef struct
+{
+  int is_log_open; /* Flag indicating if openlog() has been called      */
+  FILE *logFD;     /* File descriptor, when logging to stderr or file   */
+  int facility;    /* Which facility to use, or negative number if none */
+} UDM_LOG;
+
+
 /** Forward declaration of UDM_AGENT */
 struct udm_indexer_struct;
 
@@ -937,13 +970,15 @@ typedef struct udm_config_struct {
         UDM_CHINALIST   Chi;            /**< Chinese words list         */
         UDM_CHINALIST   Thai;           /**< Thai words list            */
 	
-	/* Various file descriptors */
-	int		is_log_open;	/**< if UdmOpenLog is already called   */
-	FILE		*logFD;		/**< FILE structure, syslog descriptor */
+	UDM_LOG Log;
+	
 	int		CVS_ignore;	/**< Skip CVS directgories - for tests */
+	
+	/* Various virtual functions */
 	void (*ThreadInfo)(struct udm_indexer_struct *,const char * state,const char * str);
 	void (*LockProc)(struct udm_indexer_struct *,int command,int type,const char *fname,int lineno);
 	void (*RefInfo)(int code,const char *url, const char *ref);
+	int  (*DumpDoc)(struct udm_indexer_struct *, UDM_DOCUMENT *Doc);
 	int (*ThreadCreate)(void **thd, void *(*start_routine)(void *), void *arg);
 	int (*ThreadJoin)(void *thd);
 #ifdef MECAB
@@ -1095,6 +1130,7 @@ enum udm_indcmd
   UDM_IND_AMBIGUOUS,
   UDM_IND_UNKNOWN,
   UDM_IND_INDEX,
+  UDM_IND_CRAWL,
   UDM_IND_STAT,
   UDM_IND_CREATE,
   UDM_IND_DROP,
@@ -1110,7 +1146,9 @@ enum udm_indcmd
   UDM_IND_HASHSPELL,
   UDM_IND_DUMPSPELL,
   UDM_IND_REWRITELIMITS,
-  UDM_IND_DUMPCONF
+  UDM_IND_DUMPCONF,
+  UDM_IND_DUMPDATA,
+  UDM_IND_RESTOREDATA
 };
 
 #endif
