@@ -61,6 +61,8 @@
 #define UDM_URL_ACTION_DOCPERSITE       17
 #define UDM_URL_ACTION_SQLIMPORTSEC     18
 #define UDM_URL_ACTION_FLUSH            19
+#define UDM_URL_ACTION_DUMPDATA         20 
+#define UDM_URL_ACTION_RESTOREDATA      21
 
 #define UDM_RES_ACTION_DOCINFO		1
 #define UDM_RES_ACTION_INSWORDS		2
@@ -113,13 +115,14 @@ extern int UdmHTDBGet(UDM_AGENT *Indexer,UDM_DOCUMENT *Doc);
 extern int UdmCheckUrlid(UDM_AGENT *A, urlid_t id);
 
 extern int UdmDBSetAddr(UDM_DB *db, const char *dbaddr, int mode);
+
+extern size_t UdmDBNumBySeed(UDM_ENV *Env, udmhash32_t seed);
 /******************/
 
 typedef struct st_udm_findword_args
 {
   UDM_AGENT *Agent;
   UDM_DB *db;
-  UDM_URLCRDLISTLIST CoordListList;
   UDM_WIDEWORDLIST *WWList;
   UDM_URLID_LIST urls; /* Combination of WHERE limit, fl, and live_deleted */
   UDM_URLID_LIST live_update_active_urls;  /* Combination of WHERE and fl */
@@ -132,6 +135,8 @@ typedef struct st_udm_findword_args
   UDM_WIDEWORD Word;
   int save_section_size;
   int live_updates;
+  int need_coords; /* 0 for regular word, 1 for multi-word */
+  UDM_SQLRESLIST SQLResults;
 } UDM_FINDWORD_ARGS;
 
 
@@ -186,14 +191,6 @@ UdmMultiCachePutIntagUsingEncoding(UDM_MULTI_CACHE_WORD *cache,
 extern char*
 UdmMultiCachePutIntagUsingRaw(UDM_MULTI_CACHE_WORD *cache);
 
-extern int UdmDeleteWordsFromURLBlob(UDM_AGENT *A, UDM_DB *db, urlid_t url_id);
-extern int UdmDeleteWordsFromURLSingle(UDM_AGENT *A, UDM_DB *db, urlid_t url_id);
-extern int UdmDeleteWordsFromURLMulti(UDM_AGENT *A, UDM_DB *db, urlid_t url_id);
-
-extern int UdmStoreWordsBlob(UDM_AGENT *Indexer,
-                             UDM_DOCUMENT *Doc,
-                             UDM_DB *db);
-
 extern int UdmConvert2BlobSQL(UDM_AGENT *Indexer, UDM_DB *db);
 extern __C_LINK int __UDMCALL UdmMulti2Blob (UDM_AGENT *Indexer);
 extern __C_LINK int __UDMCALL UdmRewriteURL(UDM_AGENT *Indexer);
@@ -203,6 +200,26 @@ extern int UdmBlobWriteURL(UDM_AGENT *A, UDM_DB *db, const char *table,
 extern int UdmBlobWriteLimits(UDM_AGENT *A, UDM_DB *db, const char *table,
     int use_deflate);
 
+/************* DBMode handler **************/
+
+typedef struct udm_dbmode_handler_st
+{
+  const char *name;
+  int (*ToBlob)(UDM_AGENT *Indexer, UDM_DB *db, UDM_URLDATALIST *URLList);
+  int (*StoreWords)(UDM_AGENT *Indexer, UDM_DB *db, UDM_DOCUMENT *Doc);
+  int (*Truncate)(UDM_AGENT *Indexer, UDM_DB *db);
+  int (*DeleteWordsFromURL)(UDM_AGENT *Indexer, UDM_DB *db, urlid_t url_id);
+  int (*FindWord)(UDM_FINDWORD_ARGS *args);
+  int (*DumpWordInfo)(UDM_AGENT *A, UDM_DB *db, UDM_DOCUMENT *Doc);
+  int (*WordStatCreate)(UDM_AGENT *A, UDM_DB *db);
+} UDM_DBMODE_HANDLER;
+
+extern UDM_DBMODE_HANDLER udm_dbmode_handler_single;
+extern UDM_DBMODE_HANDLER udm_dbmode_handler_multi;
+extern UDM_DBMODE_HANDLER udm_dbmode_handler_blob;
+extern UDM_DBMODE_HANDLER udm_dbmode_handler_rawblob;
+
+
 /* Functions from dbmode-blob.c */
 extern UDM_BLOB_CACHE *UdmBlobCacheInit (UDM_BLOB_CACHE *cache);
 extern void UdmBlobCacheFree (UDM_BLOB_CACHE *cache);
@@ -215,75 +232,64 @@ extern size_t UdmBlobCacheAdd2(UDM_BLOB_CACHE *cache, urlid_t url_id,
                                size_t nintags, char *intag,
                                size_t intag_length);
 
-extern int UdmFindWordBlob(UDM_FINDWORD_ARGS *args);
-extern int UdmFindWordBlobLiveUpdates(UDM_FINDWORD_ARGS *args);
-extern int UdmBlobGetTable(UDM_DB *db);
-extern int UdmBlobSetTable(UDM_DB *db);
-extern int UdmBlobGetWTable(UDM_DB *db, const char **name);
-extern const char *UdmBlobGetRTable (UDM_DB *db);
+extern int UdmBlobLoadLiveUpdateLimit(UDM_FINDWORD_ARGS *args, UDM_AGENT *A, UDM_DB *db);
+
+extern int UdmBlobSetTable(UDM_AGENT *A, UDM_DB *db);
+extern int UdmBlobGetWTable(UDM_AGENT *A, UDM_DB *db, const char **name);
 extern int UdmBlobWriteTimestamp(UDM_AGENT *A, UDM_DB *db, const char *table);
 extern int UdmBlobCacheWrite(UDM_DB *db,
                              UDM_BLOB_CACHE *cache,
                              const char *table, int use_deflate);
-extern int UdmBlob2BlobSQL(UDM_AGENT *Indexer, UDM_DB *db, UDM_URLDATALIST *URLList);
 extern int UdmBlobWriteLimitsInternal(UDM_AGENT *A, UDM_DB *db, 
                                       const char *table,
                                       int use_deflate);
 extern int UdmBlobLoadFastURLLimit(UDM_DB *db, const char *name,
                                    UDM_URLID_LIST *buf);
 extern int UdmBlobLoadFastOrder(UDM_DB *db, UDM_URL_INT4_LIST *,const char *name);
-extern int UdmTruncateDictBlob(UDM_AGENT *Indexer,UDM_DB *db);
+extern int UdmLoadURLDataFromBdict(UDM_AGENT *A, UDM_URLDATALIST *DataList,
+                                   UDM_DB *db, size_t num_best_rows, int flags);
 
-
-/* Functions from dbmode-multi.c */
-extern int UdmStoreWordsMulti(UDM_AGENT *Indexer,
-                              UDM_DOCUMENT *Doc,
-                              UDM_DB *db);
-extern int UdmFindWordMulti(UDM_FINDWORD_ARGS *args);
-extern int UdmMulti2BlobSQL(UDM_AGENT *Indexer,
-                            UDM_DB *db, UDM_URLDATALIST
-                            *URLList);
-extern int UdmTruncateDictMulti(UDM_AGENT *Indexer,UDM_DB *db);
+extern UDM_COORD2 *UdmBlobPackedCoordsToUnpackedCoords(const unsigned char *s,
+                                                       const unsigned char *e,
+                                                       size_t nrecs,
+                                                       UDM_COORD2 *C,
+                                                       UDM_COORD2 *Coord,
+                                                       const unsigned char **end);
 
 /* Functions from dbmode-single.c */
-extern int StoreWordsSingle(UDM_AGENT *Indexer, UDM_DOCUMENT *Doc,UDM_DB *db);
-
 extern int UdmStoreCrossWords(UDM_AGENT *Indexer, UDM_DOCUMENT *Doc,UDM_DB *db);
-
 extern int UdmTruncateCrossDict(UDM_AGENT *Indexer,UDM_DB *db);
 extern int UdmDeleteCrossWordFromURL(UDM_AGENT *Indexer,
                                      UDM_DOCUMENT *Doc, UDM_DB *db);
-extern int UdmFindWordSingle(UDM_FINDWORD_ARGS *args);
 extern int UdmFindCrossWord(UDM_FINDWORD_ARGS *args);
 extern int UdmURLCRDListListAddWithSort2(UDM_FINDWORD_ARGS *args,
-                                         UDM_URLCRDLISTLIST *List,
                                          UDM_URLCRDLIST *CoordList);
-extern int UdmSingle2BlobSQL(UDM_AGENT *Indexer,
-                             UDM_DB *db, UDM_URLDATALIST *URLList);
+extern int UdmMultiWordAdd(UDM_FINDWORD_ARGS *args,
+                           UDM_SECTIONLISTLIST *OriginalSectionListList,
+                           size_t orig_wordnum, size_t nparts);
 
 extern int UdmAddOneCoord(UDM_URLCRDLIST *CoordList,
                           urlid_t url_id, uint4 coord, udm_wordnum_t num);
 
-extern int UdmTruncateDictSingle(UDM_AGENT *Indexer,UDM_DB *db);
+extern int UdmLoadURLDataFromURL(UDM_AGENT *A, UDM_URLDATALIST *DataList,
+                                 UDM_DB *db, size_t num_best_rows, int flag);
 
 /* Functions from dbmode-rawblob.c */
 #define UDM_RAWBLOB_SEARCH  0
 #define UDM_RAWBLOB_DELTA   1
-extern int UdmFindWordRawBlob(UDM_FINDWORD_ARGS *args, int flags);
+extern int UdmFindWordRawBlobDelta(UDM_FINDWORD_ARGS *args);
+
 
 /* Functions from sql.c */
 extern int UdmFindURL(UDM_AGENT *Indexer, UDM_DOCUMENT * Doc, UDM_DB *db);
 extern int UdmApplyFastLimit(UDM_URLCRDLIST *Coord, UDM_URLID_LIST *urls);
-extern int UdmURLCRDListToSectionList(UDM_FINDWORD_ARGS *args,
-                                      UDM_SECTIONLIST *SectionList,
-                                      UDM_URLCRDLIST *CoordList);
 extern int UdmBuildCmpArgSQL(UDM_DB *db, int match, const char *word,
                              char *cmparg, size_t maxlen);
-extern const char* UdmBlobModeInflateOrAlloc(UDM_AGENT *A,
-                                            UDM_DSTR *buf, const char *name,
-                                            const char *src, size_t *len);
 extern const char* UdmSQLBuildWhereCondition(UDM_ENV * Conf,UDM_DB *db);
 
+extern int UdmURLDataListGroupBySiteUsingSort(UDM_AGENT *A,
+                                              UDM_URLDATALIST *R,
+                                              UDM_DB *db);
 /***************/
 
 int       
@@ -320,5 +326,9 @@ extern int UdmWordCacheAddURL (UDM_WORD_CACHE *cache, urlid_t url_id);
 extern int UdmWordCacheWrite (UDM_AGENT *Indexer, UDM_DB *db, size_t limit);
 extern int UdmWordCacheFlush (UDM_AGENT *Indexer);
 
+/* Functions from suggest.c */
+extern int UdmWordStatQuery(UDM_AGENT *A, UDM_DB *db, const char *src);
+extern int UdmWordStatCreate(UDM_AGENT *A, UDM_DOCUMENT *D, UDM_DB *db);
+extern int UdmResSuggest(UDM_AGENT *A, UDM_DB *db, UDM_RESULT *Res, size_t dbnum);
 
 #endif
