@@ -20,15 +20,6 @@
 
 #include <limits.h>
 
-#define UDM_SQLTYPE_UNKNOWN       0
-#define UDM_SQLTYPE_LONGVARBINARY 1
-#define UDM_SQLTYPE_LONGVARCHAR   2
-#define UDM_SQLTYPE_VARCHAR       3
-#define UDM_SQLTYPE_INT           4
-
-#define UDM_SQL_NULL_DATA	(-1)
-#define UDM_SQL_DATA_AT_EXEC	(-2)
-#define UDM_SQL_NTS		(-3)
 
 /* Multi-dict mode defines */
 #define MULTI_DICTS	0xff
@@ -39,10 +30,6 @@
 #define UDM_MAX_MULTI_INSERT_QSIZE	16*1024
 #define NDOCS_QUERY			"SELECT count(*) FROM url"
 
-#define UDM_SQL_UNKNOWN	0
-#define UDM_SQL_SELECT	1
-#define UDM_SQL_UPDATE	2
-
 typedef struct {
 	char	*sqlname;
 	int	sqltype;
@@ -160,24 +147,26 @@ typedef struct {
 
 /*******************************************/
 struct udm_db_st;
-
+struct udm_dbmode_handler_st;
 
 typedef struct udm_db_handler_st
 {
-  char	*(*SQLEscStr)(struct udm_db_st *db, char *to, const char *from, size_t len);
+  size_t (*SQLEscStr)(struct udm_db_st *db, char *to, const char *from, size_t len);
   int    (*SQLQuery)(struct udm_db_st *db, UDM_SQLRES *R, const char *query);
   int    (*SQLConnect)(struct udm_db_st *db);
   void   (*SQLClose)(struct udm_db_st *db);
   int    (*SQLBegin)(struct udm_db_st *db);
   int    (*SQLCommit)(struct udm_db_st *db);
   int    (*SQLPrepare)(struct udm_db_st *db, const char *query);
-  int    (*SQLBind)(struct udm_db_st *db, int position, void *data, int size, int type);
+  int    (*SQLBind)(struct udm_db_st *db, int position, const void *data, int size, int type);
   int    (*SQLExec)(struct udm_db_st *db);
   int    (*SQLStmtFree)(struct udm_db_st *db);
   int    (*SQLFetchRow)(struct udm_db_st *db, UDM_SQLRES *R, UDM_PSTR *);
   int    (*SQLStoreResult)(struct udm_db_st *db, UDM_SQLRES *R);
   int    (*SQLFreeResult)(struct udm_db_st *db, UDM_SQLRES *R);
   int    (*SQLExecDirect)(struct udm_db_st *db, UDM_SQLRES *R, const char *q);
+  int    (*SQLRenameTable)(struct udm_db_st *db, const char *from, const char *to);
+  int    (*SQLCopyStructure)(struct udm_db_st *db, const char *from, const char *to);
 } UDM_SQLDB_HANDLER;
 
 extern UDM_SQLDB_HANDLER udm_sqldb_ctlib_handler;
@@ -212,6 +201,7 @@ typedef struct udm_db_st
 	int		  searchd;    /* Searchd daemon descriptor         */
         UDM_VARLIST       Vars;       /* optional parameters and variables */
 	UDM_SQLDB_HANDLER *sql;
+	struct udm_dbmode_handler_st *dbmode_handler;
 	void	*specific;
 	void    *ps;                  /* for prepared statements */
 
@@ -219,7 +209,7 @@ typedef struct udm_db_st
 } UDM_DB;
 
 int UdmSQLPrepareGeneric(UDM_DB *db, const char *query);
-int UdmSQLBindGeneric(UDM_DB *db, int position, void *data, int size, int type);
+int UdmSQLBindGeneric(UDM_DB *db, int position, const void *data, int size, int type);
 int UdmSQLExecGeneric(UDM_DB *db);
 int UdmSQLStmtFreeGeneric(UDM_DB *db);
 
