@@ -19,6 +19,8 @@
 #define _UDM_SEARCH_TOOL_H
 
 #define UDM_FAST_PRESORT_DOCS	300
+#define UDM_DEFAULT_USER_WORD_WEIGHT 256
+
 
 /* Functions form urldata.c */
 extern void UdmURLDataSortBySite(UDM_URLDATALIST *L);
@@ -27,7 +29,22 @@ extern void UdmURLDataSortByPattern(UDM_URLDATALIST *D, const char *pattern);
 extern int UdmURLDataListSearch(UDM_URLDATALIST *List, urlid_t id);
 extern int UdmURLDataListClearParams(UDM_URLDATALIST *List, size_t num_best_rows);
 extern size_t UdmURLDataCompact(UDM_URLDATA *dst, UDM_URLDATA *src, size_t n);
+extern int UdmURLDataListGroupBySiteUsingHash(UDM_AGENT *A,
+                                              UDM_URLDATALIST *DataList,
+                                              const char *rec_id_str, size_t rec_id_len,
+                                              const char *site_id_str, size_t site_id_len);
+/* Functions from distance.c */
+typedef struct udm_distance_stat_st
+{
+  size_t sum;
+  size_t num;
+} UDM_WORD_DISTANCE_STAT;
 
+
+extern void CalcAverageWordDistance(size_t wf2_secno,
+                                    UDM_COORD2 *phr, size_t num, size_t nuniq,
+                                    UDM_WORD_DISTANCE_STAT *dist);
+                                                 
 /* Functions from score.c */
 extern int UdmUserScoreListApplyToURLScoreList(UDM_URLSCORELIST *URLScoreList,
                                                UDM_URL_INT4_LIST *UserScoreList,
@@ -65,6 +82,7 @@ int UdmAllForms(UDM_AGENT *Indexer,
                 UDM_WIDEWORDLIST *result,
                 UDM_WIDEWORD *uword);
 
+int UdmComplexSynonyms(UDM_AGENT *Indexer, UDM_WIDEWORDLIST *result);
 
 /* Functions from highlight.c */
 extern char *UdmHlConvert(UDM_WIDEWORDLIST *L,const char * src, UDM_CHARSET * lcs, UDM_CHARSET * bcs);
@@ -99,25 +117,21 @@ int UdmSectionListListMergeSorted(UDM_SECTIONLISTLIST *SrcList,
                                   UDM_SECTIONLIST *Dst, int opt);
 
 /* Functions from urlidlist.c */
+int UdmCmpURLID(urlid_t *s1, urlid_t *s2); /* for qsort */
 int UdmURLIdListJoin(UDM_URLID_LIST *urls, UDM_URLID_LIST *fl_urls);
 int UdmURLIdListUnion(UDM_URLID_LIST *a, UDM_URLID_LIST *b);
 int UdmURLIdListCopy(UDM_URLID_LIST *a, UDM_URLID_LIST *b);
 int UdmURLIdListMerge(UDM_URLID_LIST *a, UDM_URLID_LIST *b);
+int UdmURLIdListSort(UDM_URLID_LIST *a);
 
 /* Functions from coords.c */
-void
-UdmURLCRDListListInit(UDM_URLCRDLISTLIST *Lst);
-
-int
-UdmURLCRDListListAdd(UDM_URLCRDLISTLIST *LstLst, UDM_URLCRDLIST *List);
-
-void
-UdmURLCRDListListFree(UDM_URLCRDLISTLIST *Lst);
-
 void
 UdmURLCRDListSortByURLThenSecnoThenPos(UDM_URLCRDLIST *L);
 
-void
-UdmURLCRDListMergeMultiWord(UDM_URLCRDLIST *Phrase, size_t wordnum, size_t nparts);
 
+/* Functions from sql.c */
+int
+UdmUserSiteScoreListLoadAndApplyToURLDataList(UDM_AGENT *Agent,
+                                              UDM_URLDATALIST *List,
+                                              UDM_DB *db);
 #endif
