@@ -18,6 +18,18 @@
 #ifndef UDM_SQLDBMS_H
 #define UDM_SQLDBMS_H
 
+
+/* SQL Data types */
+#define UDM_SQLTYPE_UNKNOWN       0
+#define UDM_SQLTYPE_LONGVARBINARY 1
+#define UDM_SQLTYPE_LONGVARCHAR   2
+#define UDM_SQLTYPE_VARCHAR       3
+#define UDM_SQLTYPE_INT           4
+
+#define UDM_SQL_NULL_DATA	(-1)
+#define UDM_SQL_DATA_AT_EXEC	(-2)
+#define UDM_SQL_NTS		(-3)
+
 #define UDM_SQL_HAVE_GROUPBY       1
 #define UDM_SQL_HAVE_TRUNCATE      2
 #define UDM_SQL_HAVE_SUBSELECT     4
@@ -32,21 +44,34 @@
 #define UDM_SQL_DEBUG_QUERY     2048
 #define UDM_SQL_HAVE_FIRST_SKIP 4096  /* SELECT FIRST 10 * FROM t1 */
 #define UDM_SQL_HAVE_TRANSACT   8192  /* If transactions are  supported */
+#define UDM_SQL_HAVE_RENAME         0x00004000 /* If can rename tables */
+#define UDM_SQL_HAVE_CREATE_LIKE    0x00008000 /* If can CREATE TABLE LIKE */
+#define UDM_SQL_HAVE_DROP_IF_EXISTS 0x00010000 /* If can DROP TABLE IF EXISTS */
 
 #define UDM_SQL_MAX_BIND_PARAM 64
 
-extern void DecodeHexStr (char *src, UDM_PSTR *dst, size_t size);
-char*  UdmSQLEscStrGeneric(UDM_DB *db, char *to, const char *from, size_t len);
-char*  UdmSQLEscStr(UDM_DB *db, char *to, const char *from, size_t l);
+extern void DecodeHexStr (const char *src, UDM_PSTR *dst, size_t size);
+extern int UdmSQLEscDSTR(UDM_DB *db, UDM_DSTR *dst, const char *src, size_t srclen);
+
+size_t UdmSQLEscStrGeneric(UDM_DB *db, char *to, const char *from, size_t len);
+size_t UdmSQLEscStr(UDM_DB *db, char *to, const char *from, size_t l);
+char*  UdmSQLEscStrAlloc(UDM_DB *db, const char *src, size_t srclen);
 char*  UdmSQLEscStrSimple(UDM_DB *db, char *to, const char *from, size_t l);
-size_t UdmSQLBinEscStr(UDM_DB *db, char *dst, const char *src0, size_t len);
+size_t UdmSQLBinEscStr(UDM_DB *db, char *dst, size_t dstlen, const char *src, size_t srclen);
 int UdmSQLTableTruncateOrDelete(UDM_DB *db, const char *name);
 int UdmSQLDropTableIfExists(UDM_DB *db, const char *name);
 int UdmSQLQueryOneRowInt(UDM_DB *db, int *res, const char *qbuf);
-void UdmSQLTopClause(UDM_DB *db, size_t top_num,
-                     char *topstr, size_t topstr_size,
-                     char *rownumstr, size_t rownumstr_size,
-                     char *limitstr, size_t limitstr_size);
+
+#define UDM_SQL_TOP_BUF_SIZE 64
+
+typedef struct udm_sql_top_st
+{
+  char rownum[UDM_SQL_TOP_BUF_SIZE];
+  char limit[UDM_SQL_TOP_BUF_SIZE];
+  char top[UDM_SQL_TOP_BUF_SIZE];
+} UDM_SQL_TOP_CLAUSE;
+void UdmSQLTopClause(UDM_DB *db, size_t top_num, UDM_SQL_TOP_CLAUSE *Top);
+
 const char *UdmSQLParamPlaceHolder(UDM_DB *db, size_t i);
 int UdmSQLLongVarCharBindType(UDM_DB *db);
 
@@ -69,7 +94,13 @@ extern int UdmSQLPrepareDebug(UDM_DB *db, const char *q, const char *file, const
 extern int UdmSQLExecuteDebug(UDM_DB *db, const char *file, const int line);
 extern int UdmSQLStmtFreeDebug(UDM_DB *db, const char *file, const int line);
 extern int UdmSQLBindParameterDebug(UDM_DB *db,
-                                    int pos, void *data, int size, int type,
+                                    int pos, const void *data, int size, int type,
+                                    const char *file, const int line);
+extern int UdmSQLRenameTableDebug(UDM_DB *db,
+                                  const char *from, const char *to,
+                                  const char *file, const int line);
+extern int UdmSQLCopyStructureDebug(UDM_DB *db,
+                                    const char *from, const char *to,
                                     const char *file, const int line);
 #define UdmSQLBegin(db)   UdmSQLBeginDebug(db, __FILE__, __LINE__)
 #define UdmSQLCommit(db)  UdmSQLCommitDebug(db, __FILE__, __LINE__)
@@ -78,6 +109,8 @@ extern int UdmSQLBindParameterDebug(UDM_DB *db,
 #define UdmSQLExecute(db) UdmSQLExecuteDebug(db, __FILE__, __LINE__)
 #define UdmSQLStmtFree(db) UdmSQLStmtFreeDebug(db, __FILE__, __LINE__)
 #define UdmSQLBindParameter(db, pos, data, size, type) UdmSQLBindParameterDebug(db, pos, data, size, type, __FILE__, __LINE__)
+#define UdmSQLRenameTable(db, from, to) UdmSQLRenameTableDebug(db, from, to, __FILE__, __LINE__)
+#define UdmSQLCopyStructure(db, from, to) UdmSQLCopyStructureDebug(db, from, to, __FILE__, __LINE__)
 #else
 extern int UdmSQLBegin(UDM_DB *db);
 extern int UdmSQLCommit(UDM_DB *db);
@@ -85,23 +118,36 @@ extern int UdmSQLExecDirect(UDM_DB *db, UDM_SQLRES *R, const char *query);
 extern int UdmSQLPrepare(UDM_DB *db, const char *query);
 extern int UdmSQLExecute(UDM_DB *db);
 extern int UdmSQLStmtFree(UDM_DB *db);
-extern int UdmSQLBindParameter(UDM_DB *db, int pos, void *data, int size, int type);
+extern int UdmSQLBindParameter(UDM_DB *db, int pos, const void *data, int size, int type);
+extern int UdmSQLRenameTable(UDM_DB *db, const char *from, const char *to);
+extern int UdmSQLCopyStructure(UDM_DB *db, const char *from, const char *to);
 #endif
 
 
 #define UdmSQLQuery(db, R, query) _UdmSQLQuery(db, R, query, __FILE__, __LINE__)
 
 #define UDM_SQLMON_DISPLAY_FIELDS	1
+#define UDM_SQLMON_DONT_NEED_SEMICOLON  2 /* Execute immediately, for --exec */
+
 #define UDM_SQLMON_MSG_ERROR		1
 #define UDM_SQLMON_MSG_PROMPT		2
 
+typedef enum udm_sqlmon_mode_enum
+{
+  udm_sqlmon_mode_batch= 0,
+  udm_sqlmon_mode_interactive= 1
+} udm_sqlmon_mode_t;
+
 typedef struct udm_sqlmon_param_st
 {
   int flags;
   int colflags[10];
+  int loglevel;
   size_t nqueries;
   size_t ngood;
   size_t nbad;
+  size_t lineno;
+  udm_sqlmon_mode_t mode;
   FILE *infile;
   FILE *outfile;
   char *(*gets)(struct udm_sqlmon_param_st *prm, char *str, size_t size);
