@@ -28,7 +28,11 @@ extern void UdmSynonymListListFree(UDM_SYNONYMLISTLIST *L);
 extern int  UdmSynonymListListAdd(UDM_SYNONYMLISTLIST *L, UDM_SYNONYMLIST *I);
 extern UDM_WIDEWORDLIST *UdmSynonymListListFind(const UDM_SYNONYMLISTLIST *List,
                                                 UDM_WIDEWORD *wword);
+extern UDM_WIDEWORDLIST *UdmSynonymListFind(UDM_WIDEWORDLIST *Res,
+                                            const UDM_SYNONYMLIST *List,
+                                            const UDM_WIDEWORD *wword);
 extern __C_LINK void __UDMCALL UdmSynonymListListSortItems(UDM_SYNONYMLISTLIST * List);
 
+extern size_t UdmMultiWordPhraseLength(const char *word);
 
 #endif
