@@ -54,6 +54,7 @@
 #define UDM_FREE(x)		{if((x)!=NULL){UdmFree(x);x=NULL;}}
 #define UDM_SKIP(s,set)		while((*s)&&(strchr(set,*s)))s++;
 #define UDM_SKIPN(s,set)	while((*s)&&(!strchr(set,*s)))s++;
+#define UDM_SQR(x)		((x)*(x))
 
 /* NULL safe atoi*/
 #define UDM_ATOI(x)		((x)?atoi(x):0)
@@ -221,8 +222,8 @@ extern int udm_dezint4(const char *buf, int4 *array, int buf_len);
 /**************************** end of zint4 stuff *****************************/
 
 #ifdef HAVE_ZLIB
-size_t UdmInflate(char *dst, size_t dstlen, char *src, size_t srclen);
-size_t UdmDeflate(char *dst, size_t dstlen, char *src, size_t srclen);
+size_t UdmInflate(char *dst, size_t dstlen, const char *src, size_t srclen);
+size_t UdmDeflate(char *dst, size_t dstlen, const char *src, size_t srclen);
 #endif
 
 /*****************************************************************************/
