@@ -30,7 +30,7 @@ extern void UdmWideWordFree(UDM_WIDEWORD *W);
 extern UDM_WIDEWORDLIST * UdmWideWordListInit(UDM_WIDEWORDLIST * List);
 extern size_t UdmWideWordListAdd(UDM_WIDEWORDLIST * List,UDM_WIDEWORD * Word);
 extern size_t UdmWideWordListAddLike(UDM_WIDEWORDLIST *WWList,
-                                     UDM_WIDEWORD *orig, char *word);
+                                     UDM_WIDEWORD *orig, const char *word);
 extern size_t UdmWideWordListAddForStat(UDM_WIDEWORDLIST * List,UDM_WIDEWORD * Word);
 extern void UdmWideWordListFree(UDM_WIDEWORDLIST * List);
 extern int UdmWideWordListCopy(UDM_WIDEWORDLIST *Dst, UDM_WIDEWORDLIST *Src);
