@@ -20,8 +20,19 @@
 
 #define UDM_FREE(x)    {if((x)!=NULL){free(x);x=NULL;}}
 
-FILE *logfile;
+/*
+  The version with logfile does not mix
+  output from msearchtest and from the programs being
+  executed well.
+*/
 
+/*
+#define USE_LOGFILE_NOT_STDERR
+*/
+
+#ifdef USE_LOGFILE_NOT_STDERR
+FILE *logfile;
+#endif
 
 static int
 UdmSetEnv(const char * name,const char * value)
@@ -161,9 +172,13 @@ debug(const char *fmt, ...)
   str= strf(fmt, list);
   va_end(list);
 
+#ifdef USE_LOGFILE_NOT_STDERR
   fseek(logfile, 0, SEEK_END);
   fprintf(logfile, "%s: %s\n", tim, str);
   fflush(logfile);
+#else
+  fprintf(stderr, "%s: %s\n", tim, str);
+#endif
 }
 
 
@@ -684,11 +699,13 @@ int main (int argc, char **argv)
     return(1);
   }
 
+#ifdef USE_LOGFILE_NOT_STDERR
   if (!(logfile= fopen(UDM_TEST_LOG, "w")))
   {
     fprintf(stderr, "fopen '%s' failed: %s.\n", UDM_TEST_LOG, strerror(errno));
     return(1);
   }
+#endif
 
   printf("Starting tests from '%s'.\n", UDM_TEST_ROOT);
   debug("Starting test from '%s'.", UDM_TEST_ROOT);
