@@ -21,7 +21,7 @@ $MAX_PS=100;
 
 /* variables section */
 
-$dbaddr='mysql://root@localhost/test/?dbmode=single';
+$dbaddr='mysql://root@localhost/test/?dbmode=blob';
 
 $localcharset='iso-8859-1';
 $browsercharset='iso-8859-1';
@@ -133,8 +133,8 @@ if (($MAX_PS > 0) && ($ps>$MAX_PS)) $ps=$MAX_PS;
 // print_bottom()
 // -----------------------------------------------
 function print_bottom(){
-	print ("<HR><center><img src=\"http://mnogosearch.org/img/mnogo.gif\">\n");
-	print ("<font size=\"-1\">Powered by <a href=\"http://mnogosearch.org/\">mnoGoSearch</a></font><br>\n");
+	print ("<HR><center><img src=\"http://www.mnogosearch.org/img/mnogo.gif\">\n");
+	print ("<font size=\"-1\">Powered by <a href=\"http://www.mnogosearch.org/\">mnoGoSearch</a></font><br>\n");
 	print ("<p></BODY></HTML>\n");
 }
 
