@@ -55,6 +55,8 @@ static int EnvLoad(UDM_CFG *Cfg,const char *cname);
 int UdmSearchMode(const char * mode)
 {
   if(!mode)return(UDM_MODE_ALL);
+  if(!strcmp(mode,"all-minus"))return(UDM_MODE_ALL_MINUS);
+  if(!strcmp(mode,"all-minus-half"))return(UDM_MODE_ALL_MINUS_HALF);
   if(!strcmp(mode,"all"))return(UDM_MODE_ALL);
   if(!strcmp(mode,"any"))return(UDM_MODE_ANY);
   if(!strcmp(mode,"bool"))return(UDM_MODE_BOOL);
@@ -150,9 +152,16 @@ void UdmWeightFactorsInit(char *res, const char *wf, size_t num)
   if((len>0)&&(len<256))
   {
     const char *sec;
-      
-    for(sec=wf+len-1; sec >= wf; sec--)
-      res[len-(sec-wf)]= UdmHex2Int(*sec);
+    int secno;
+    
+    for(sec= wf + len - 1, secno= 1; sec >= wf; sec--)
+    {
+      if (*sec != '-' && *sec != '.')
+      {
+        res[secno]= UdmHex2Int(*sec);
+        secno++;
+      }
+    }
   }
   
   for (sn= num + 1 ; sn < 256; sn++)
@@ -1425,6 +1434,7 @@ static UDM_CONFCMD commands[] =
   {"Locale",               1,1,   env_rpl_var},     /* Documented */
   {"WordDistanceWeight",   1,1,   env_rpl_num_var}, /* Documented */
   {"MaxCoordFactor",       1,1,   env_rpl_num_var},
+  {"IDFFactor",            1,1,   env_rpl_num_var}, /* Documented */
   {"MinCoordFactor",       1,1,   env_rpl_num_var}, /* Documented */
   {"NumWordFactor",        1,1,   env_rpl_var},     /* Documented */
   {"NumDistinctWordFactor",1,1,   env_rpl_num_var}, /* Documented */
@@ -1452,9 +1462,11 @@ static UDM_CONFCMD commands[] =
   {"UseNumericOperators",  1,1,   env_rpl_bool_var},/* Documented */
   {"SaveSectionSize",      1,1,   env_rpl_bool_var},/* Documented */
   {"Dehyphenate",          1,1,   env_rpl_bool_var},/* Documented */
+  {"HyphenateNumbers",     1,1,   env_rpl_bool_var},/* TODO       */
   {"StripAccents",         1,1,   env_rpl_bool_var},/* Documented */
   {"LoadURLInfo",          1,1,   env_rpl_bool_var},/* Documented */
   {"LoadTagInfo",          1,1,   env_rpl_bool_var},/* Documented */
+  {"ComplexSynonyms",      1,1,   env_rpl_bool_var},
 
   {"ReplaceVar",           2,2,   env_rpl_named_var},/* Documented */
 
