@@ -30,7 +30,7 @@
 
 
 size_t
-UdmInflate(char *dst, size_t dstlen, char *src, size_t srclen)
+UdmInflate(char *dst, size_t dstlen, const char *src, size_t srclen)
 {
   z_stream z;
   z.zalloc= Z_NULL;
@@ -51,7 +51,7 @@ UdmInflate(char *dst, size_t dstlen, char *src, size_t srclen)
 
 
 size_t
-UdmDeflate(char *dst, size_t dstlen, char *src, size_t srclen)
+UdmDeflate(char *dst, size_t dstlen, const char *src, size_t srclen)
 {
   z_stream z;
   z.zalloc= Z_NULL;
