@@ -55,10 +55,6 @@
 #include "udm_signals.h"
 #include "udm_utils.h"
 
-#ifdef WIN32
-#include <assert.h>
-#endif
-
 long tz_offset=0;	/* EXT */
 
 /* Function to convert date returned by web-server to time_t
@@ -425,6 +421,15 @@ time_t UdmHttpDate2Time_t(const char *date){
     scan_hh_mm_ss(&ds, date + 11);
     goto check_valid_time;
   }
+  else if (ap_checkmask(date, "##########") ||
+           ap_checkmask(date, "#########"))
+  {
+    /*
+      Unix timestamp format: 9 or 10 digits.
+    */
+    return atoi(date);
+  }
+
 
   if ((date = strchr(date, ' ')) == NULL)	/* Find space after weekday */
     return BAD_DATE;
