@@ -50,7 +50,6 @@
 #ifdef HAVE_NETDB_H
 #include <netdb.h>
 #endif
-#include <assert.h>
 
 
 #include "udm_common.h"
@@ -439,6 +438,14 @@ static int UdmDocUpdate(UDM_AGENT *Indexer, UDM_DOCUMENT *Doc)
   return rc;
 }
 
+
+size_t
+UdmDBNumBySeed(UDM_ENV *Env, udmhash32_t seed)
+{
+  return seed * Env->dbl.nitems / 256;
+}
+
+
 int UdmURLActionNoLock(UDM_AGENT *A, UDM_DOCUMENT *D, int cmd)
 {
   int res=UDM_ERROR, execflag = 0;
@@ -461,7 +468,7 @@ int UdmURLActionNoLock(UDM_AGENT *A, UDM_DOCUMENT *D, int cmd)
     udmhash32_t url_id=UdmVarListFindInt(&D->Sections,"URL_ID", 0);
     udmhash32_t seed= (((url_id) ? url_id :
                        UdmStrHash32(UdmVarListFindStr(&D->Sections, "URL", ""))) & 0xFF);
-    dbfrom= dbto= seed * A->Conf->dbl.nitems / 256;
+    dbfrom= dbto= UdmDBNumBySeed(A->Conf, seed);
     dbto++;
   }
 
@@ -493,7 +500,7 @@ int UdmURLActionNoLock(UDM_AGENT *A, UDM_DOCUMENT *D, int cmd)
     
     if (res != UDM_OK && execflag)
     {
-      UdmLog (A, UDM_LOG_ERROR, db->errstr);
+      UdmLog (A, UDM_LOG_ERROR, "%s", db->errstr);
     }
     UDM_RELEASELOCK(A, UDM_LOCK_DB);
     if (res != UDM_OK) break;
@@ -542,7 +549,7 @@ __C_LINK int __UDMCALL UdmTargets(UDM_AGENT *A)
 #endif
     if(res != UDM_OK)
     {
-      UdmLog(A, UDM_LOG_ERROR, db->errstr);
+      UdmLog(A, UDM_LOG_ERROR, "%s", db->errstr);
     }
     UDM_RELEASELOCK(A, UDM_LOCK_DB);
     if (res != UDM_OK) break;
@@ -575,7 +582,7 @@ __C_LINK int __UDMCALL UdmResAction(UDM_AGENT *A, UDM_RESULT *R, int cmd)
     }
     if(rc != UDM_OK)
     {
-      UdmLog(A, UDM_LOG_ERROR, db->errstr);
+      UdmLog(A, UDM_LOG_ERROR, "%s", db->errstr);
     }
     UDM_RELEASELOCK(A, UDM_LOCK_DB);
     if (rc != UDM_OK) break;
@@ -610,7 +617,7 @@ __C_LINK int __UDMCALL UdmCatAction(UDM_AGENT *A, UDM_CATEGORY *C, int cmd)
     }
     if(res != UDM_OK)
     {
-      UdmLog(A, UDM_LOG_ERROR, db->errstr);
+      UdmLog(A, UDM_LOG_ERROR, "%s", db->errstr);
     }
     UDM_RELEASELOCK(A, UDM_LOCK_DB);
     if (res != UDM_OK) break;
@@ -636,7 +643,7 @@ __C_LINK int __UDMCALL UdmSrvAction(UDM_AGENT *A, UDM_SERVERLIST *S, int cmd)
     res = UdmSrvActionSQL(A, S, cmd, db);
 #endif
     if(res != UDM_OK){
-    UdmLog(A, UDM_LOG_ERROR, db->errstr);
+    UdmLog(A, UDM_LOG_ERROR, "%s", db->errstr);
     }
     UDM_RELEASELOCK(A, UDM_LOCK_DB);
     if (res != UDM_OK) break;
@@ -740,7 +747,7 @@ UdmResultJoin(UDM_AGENT *A,
               UDM_RESULT *Res)
 {
   size_t i, max_total_found= 0;
-  unsigned long ticks;
+  udm_timer_t ticks;
   
   for (Res->URLData.nitems= 0,
        Res->num_rows= 0,
@@ -780,7 +787,7 @@ UdmResultJoin(UDM_AGENT *A,
     }
   }
 
-  if (nresults > 1)
+  if (nresults > 1 && Res->URLData.nitems > 0)
   {
     int use_site_id= UdmVarListFindBool(&A->Conf->Vars, "GroupBySite", 0) &&
                      (UdmVarListFindInt(&A->Conf->Vars, "site", 0) == 0);
@@ -802,8 +809,7 @@ UdmResultJoin(UDM_AGENT *A,
                               UdmVarListFindStr(&A->Conf->Vars, "s", "R"));
   }
 
-  ticks= UdmStartTimer() - ticks;
-  UdmLog(A, UDM_LOG_DEBUG, "Stop  joining results:\t%.2f", (float)ticks / 1000);
+  UdmLog(A, UDM_LOG_DEBUG, "Stop  joining results:\t%.2f", UdmStopTimer(&ticks));
 
   return UDM_OK;
 }
@@ -865,8 +871,7 @@ UdmFindWordsMulDB(UDM_AGENT *A, UDM_RESULT *TmpRes,
       UdmLog(A, UDM_LOG_DEBUG, "Joining thread[%d]", i + 1);
       A->Conf->ThreadJoin(thd);
     }
-    ticks= UdmStartTimer() - ticks;
-    UdmLog(A, UDM_LOG_DEBUG, "Threads finished: %.2f", (float) ticks/1000);
+    UdmLog(A, UDM_LOG_DEBUG, "Threads finished: %.2f", UdmStopTimer(&ticks));
   }
   
   if (use_threads)
@@ -887,7 +892,6 @@ UdmFindWords(UDM_AGENT *A, UDM_RESULT *TmpRes,
 {
   const char  *cache_mode= UdmVarListFindStr(&A->Conf->Vars, "Cache", "no");
   int res=    UDM_OK;
-  size_t      ResultsLimit = UdmVarListFindUnsigned(&A->Conf->Vars, "ResultsLimit", 0);
 
   if( strcasecmp(cache_mode, "yes") || UdmSearchCacheFind(A, Res))
   {
@@ -904,11 +908,6 @@ UdmFindWords(UDM_AGENT *A, UDM_RESULT *TmpRes,
       UdmResultSetMachineNumber(&Res->URLData, 0);
     }
 
-    if (ResultsLimit > 0 && ResultsLimit < Res->URLData.nitems)
-    {
-      Res->URLData.nitems= ResultsLimit;
-    }
-
     if((!strcasecmp(cache_mode,"yes"))&&(search_cache_size>-1))
     {
       fflush(stdout);
@@ -1042,7 +1041,7 @@ UDM_RESULT * __UDMCALL UdmFind(UDM_AGENT *A)
 {
   UDM_RESULT  *Res, *TmpRes;
   int    res=UDM_OK;
-  unsigned long  ticks=UdmStartTimer(), ticks_;
+  udm_timer_t ticks= UdmStartTimer(), ticks_;
   size_t  i, nbytes, numdatabases=  A->Conf->dbl.nitems;
   size_t  page_number= (size_t) UdmVarListFindInt(&A->Conf->Vars, "np", 0);
   size_t  page_size=   (size_t) UdmVarListFindInt(&A->Conf->Vars, "ps", 10);
@@ -1050,9 +1049,26 @@ UDM_RESULT * __UDMCALL UdmFind(UDM_AGENT *A)
   size_t  ExcerptSize= (size_t)UdmVarListFindInt(&A->Conf->Vars, "ExcerptSize", 256);
   size_t  ExcerptPadding = (size_t)UdmVarListFindInt(&A->Conf->Vars, "ExcerptPadding", 40);
   int     UseLocalCachedCopy= UdmVarListFindBool(&A->Conf->Vars, "UseLocalCachedCopy", 0);
-  size_t  max_offs= 1000; /* Can be a search.htm command, make sure it is not overwritten by query string */
+  size_t  ResultsLimit= 1000;
+  UDM_VAR *ResultsLimitVar= UdmVarListFind(&A->Conf->Vars, "ResultsLimit");
   char    str[128];
 
+  new_version= UdmVarListFindBool(&A->Conf->Vars, "NewVersion",
+               UdmVarListFindBool(&A->Conf->Vars, "ENV.NEWVERSION", 0));
+
+  /*
+    Make sure we use only search.htm command,
+    do not allow to overwrite in query string.
+    FIXME: Using template command and query string parameter
+    at the same time makes limit equal to 1000.
+    This should be fixed to use the template value instead.
+    This needed when the template value is smaller than 1000.
+  */
+  if (ResultsLimitVar && (ResultsLimitVar->section == 0))
+  {
+    ResultsLimit= UdmVarIntVal(ResultsLimitVar);
+  }
+
   UdmLog(A,UDM_LOG_DEBUG,"Start UdmFind");
   
   ticks_= UdmStartTimer();
@@ -1065,13 +1081,13 @@ UDM_RESULT * __UDMCALL UdmFind(UDM_AGENT *A)
       UDM_OK != UdmAddCategoryInfo(A))
     goto conv;
   UdmVarListAddStr(&A->Conf->Vars, "orig_m", UdmVarListFindStr(&A->Conf->Vars, "m", "all"));
-  ticks_= UdmStartTimer() - ticks_;
-  UdmLog(A, UDM_LOG_DEBUG, "Stop  Prepare:\t\t%.2f", (float)ticks_/1000);
+  UdmLog(A, UDM_LOG_DEBUG, "Stop  Prepare:\t\t%.2f", UdmStopTimer(&ticks_));
 
   Res->first = offs ? offs : page_number * page_size;  
-  if (Res->first + page_size > max_offs)
+  if (ResultsLimit && (Res->first + page_size > ResultsLimit))
   {
-    UdmLog(A, UDM_LOG_DEBUG, "Too large ps or np: offset=%d limit=%d\n", Res->first + page_size, max_offs);
+    UdmLog(A, UDM_LOG_DEBUG, "Too large ps or np: offset=%d limit=%d\n",
+           Res->first + page_size, ResultsLimit);
     goto conv;
   }
 
@@ -1080,8 +1096,7 @@ UDM_RESULT * __UDMCALL UdmFind(UDM_AGENT *A)
   if (!Res->WWList.nwords ||
       UDM_OK != (res= UdmFindWords(A, TmpRes, Res, Res->first + page_size)))
     goto conv;
-  ticks_= UdmStartTimer() - ticks_;
-  UdmLog(A, UDM_LOG_DEBUG, "Stop  UdmFindWords:\t\t%.2f", (float)ticks_/1000);
+  UdmLog(A, UDM_LOG_DEBUG, "Stop  UdmFindWords:\t\t%.2f", UdmStopTimer(&ticks_));
   
   if (!Res->total_found)
   {
@@ -1154,8 +1169,7 @@ UDM_RESULT * __UDMCALL UdmFind(UDM_AGENT *A)
   for (i= 0; i < Res->num_rows; i++)
     CreateAlias(A->Conf, &Res->Doc[i]);
     
-  ticks_= UdmStartTimer() - ticks_;
-  UdmLog(A, UDM_LOG_DEBUG, "Stop  AddDocInfo:\t\t%.2f", (float)ticks_/1000);
+  UdmLog(A, UDM_LOG_DEBUG, "Stop  AddDocInfo:\t\t%.2f", UdmStopTimer(&ticks_));
   
   if (UdmVarListFindInt(&A->Conf->Vars, "DetectClones", 0))
   {
@@ -1180,8 +1194,7 @@ UDM_RESULT * __UDMCALL UdmFind(UDM_AGENT *A)
         UdmResultFree(Cl);
       }
     }
-    ticks_ = UdmStartTimer() - ticks_;
-    UdmLog(A, UDM_LOG_DEBUG, "Stop  Clones:\t\t%.2f", (float)ticks_/1000);
+    UdmLog(A, UDM_LOG_DEBUG, "Stop  Clones:\t\t%.2f", UdmStopTimer(&ticks_));
   }
   
   /* first and last begins from 0, make it begin from 1 */
@@ -1194,8 +1207,7 @@ conv:
   ticks_=UdmStartTimer();
   UdmLog(A, UDM_LOG_DEBUG, "Start UdmConvert");
   UdmConvert(A->Conf, Res, A->Conf->lcs, A->Conf->bcs);
-  ticks_ = UdmStartTimer() - ticks_;
-  UdmLog(A, UDM_LOG_DEBUG, "Stop  UdmConvert:\t\t%.2f", (float)ticks_/1000);
+  UdmLog(A, UDM_LOG_DEBUG, "Stop  UdmConvert:\t\t%.2f", UdmStopTimer(&ticks_));
 #endif
 
   ticks_=UdmStartTimer();
@@ -1209,14 +1221,13 @@ conv:
       UdmFree(Excerpt);
     }
   }
-  ticks_ = UdmStartTimer() - ticks_;
-  UdmLog(A, UDM_LOG_DEBUG, "Stop  Excerpts:\t\t%.2f", (float)ticks_/1000);
+  UdmLog(A, UDM_LOG_DEBUG, "Stop  Excerpts:\t\t%.2f", UdmStopTimer(&ticks_));
 
   UdmLog(A, UDM_LOG_DEBUG, "Start WordInfo");
   ticks_=UdmStartTimer();
   UdmResWordInfo(A->Conf, Res);
   ticks_= UdmStartTimer() - ticks_;
-  UdmLog(A, UDM_LOG_DEBUG, "Stop  WordInfo:\t\t%.2f", (float)ticks_/1000);
+  UdmLog(A, UDM_LOG_DEBUG, "Stop  WordInfo:\t\t%.2f", UdmStopTimer(&ticks_));
 
   Res->work_time= ticks= UdmStartTimer() - ticks;
 
@@ -1318,14 +1329,16 @@ static UDM_SQLDB_DRIVER SQLDriver[]=
     "oracle8", UDM_DB_ORACLE8, UDM_DB_ORACLE8, 1, 
     UDM_SQL_HAVE_GROUPBY   | UDM_SQL_HAVE_TRUNCATE |
     UDM_SQL_HAVE_SUBSELECT | UDM_SQL_HAVE_BIND |
-    UDM_SQL_HAVE_ROWNUM    | UDM_SQL_HAVE_GOOD_COMMIT | UDM_SQL_HAVE_TRANSACT,
+    UDM_SQL_HAVE_ROWNUM    | UDM_SQL_HAVE_GOOD_COMMIT | UDM_SQL_HAVE_TRANSACT |
+    UDM_SQL_HAVE_RENAME    | UDM_SQL_HAVE_CREATE_LIKE,
     &udm_sqldb_oracle_handler
   },
   {
     "oracle", UDM_DB_ORACLE8, UDM_DB_ORACLE8, 1, 
     UDM_SQL_HAVE_GROUPBY   | UDM_SQL_HAVE_TRUNCATE |
     UDM_SQL_HAVE_SUBSELECT | UDM_SQL_HAVE_BIND |
-    UDM_SQL_HAVE_ROWNUM    | UDM_SQL_HAVE_GOOD_COMMIT | UDM_SQL_HAVE_TRANSACT,
+    UDM_SQL_HAVE_ROWNUM    | UDM_SQL_HAVE_GOOD_COMMIT | UDM_SQL_HAVE_TRANSACT |
+    UDM_SQL_HAVE_RENAME    | UDM_SQL_HAVE_CREATE_LIKE,
     &udm_sqldb_oracle_handler
   },
 #endif
@@ -1338,9 +1351,21 @@ static UDM_SQLDB_DRIVER SQLDriver[]=
   },
   {
     "sybase", UDM_DB_SYBASE, UDM_DB_MSSQL, 1,
-    UDM_SQL_HAVE_GROUPBY | UDM_SQL_HAVE_TRUNCATE |
+    UDM_SQL_HAVE_GROUPBY | /*UDM_SQL_HAVE_TRUNCATE |*/
+    /*
+      Don't use TRUNCATE with Sybase.
+      It gives error:
+      'TRUNCATE TABLE command not allowed within multi-statement
+      transaction.
+      TODO: modify the code to use TRUNCATE outside a transaction
+    */
     UDM_SQL_HAVE_TOP     | UDM_SQL_HAVE_0xHEX    |
-    UDM_SQL_HAVE_GOOD_COMMIT | UDM_SQL_HAVE_TRANSACT,
+    UDM_SQL_HAVE_GOOD_COMMIT | UDM_SQL_HAVE_TRANSACT
+    /*
+    Something goes wrong with sp_rename!
+    UDM_SQL_HAVE_RENAME  | UDM_SQL_HAVE_CREATE_LIKE
+    */
+     ,
     &udm_sqldb_ctlib_handler
   },
 #endif
@@ -1348,7 +1373,9 @@ static UDM_SQLDB_DRIVER SQLDriver[]=
   { 
     "mysql", UDM_DB_MYSQL, UDM_DB_MYSQL, 1,
     UDM_SQL_HAVE_BIND  |
-    UDM_SQL_HAVE_LIMIT | UDM_SQL_HAVE_GROUPBY | UDM_SQL_HAVE_0xHEX,
+    UDM_SQL_HAVE_LIMIT | UDM_SQL_HAVE_GROUPBY | UDM_SQL_HAVE_0xHEX |
+    UDM_SQL_HAVE_RENAME | UDM_SQL_HAVE_CREATE_LIKE |
+    UDM_SQL_HAVE_DROP_IF_EXISTS,
     &udm_sqldb_mysql_handler
   },
 #endif
@@ -1357,7 +1384,9 @@ static UDM_SQLDB_DRIVER SQLDriver[]=
     "pgsql", UDM_DB_PGSQL, UDM_DB_PGSQL, 1,
     UDM_SQL_HAVE_BIND  |
     UDM_SQL_HAVE_LIMIT | UDM_SQL_HAVE_GROUPBY |
-    UDM_SQL_HAVE_SUBSELECT | UDM_SQL_HAVE_TRANSACT,
+    UDM_SQL_HAVE_SUBSELECT | UDM_SQL_HAVE_TRANSACT |
+    UDM_SQL_HAVE_RENAME | UDM_SQL_HAVE_CREATE_LIKE,
+    /* UDM_SQL_HAVE_DROP_IF_EXISTS depends on server version */
     &udm_sqldb_pgsql_handler,
   },
 #endif
@@ -1390,7 +1419,9 @@ static UDM_SQLDB_DRIVER SQLDriver[]=
     UDM_DB_SQLITE3, UDM_DB_SQLITE3, 1,
     UDM_SQL_HAVE_BIND   |
     UDM_SQL_HAVE_LIMIT  | UDM_SQL_HAVE_GROUPBY |
-    UDM_SQL_HAVE_STDHEX | UDM_SQL_HAVE_GOOD_COMMIT | UDM_SQL_HAVE_TRANSACT,
+    UDM_SQL_HAVE_STDHEX | UDM_SQL_HAVE_GOOD_COMMIT | UDM_SQL_HAVE_TRANSACT |
+    UDM_SQL_HAVE_DROP_IF_EXISTS | UDM_SQL_HAVE_RENAME |
+    UDM_SQL_HAVE_CREATE_LIKE,
     &udm_sqldb_sqlite3_handler,
   },
 #endif
@@ -1408,7 +1439,8 @@ static UDM_SQLDB_DRIVER SQLDriver[]=
   {
     "odbc-db2", UDM_DB_DB2, UDM_DB_ODBC, 1,
     UDM_SQL_HAVE_GROUPBY | UDM_SQL_HAVE_BIND |
-    UDM_SQL_HAVE_BIND    | UDM_SQL_HAVE_STDHEX | UDM_SQL_HAVE_TRANSACT,
+    UDM_SQL_HAVE_BIND    | UDM_SQL_HAVE_STDHEX | UDM_SQL_HAVE_TRANSACT |
+    UDM_SQL_HAVE_RENAME | UDM_SQL_HAVE_CREATE_LIKE,    
     &udm_sqldb_odbc_handler,
   },
   {
@@ -1440,14 +1472,16 @@ static UDM_SQLDB_DRIVER SQLDriver[]=
     UDM_SQL_HAVE_GROUPBY   | UDM_SQL_HAVE_TRUNCATE |
     UDM_SQL_HAVE_SUBSELECT | UDM_SQL_HAVE_BIND |
     UDM_SQL_HAVE_ROWNUM    | UDM_SQL_HAVE_GOOD_COMMIT |
-    UDM_SQL_HAVE_TRANSACT,
+    UDM_SQL_HAVE_TRANSACT  |
+    UDM_SQL_HAVE_RENAME    | UDM_SQL_HAVE_CREATE_LIKE,
     &udm_sqldb_odbc_handler,
   },
   {
     "odbc-oracle8", UDM_DB_ORACLE8, UDM_DB_ODBC, 1,
     UDM_SQL_HAVE_GROUPBY   | UDM_SQL_HAVE_TRUNCATE |
     UDM_SQL_HAVE_SUBSELECT | UDM_SQL_HAVE_BIND | UDM_SQL_HAVE_ROWNUM |
-    UDM_SQL_HAVE_TRANSACT,
+    UDM_SQL_HAVE_TRANSACT  |
+    UDM_SQL_HAVE_RENAME    | UDM_SQL_HAVE_CREATE_LIKE,
     &udm_sqldb_odbc_handler,
   },
   {
@@ -1459,15 +1493,17 @@ static UDM_SQLDB_DRIVER SQLDriver[]=
   },
   {
     "odbc-sybase", UDM_DB_SYBASE, UDM_DB_ODBC, 1,
-    UDM_SQL_HAVE_GROUPBY | UDM_SQL_HAVE_TRUNCATE |
+    UDM_SQL_HAVE_GROUPBY | /*UDM_SQL_HAVE_TRUNCATE |*/
     UDM_SQL_HAVE_TOP     | UDM_SQL_HAVE_0xHEX |
-    UDM_SQL_HAVE_TRANSACT,
+    UDM_SQL_HAVE_TRANSACT| UDM_SQL_HAVE_GOOD_COMMIT,
     &udm_sqldb_odbc_handler,
   },
   {
     "odbc-mysql", UDM_DB_MYSQL, UDM_DB_ODBC, 1,
     UDM_SQL_HAVE_BIND  |
-    UDM_SQL_HAVE_LIMIT | UDM_SQL_HAVE_GROUPBY | UDM_SQL_HAVE_0xHEX,
+    UDM_SQL_HAVE_LIMIT | UDM_SQL_HAVE_GROUPBY | UDM_SQL_HAVE_0xHEX |
+    UDM_SQL_HAVE_RENAME | UDM_SQL_HAVE_CREATE_LIKE |
+    UDM_SQL_HAVE_DROP_IF_EXISTS,
     &udm_sqldb_odbc_handler,
   },
   {
@@ -1512,6 +1548,26 @@ static UDM_SQLDB_DRIVER *UdmSQLDriverByName(const char *name)
 }
 
 
+static UDM_DBMODE_HANDLER *
+UdmDBModeHandlerByID(int DBMode)
+{
+#ifdef HAVE_SQL
+  switch (DBMode)
+  {
+    case UDM_DBMODE_BLOB:
+      return &udm_dbmode_handler_blob;
+    case UDM_DBMODE_SINGLE:
+      return &udm_dbmode_handler_single;
+    case UDM_DBMODE_MULTI:
+      return &udm_dbmode_handler_multi;
+    case UDM_DBMODE_RAWBLOB:
+      return &udm_dbmode_handler_rawblob;
+  }
+  UDM_ASSERT(0);
+#endif
+  return NULL;
+}
+
 int UdmDBSetAddr(UDM_DB *db, const char *dbaddr, int mode)
 {
   UDM_URL    addr;
@@ -1610,6 +1666,8 @@ int UdmDBSetAddr(UDM_DB *db, const char *dbaddr, int mode)
     return UDM_ERROR;
   }
   
+  db->dbmode_handler= UdmDBModeHandlerByID(db->DBMode);
+  
   if((v= UdmVarListFindStr(&db->Vars,"dbmodesearch",NULL)))
   {
     int DBMode;
@@ -1662,11 +1720,13 @@ int UdmDBSetAddr(UDM_DB *db, const char *dbaddr, int mode)
   
   if (UdmVarListFindInt(&db->Vars, "ps", 0) == 123)
   {
+#ifdef HAVE_SQL
     db->sql->SQLPrepare= UdmSQLPrepareGeneric;
     db->sql->SQLBind= UdmSQLBindGeneric; 
     db->sql->SQLExec= UdmSQLExecGeneric;
     db->sql->SQLStmtFree= UdmSQLStmtFreeGeneric;
     db->flags|= UDM_SQL_HAVE_BIND;
+#endif
   }
   else if ((db->DBType == UDM_DB_MSSQL   ||
             db->DBType == UDM_DB_SYBASE  ||
@@ -1717,7 +1777,7 @@ __C_LINK int __UDMCALL UdmStatAction(UDM_AGENT *A, UDM_STATLIST *S)
 #endif
     if (res != UDM_OK)
     {
-      strcpy(A->Conf->errstr,db->errstr);
+      strcpy(A->Conf->errstr, db->errstr);
       db->errcode= 0;
       break;
     }
@@ -1866,7 +1926,7 @@ int UdmMulti2Blob (UDM_AGENT *Indexer)
 {
 #ifdef HAVE_SQL
   size_t i;
-  unsigned long ticks;
+  udm_timer_t ticks;
 
   UdmLog(Indexer,UDM_LOG_ERROR,"Converting to blob");
   ticks=UdmStartTimer();
@@ -1887,8 +1947,7 @@ int UdmMulti2Blob (UDM_AGENT *Indexer)
     }
   }
 
-  ticks=UdmStartTimer()-ticks;
-  UdmLog(Indexer,UDM_LOG_ERROR,"Converting to blob finished\t%.2f",(float)ticks/1000);
+  UdmLog(Indexer,UDM_LOG_ERROR,"Converting to blob finished\t%.2f", UdmStopTimer(&ticks));
 #endif
   return UDM_OK;
 }
@@ -1899,7 +1958,7 @@ int UdmExport (UDM_AGENT *Indexer)
   int rc= UDM_OK;
 #ifdef HAVE_SQL
   size_t i;
-  unsigned long ticks;
+  udm_timer_t ticks;
 
   UdmLog(Indexer,UDM_LOG_ERROR,"Starting export");
   ticks=UdmStartTimer();
@@ -1919,8 +1978,9 @@ int UdmExport (UDM_AGENT *Indexer)
     }
   }
   
-  ticks=UdmStartTimer()-ticks;
-  UdmLog(Indexer,UDM_LOG_ERROR,"Export finished\t%.2f",(float)ticks/1000);
+  UdmLog(Indexer,UDM_LOG_ERROR,"Export finished\t%.2f",UdmStopTimer(&ticks));
 #endif
   return rc;
 }
+
+int new_version= 0;
