@@ -30,6 +30,8 @@
 #include "udm_word.h"
 #include "udm_searchtool.h"
 
+#ifdef HAVE_SQL
+
 /********************* MultiCache stuff *************************/
 UDM_MULTI_CACHE *UdmMultiCacheInit (UDM_MULTI_CACHE *cache)
 {
@@ -402,11 +404,12 @@ UdmMultiCachePutIntagUsingEncoding(UDM_MULTI_CACHE_WORD *cache,
   size_t nbytes;
   size_t last;
   size_t multiply= (type == 2) ? 3*5 : 3*2;
+  size_t alloced_length;
 
   if (! cache->nintags) return(NULL);
 
   /* Allocate maximum possible length. */
-  _ = UdmMalloc(cache->nintags * multiply + 3);
+  _ = UdmMalloc((alloced_length= cache->nintags * multiply + 3));
   if (! _) return(NULL);
 
   if (type == 1)
@@ -438,7 +441,7 @@ UdmMultiCachePutIntagUsingEncoding(UDM_MULTI_CACHE_WORD *cache,
 
     if (type == 2)
     {
-      len+= UdmSQLBinEscStr(db, _ + len, (char*) buf, nbytes);
+      len+= UdmSQLBinEscStr(db, _ + len, alloced_length, (char*) buf, nbytes);
       _[len]= '\0'; /* Check if really necessary - possibly  already done */
     }
     else
@@ -499,19 +502,6 @@ UdmMultiCachePutIntagUsingRaw(UDM_MULTI_CACHE_WORD *cache)
 #define UDM_THREADINFO(A,s,m)	if(A->Conf->ThreadInfo)A->Conf->ThreadInfo(A,s,m)
 
 
-/*
-  TODO:
-  reuse the same function from
-  urlidlist.c, sql.c, dbmode-blob.c, dbmode-multi
-*/
-static int cmpaurls (urlid_t *s1, urlid_t *s2)
-{
-  if (*s1 > *s2) return(1);
-  if (*s1 < *s2) return(-1);
-  return(0);
-}
-
-
 static int
 UdmWordCacheWriteUsingBind(UDM_DB *db, int seed, urlid_t url_id,
                            int secno, const char *word,
@@ -590,7 +580,7 @@ int UdmWordCacheWrite (UDM_AGENT *Indexer, UDM_DB *db, size_t limit)
   
   /* Sort IN list */
   if (cache->nurls)
-    UdmSort(cache->urls, cache->nurls, sizeof(urlid_t), (udm_qsort_cmp)cmpaurls);
+    UdmSort(cache->urls, cache->nurls, sizeof(urlid_t), (udm_qsort_cmp)UdmCmpURLID);
 
   /* Remove expired words */
   for (i= 0, in_num= 0; i < cache->nurls; i++)
@@ -630,7 +620,6 @@ int UdmWordCacheWrite (UDM_AGENT *Indexer, UDM_DB *db, size_t limit)
     char *word = cword->word;
     urlid_t url_id = cword->url_id;
     unsigned char secno = cword->secno;
-    char *intag;
 
     mintag.nintags = 0;
 
@@ -664,17 +653,18 @@ int UdmWordCacheWrite (UDM_AGENT *Indexer, UDM_DB *db, size_t limit)
 
     if (db->flags & UDM_SQL_HAVE_BIND)
     {
-      intag= UdmMultiCachePutIntagUsingRaw(&mintag);
+      char *intag= UdmMultiCachePutIntagUsingRaw(&mintag);
       if (!intag)
         continue;
 
       if (UDM_OK != (rc= UdmWordCacheWriteUsingBind(db, seed, url_id,
                            secno, word, intag, strlen(intag))))
         goto unlock_UdmStoreWordsMulti;
+      UdmFree(intag);
     }
     else if (db->DBType == UDM_DB_MYSQL)
     {
-      intag = UdmMultiCachePutIntagUsingEncoding(&mintag, db, 1);
+      char *intag= UdmMultiCachePutIntagUsingEncoding(&mintag, db, 1);
       if (! intag) continue;
 
       if (buf.size_data)
@@ -714,24 +704,22 @@ int UdmWordCacheWrite (UDM_AGENT *Indexer, UDM_DB *db, size_t limit)
     }
     else
     {
-      const char *quot;
-      const char *x;
-      const char *castb;
-      const char *caste;
-
+      const char *quot, *x, *castb, *caste;
+      char *intag;
+      
       if (db->DBType == UDM_DB_ORACLE8 || db->DBType == UDM_DB_DB2) 
-        intag = UdmMultiCachePutIntagUsingEncoding(&mintag, db, 0);
+        intag= UdmMultiCachePutIntagUsingEncoding(&mintag, db, 0);
       else if (db->DBType == UDM_DB_MSSQL ||
                db->DBType == UDM_DB_SYBASE ||
                db->DBType == UDM_DB_ACCESS)
-        intag = UdmMultiCachePutIntagUsingEncoding(&mintag, db, 1);
+        intag= UdmMultiCachePutIntagUsingEncoding(&mintag, db, 1);
       else if (db->DBType == UDM_DB_PGSQL)
         intag= UdmMultiCachePutIntagUsingEncoding(&mintag, db, 2);
       else
       {
         if ((intag= UdmMultiCachePutIntagUsingRaw(&mintag)))
         {
-          char *tmp= UdmSQLEscStr(db, NULL, intag, strlen(intag)); /* Data */
+          char *tmp= UdmSQLEscStrAlloc(db, intag, strlen(intag)); /* Data */
           UdmFree(intag);
           intag= tmp;
         }
@@ -785,14 +773,20 @@ unlock_UdmStoreWordsMulti:
 }
 
 
-int
-UdmStoreWordsMulti(UDM_AGENT *Indexer, UDM_DOCUMENT *Doc, UDM_DB *db)
+static int
+UdmStoreWordsMulti(UDM_AGENT *Indexer, UDM_DB *db, UDM_DOCUMENT *Doc)
 {
   int rc= UDM_OK;
   urlid_t  url_id= UdmVarListFindInt(&Doc->Sections, "ID", 0);
   unsigned char PrevStatus= UdmVarListFindInt(&Doc->Sections, "PrevStatus", 0) ? 1 : 0;
   int WordCacheSize= UdmVarListFindInt(&Indexer->Conf->Vars, "WordCacheSize", 0);
 
+  /*
+    Don't need to delete words here.
+    DBMode=multi deletes words from multiple
+    documents in a portion, when doing UdmWordCacheWrite()
+  */
+
   if (UdmVarListFindInt(&Indexer->Conf->Vars, "SaveSectionSize", 1))
   {
     if (UDM_OK!= (rc= UdmWordListSaveSectionSize(Doc)))
@@ -860,19 +854,20 @@ UdmMultiAddCoords(UDM_FINDWORD_ARGS *args, UDM_SQLRES *SQLRes)
       returns data unsorted.
     */
     UdmURLCRDListSortByURLThenSecnoThenPos(&CoordList);
-    UdmURLCRDListListAddWithSort2(args, &args->CoordListList, &CoordList);
+    UdmURLCRDListListAddWithSort2(args, &CoordList);
   }
   args->Word.count= CoordList.ncoords;
   return UDM_OK;
 }
 
 
-int UdmFindWordMulti (UDM_FINDWORD_ARGS *args)
+static int
+UdmFindWordMulti(UDM_FINDWORD_ARGS *args)
 {
   char qbuf[4096], secno[64]= "";
   UDM_SQLRES SQLRes;
   size_t tnum, tmin, tmax;
-  unsigned long ticks;
+  udm_timer_t ticks;
   int rc;
 
   if (args->Word.match != UDM_MATCH_FULL)
@@ -917,15 +912,14 @@ AND url.rec_id=dict.url_id AND %s%s",
     UdmLog(args->Agent, UDM_LOG_DEBUG, "Start UdmMultiAddCoords");
     ticks= UdmStartTimer();
     UdmMultiAddCoords(args, &SQLRes);
-    ticks= UdmStartTimer() - ticks;
-    UdmLog(args->Agent, UDM_LOG_DEBUG, "Stop UdmMultiAddCoords\t%.2f", (float)ticks / 1000);
+    UdmLog(args->Agent, UDM_LOG_DEBUG, "Stop UdmMultiAddCoords\t%.2f", UdmStopTimer(&ticks));
     UdmSQLFree(&SQLRes);
   }
   return UDM_OK;
 }
 
 
-int
+static int
 UdmDeleteWordsFromURLMulti(UDM_AGENT *Indexer, UDM_DB *db, urlid_t url_id)
 {
   size_t i;
@@ -942,7 +936,7 @@ UdmDeleteWordsFromURLMulti(UDM_AGENT *Indexer, UDM_DB *db, urlid_t url_id)
 }
 
 
-int
+static int
 UdmTruncateDictMulti(UDM_AGENT *Indexer,UDM_DB *db)
 {
   int rc= UDM_OK;
@@ -958,7 +952,7 @@ UdmTruncateDictMulti(UDM_AGENT *Indexer,UDM_DB *db)
 }
 
 
-int
+static int
 UdmMulti2BlobSQL(UDM_AGENT *Indexer, UDM_DB *db, UDM_URLDATALIST *URLList)
 {
   size_t t, i, use_deflate= 0;
@@ -989,7 +983,7 @@ UdmMulti2BlobSQL(UDM_AGENT *Indexer, UDM_DB *db, UDM_URLDATALIST *URLList)
 #endif
   
   
-  if (UDM_OK != (rc= UdmBlobGetWTable(db, &wtable)))
+  if (UDM_OK != (rc= UdmBlobGetWTable(Indexer, db, &wtable)))
     return rc;
 
   /* Delete old words from bdict */
@@ -1068,6 +1062,48 @@ UdmMulti2BlobSQL(UDM_AGENT *Indexer, UDM_DB *db, UDM_URLDATALIST *URLList)
     return rc;
 
   UdmLog(Indexer, UDM_LOG_ERROR, "Switching to new blob table.");
-  rc= UdmBlobSetTable(db);
+  rc= UdmBlobSetTable(Indexer, db);
   return rc;
 }
+
+
+static int
+UdmWordStatCreateMulti(UDM_AGENT *A, UDM_DB *db)
+{
+  int rc;
+  size_t i;
+  
+  for (i=0; i <= 0xFF; i++)
+  {
+    char qbuf[128];
+    UdmLog(A, UDM_LOG_EXTRA, "Processing table %02X", i);
+
+    sprintf(qbuf, "SELECT word, count(*) FROM dict%02X GROUP BY word", i);
+    if (UDM_OK != (rc= UdmWordStatQuery(A, db, qbuf)))
+      return rc;
+  }
+  return UDM_OK;
+}
+
+
+
+static int
+UdmDumpWordInfoOneDocMulti(UDM_AGENT *A, UDM_DB *db, UDM_DOCUMENT *Doc)
+{
+  return UDM_OK;
+}
+
+
+UDM_DBMODE_HANDLER udm_dbmode_handler_multi=
+{
+  "multi",
+  UdmMulti2BlobSQL,
+  UdmStoreWordsMulti,
+  UdmTruncateDictMulti,
+  UdmDeleteWordsFromURLMulti,
+  UdmFindWordMulti,
+  UdmDumpWordInfoOneDocMulti,
+  UdmWordStatCreateMulti,
+};
+
+#endif /* HAVE_SQL */
