@@ -150,8 +150,8 @@ UdmEnvPrepare(UDM_ENV *Env)
 #ifdef MECAB
    if (!Env->mecab)
    {
-     char   *mecab_argv[] = { "mecab", "-F%m\" \"", "-E\" \"", "-B\" \"", NULL };
-     if (!(Env->mecab= mecab_new (4, mecab_argv)))
+     const char *mecab_argv[] = { "mecab", "-F%m\" \"", "-E\" \"", "-B\" \"", NULL };
+     if (!(Env->mecab= mecab_new (4, (char**) mecab_argv)))
      {
        udm_snprintf(Env->errstr, sizeof(Env->errstr) - 1,
                     "mecab_new failed: %s", mecab_strerror(Env->mecab));
