@@ -24,7 +24,6 @@
 #include <errno.h>
 #include <ctype.h>
 #include <signal.h>
-#include <assert.h>
 
 #ifdef WIN32
 #include <time.h>
@@ -194,14 +193,16 @@ ret:
 __C_LINK int __UDMCALL
 UdmStoreHrefs(UDM_AGENT *Indexer)
 {
-  int rc;
+  int rc= UDM_ERROR;
   
   UDM_GETLOCK(Indexer, UDM_LOCK_CONF);
 
   if (UDM_OK != (rc= UdmHrefsCheckDocPerSite(Indexer)))
     goto ret; 
   
+#ifdef HAVE_SQL
   rc= UdmStoreHrefsSQL(Indexer);
+#endif
 
 ret:
 
