@@ -24,7 +24,6 @@
 #include <sys/types.h>
 #include <fcntl.h>
 #include <signal.h>
-#include <assert.h>
 
 #ifdef WIN32
 #include <process.h>
@@ -167,6 +166,20 @@ static int UdmDisplaySQLQuery(UDM_SQLMON_PARAM *prm, UDM_SQLRES *sqlres)
      return res;
 }
 
+
+static const char *execsql= NULL;
+static char *
+sqlexecgets(UDM_SQLMON_PARAM *prm, char *str, size_t len)
+{
+  if (!execsql)
+    return NULL;
+  udm_snprintf(str, len, "%s", execsql);
+  prm->flags|= UDM_SQLMON_DONT_NEED_SEMICOLON;
+  execsql= NULL;
+  return str;
+}
+
+
 static char* sqlmongets(UDM_SQLMON_PARAM *prm, char *str, size_t size)
 {
 #ifdef HAVE_READLINE
@@ -178,12 +191,14 @@ static char* sqlmongets(UDM_SQLMON_PARAM *prm, char *str, size_t size)
        return 0;
      
      if (*line) add_history(line);
-     strncpy(str, line, size);
+     /* We need "\n" at the end to make sqlmon work properly */
+     udm_snprintf(str, size, "%s\n", line);
   }
   else
 #endif
   {
-    prm->prompt(prm, UDM_SQLMON_MSG_PROMPT, "SQL>");
+    if (prm->loglevel >= UDM_LOG_INFO)
+      prm->prompt(prm, UDM_SQLMON_MSG_PROMPT, "SQL>");
     if (!fgets(str, size, prm->infile))
       return 0;
   }
@@ -241,6 +256,7 @@ static int CreateOrDrop(UDM_AGENT *A, enum udm_indcmd cmd)
     prm.gets= sqlmongets;
     prm.display= UdmDisplaySQLQuery;
     prm.prompt= sqlmonprompt;
+    prm.loglevel= UDM_LOG_DEBUG;
     UdmSQLMonitor(A, A->Conf,&prm);
     printf("%d queries sent, %d succeeded, %d failed\n",
       prm.nqueries, prm.ngood, prm.nbad);
@@ -249,6 +265,21 @@ static int CreateOrDrop(UDM_AGENT *A, enum udm_indcmd cmd)
   return UDM_OK;
 }
 
+
+static int
+UdmIndCreate(UDM_AGENT *Agent)
+{
+  return CreateOrDrop(Agent, UDM_IND_CREATE);
+}
+
+
+static int
+UdmIndDrop(UDM_AGENT *Agent)
+{
+  return CreateOrDrop(Agent, UDM_IND_DROP);
+}
+
+
 static int ShowStatistics(UDM_AGENT *Indexer)
 {
   int       res;
@@ -311,6 +342,9 @@ static int ShowStatistics(UDM_AGENT *Indexer)
   printf("   -----------------------------\n");
   printf("%10s %10d %10d\n","Total",Total.expired,Total.total);
   printf("\n");
+  if (res != UDM_OK)
+    UdmLog(Indexer, UDM_LOG_ERROR, "Error: '%s'", UdmEnvErrMsg(Indexer->Conf));
+
   UDM_FREE(Stats.Stat);
   return(res);
 }
@@ -325,7 +359,7 @@ __C_LINK static int __UDMCALL ShowReferers(UDM_AGENT * Indexer)
 {
   int res;
   printf("\n          URLs and referers \n\n");
-  res = UdmURLAction(Indexer, NULL, UDM_URL_ACTION_REFERERS);
+  res= UdmURLAction(Indexer, NULL, UDM_URL_ACTION_REFERERS);
   return(res);
 }
 
@@ -573,7 +607,7 @@ static int usage(int level)
   fprintf(file, "                  big number of URLs)\n");
   fprintf(file, "  -n n            index only n documents and exit\n");
   fprintf(file, "  -c n            index only n seconds and exit\n");
-  fprintf(file, "  -q              quick startup (do not add Server URLs)\n");
+  fprintf(file, "  -q              quick startup (do not add Server URLs);  -qq even quicker\n");
 #endif
 
   fprintf(file, "  -b              block starting more than one indexer instances\n");
@@ -596,6 +630,7 @@ static int usage(int level)
   fprintf(file, "  -L language     limit indexer to documents matching language\n");
   fprintf(file, "  -u pattern      limit indexer to documents with URLs matching pattern\n");
   fprintf(file, "                  (supports SQL LIKE wildcard '%%')\n");
+  fprintf(file, "  --seed=number   limit indexer to docunents with the given seed (0-255)\n");
   fprintf(file, "  -D n            work with the n-th database only (i.e. with the n-th DBAddr)\n");
   fprintf(file, "  -f filename     read URLs to be indexed/inserted/cleared from file (with -a\n");
   fprintf(file, "                  or -C option, supports SQL LIKE wildcard '%%'; has no effect\n");
@@ -630,6 +665,8 @@ static int usage(int level)
   fprintf(file, "  -Edrop          drop SQL table structure and exit\n");
   fprintf(file, "  -Eblob          create fast search index\n");
   fprintf(file, "  -Ewordstat      create statistics for misspelled word suggestions\n");
+  fprintf(file, "  -Esqlmon        run interactive SQL monitor\n");
+  fprintf(file, "  -Esqlmon --exec=stmt execute the given SQL statement\n");
 #endif
   
   fprintf(file, "  -F pattern      print compile configuration and exit, e.g. -F '*'\n");
@@ -677,6 +714,7 @@ struct udm_indcmd2str_st
 static struct udm_indcmd2str_st indcmd[]=
 {
   {"index",         UDM_IND_INDEX},
+  {"crawl",         UDM_IND_CRAWL},
   {"statistics",    UDM_IND_STAT},
   {"create",        UDM_IND_CREATE},
   {"drop",          UDM_IND_DROP},
@@ -692,14 +730,16 @@ static struct udm_indcmd2str_st indcmd[]=
   {"hashspell",     UDM_IND_HASHSPELL},
   {"dumpspell",     UDM_IND_DUMPSPELL},
   {"dumpconf",      UDM_IND_DUMPCONF},
-  {"",              UDM_IND_INDEX}
+  {"dumpdata",      UDM_IND_DUMPDATA},
+  {"restoredata",   UDM_IND_RESTOREDATA},
+  {"",              UDM_IND_CRAWL}
 };
 
 static enum udm_indcmd UdmIndCmd(const char *cmdname)
 {
   struct udm_indcmd2str_st *cmd;
   size_t matches;
-  enum udm_indcmd res= UDM_IND_INDEX;
+  enum udm_indcmd res= UDM_IND_CRAWL;
   size_t cmdlen= strlen(cmdname);
   
   for (matches= 0, cmd= indcmd; cmd->name[0]; cmd++)
@@ -720,8 +760,8 @@ static enum udm_indcmd UdmIndCmd(const char *cmdname)
 */
 static int UdmARGC;
 static char **UdmARGV;
-static enum udm_indcmd cmd = UDM_IND_INDEX;
-static int insert = 0, expire = 0, pop_rank = 0, mkind = 0, block = 0, help = 0;
+static enum udm_indcmd cmd = UDM_IND_CRAWL;
+static int insert = 0, expire = 0, pop_rank = 0, block = 0, help = 0;
 static char *url_filename=NULL;
 static int thd_errors= 0;
 
@@ -731,7 +771,7 @@ static void UdmParseCmdLine(void)
 
   /* 
     Available new options:
-    Capital letters:  B    GH JK   OP   T VXWXYZ
+    Capital letters:  B    GH JK M OP   T VXWXYZ
     Small   letters:           k             x z
     Digits:          123456789
   */
@@ -758,8 +798,18 @@ static void UdmParseCmdLine(void)
       case 'Q': cmd= UDM_IND_SQLMON;  add_servers=0;load_langmaps=0;load_spells=0;break;
       case 'E': cmd= UdmIndCmd(optarg);break;
       case 'R': pop_rank++; break;
-      case 'M': mkind=1;break;
-      case 'q': add_server_urls = 0; break;
+      case 'q': 
+        if (add_server_urls == 0) /* -q already given */
+        {
+          /*
+            "indexer -qq" is given, do even faster start-up.
+            Don't synchonize "Server" commands in indexer.conf
+            with the "server" table content.
+          */
+          add_servers|= UDM_FLAG_DONT_ADD_TO_DB;
+        }
+        add_server_urls= 0;
+        break;
       case 'l': log2stderr=0;break;
       case 'a': expire=1;break;
       case 'b': block++;break;
@@ -1288,7 +1338,7 @@ UdmThreadCreate(udm_thread_t *thd, void *(*start_routine)(void*), void *arg)
 #ifdef WIN32
   {
     res= _beginthreadex(NULL, 0, start_routine, arg, 0, NULL);
-    assert(res != -1);
+    UDM_ASSERT(res != -1);
   }
 #else
   {
@@ -1549,11 +1599,283 @@ UdmEnvSetDirs(UDM_ENV *Env)
 
 }
 
+
+static int
+UdmDumpDocument(UDM_AGENT *A, UDM_DOCUMENT *D)
+{
+  size_t i;
+  printf("  <item>\n");
+  /*
+  printf("    <link>%s</link>\n", UdmVarListFindStr(&D->Sections, "URL", ""));
+  printf("    <id>%s</id>\n", UdmVarListFindStr(&D->Sections, "ID", ""));
+  printf("    <content-length>%s</content-length>\n", UdmVarListFindStr(&D->Sections, "Content-Length", ""));
+  printf("    <status>%s</status>\n", UdmVarListFindStr(&D->Sections, "Status", ""));
+  printf("    <hops>%s</hops>\n", UdmVarListFindStr(&D->Sections, "Hops", ""));
+  printf("    <crc32>%s</crc32>\n", UdmVarListFindStr(&D->Sections, "crc32", ""));
+  printf("    <modified>%s</modified>\n", UdmVarListFindStr(&D->Sections, "Last-Modified", ""));
+  */
+  for (i= 0; i < D->Sections.nvars; i++)
+  {
+    UDM_VAR *S= &D->Sections.Var[i];
+    printf("    <%s>%s</%s>\n", S->name, S->val, S->name);
+  }
+  printf("  </item>\n");
+  return 0;
+}
+
+
+static int
+UdmDumpDocuments(UDM_AGENT *A)
+{
+  int rc;
+  char errmgs[256];
+  A->Conf->DumpDoc= UdmDumpDocument;
+  
+  /* printf("<rss><channel>\n"); */
+  if (UDM_OK != (rc= UdmURLAction(&Main, NULL, UDM_URL_ACTION_DUMPDATA)))
+  {
+    fprintf(stderr, "error: %s\n", errmgs);
+    return 1;
+  }
+  /* printf("</channel></rss>\n"); */
+  return 0;
+}
+
+
+#include <sys/stat.h>
+/*
+  TODO for dump/restore:
+  - Escape bad characters to entities on dump, unescape on restore.
+  - Huge file size
+  - Subsection control
+  - Database number limit
+  - Insert into database
+*/
+static int
+UdmRestoreDocuments(UDM_AGENT *A)
+{
+  int rc= UDM_OK;
+  UDM_DSTR dstr;
+  struct stat sb;
+  int fd;
+  ssize_t nbytes;
+  UDM_CHARSET *cs;
+  
+  if (!url_filename)
+  {
+    fprintf(stderr, "Required option -f ommitted\n");
+    return UDM_ERROR;
+  }
+  fprintf(stderr, "Restoring data from '%s'\n", url_filename);
+  
+  if (stat(url_filename, &sb))
+  {
+    fprintf(stderr, "Can't stat '%s'\n", url_filename);
+    return UDM_ERROR;
+  }
+  UdmDSTRInit(&dstr, 1024);
+  if (UDM_OK != (rc= UdmDSTRAlloc(&dstr, sb.st_size + 1)))
+  {
+    fprintf(stderr, "failed to allocate %lld bytes\n",
+            (long long) sb.st_size + 1);
+    goto ex;
+  }
+  
+  if((fd= open(url_filename, O_RDONLY|UDM_BINARY)) <= 0)
+  {
+    rc= UDM_ERROR;
+    fprintf(stderr, "Can't open '%s'", url_filename);
+    goto ex;
+  }
+
+  if ((nbytes= read(fd, dstr.data, sb.st_size)) != sb.st_size)
+  {
+    rc= UDM_ERROR;
+    fprintf(stderr, "Failed to read %lld bytes from '%s'\n",
+            (long long) sb.st_size, url_filename);
+    goto ex; 
+  }
+  dstr.size_data= nbytes + 1;
+  dstr.data[nbytes]= '\0';
+  
+  cs= UdmGetCharSet("latin1");
+  rc= UdmResultFromXML(A, NULL, dstr.data, dstr.size_data, cs);
+ex:
+  UdmDSTRFree(&dstr);
+  return rc;
+}
+
+
+static int
+UdmIndHashSpell(UDM_AGENT *Agent)
+{
+  UDM_ENV *Env= Agent->Conf;
+  char errmsg[256];
+  int rc;
+  if (UDM_OK != (rc= UdmSpellListListLoad(&Env->Spells, errmsg, sizeof(errmsg)))||
+      UDM_OK != (rc= UdmSpellListListWriteHash(&Env->Spells, errmsg, sizeof(errmsg))))
+  {
+    fprintf(stderr, "error: %s\n", errmsg);
+  }
+  return rc;
+}
+
+
+static int
+UdmIndDumpSpell(UDM_AGENT *Agent)
+{
+  char errmsg[256];
+  int spflags= UDM_SPELL_NOPREFIX;
+  UDM_ENV *Env= Agent->Conf;
+  int rc;
+  if (UDM_OK != (rc= UdmSpellListListLoad(&Env->Spells,
+                                          errmsg, sizeof(errmsg)))||
+      UDM_OK != (rc= UdmAffixListListLoad(&Env->Affixes, spflags,
+                                          errmsg, sizeof(errmsg)))||
+      UDM_OK != (rc= UdmSpellDump(&Env->Spells, &Env->Affixes,
+                                          errmsg, sizeof(errmsg))))
+  {
+    fprintf(stderr, "error: %s\n", errmsg);
+  }
+  return rc;
+}
+
+
+static int
+UdmIndSQLMonitor(UDM_AGENT *Agent)
+{
+  int rc;
+  UDM_SQLMON_PARAM prm;
+  bzero((void*)&prm,sizeof(prm));
+  prm.infile= stdin;
+  prm.outfile= stdout;
+  prm.flags= UDM_SQLMON_DISPLAY_FIELDS;
+  execsql= UdmVarListFindStr(&Conf.Vars, "exec", NULL);
+  prm.gets= execsql ? sqlexecgets : sqlmongets;
+  prm.display= UdmDisplaySQLQuery;
+  prm.prompt= sqlmonprompt;
+  prm.loglevel= loglevel;
+#ifdef HAVE_READLINE
+  if (isatty(0))
+    prm.mode= udm_sqlmon_mode_interactive;
+#endif
+  rc= UdmSQLMonitor(&Main, &Conf, &prm);
+  return rc;
+}
+
+
+static int
+UdmIndWordStat(UDM_AGENT *Agent)
+{
+  int rc= UdmURLAction(Agent, NULL, UDM_URL_ACTION_WRDSTAT);
+  return rc;
+}
+
+
+static int
+UdmIndBlock(void)
+{
+  int pid_fd;
+  char pidbuf[128];
+  /* Check that another instance isn't running and create PID file. */
+  const char *vardir= UdmVarListFindStr(&Conf.Vars,"VarDir",UDM_VAR_DIR);
+  sprintf(pidname,"%s/%s", vardir ,"indexer.pid");
+  pid_fd = open(pidname,O_CREAT|O_EXCL|O_WRONLY,0644);
+  if (pid_fd < 0)
+  {
+    fprintf(stderr,"%s Can't create '%s': %s\n", time_pid_info(), pidname, strerror(errno));
+    if (errno == EEXIST)
+    {
+      fprintf(stderr,"It seems that another indexer is already running!\n");
+      fprintf(stderr,"Remove '%s' if it is not true.\n",pidname);
+    }
+    return UDM_ERROR;
+  }
+  udm_snprintf(pidbuf, sizeof(pidbuf), "%d\n", (int)getpid());
+  write(pid_fd, &pidbuf, strlen(pidbuf));
+#ifdef HAVE_ATEXIT
+  atexit(&exitproc);
+#endif
+  return UDM_OK;
+}
+
+
+static int
+UdmIndInsertFromFile(UDM_AGENT *Agent)
+{
+  int rc= UDM_OK;
+  if (strcmp(url_filename,"-"))
+  {
+    /* Make sure all URLs to be inserted are OK */
+    if (UDM_OK != (rc= UdmURLFile(Agent, url_filename,UDM_URL_FILE_PARSE)))
+    {
+      UdmLog(&Main,UDM_LOG_ERROR,"Error: Invalid URL in '%s'",url_filename);
+      goto ex;
+    }
+  }
+  if (UDM_OK != (rc= UdmURLFile(&Main,url_filename,UDM_URL_FILE_INSERT)))
+  {
+    UdmLog(&Main,UDM_LOG_ERROR,"Error: '%s'",UdmEnvErrMsg(Main.Conf));
+    goto ex;
+  }
+ex:
+  return rc;
+}
+
+
+static int
+UdmIndExpire(UDM_AGENT *Agent)
+{
+  int rc= url_filename ?
+          UdmURLFile(&Main,url_filename,UDM_URL_FILE_REINDEX) :
+          UdmURLAction(&Main, NULL, UDM_URL_ACTION_EXPIRE);
+  if (rc != UDM_OK)
+    UdmLog(&Main,UDM_LOG_ERROR,"Error: '%s'",UdmEnvErrMsg(Main.Conf));
+  return rc;
+}
+
+
+static int
+UdmIndDelete(UDM_AGENT *Agent)
+{
+  int rc;
+  if (UDM_OK != (rc= UdmClear(Agent, url_filename)))
+    UdmLog(&Main,UDM_LOG_ERROR,"Error: '%s'",UdmEnvErrMsg(Main.Conf));
+  return rc;
+}
+
+
+static int
+UdmIndCrawl(UDM_AGENT *Agent)
+{
+  int rc;
+  if (block && UDM_OK != (rc= UdmIndBlock()))
+    goto ex;
+  UdmLog(&Main,UDM_LOG_ERROR, "indexer from %s-%s-%s started with '%s'", PACKAGE, VERSION, UDM_DBTYPE, cname);
+  UdmStoreHrefs(&Main);    /**< store hrefs from config and command line */
+  UdmSigHandlersInit(&Main);
+
+  if (expire)
+  {
+    if (UDM_OK != (rc= UdmIndExpire(&Main)))
+      goto ex;
+  }
+
+  if (UDM_OK != (rc= UdmIndex(&Main)))
+    goto ex;
+#ifdef HAVE_SQL
+  if (pop_rank)
+    rc= UdmSrvAction(&Main, NULL, UDM_SRV_ACTION_POPRANK);
+#endif
+
+ex:
+  return rc;
+}
+
+
 int main(int argc, char **argv)
 {
-  char  *language=NULL,*affix=NULL,*dictionary=NULL;
-  int   pid_fd, rc= 0;
-  char  pidbuf[1024];
+  int   rc= 0;
   char  *REQUEST_METHOD= getenv("REQUEST_METHOD");
   FILE  *logfile= REQUEST_METHOD ? stdout : stderr;
 
@@ -1585,6 +1907,13 @@ int main(int argc, char **argv)
 
   UdmParseCmdLine();
   
+  if (cmd == UDM_IND_INDEX)
+  {
+    fprintf(stderr, "\nWARNING: \"indexer -Eindex\" is deprecated. "
+                    "Use \"indexer -Ecrawl\" instead!\n\n");
+    cmd= UDM_IND_CRAWL;
+  }
+  
   if (cmd == UDM_IND_AMBIGUOUS)
   {
     fprintf(stderr, "Ambiguous indexer command in -E\n");
@@ -1596,14 +1925,14 @@ int main(int argc, char **argv)
     fprintf(stderr, "Unknown indexer command in -E\n");
     help++;
   }
-
+  
   if (cmd == UDM_IND_DUMPCONF)
   {
     load_for_dump|= UDM_FLAG_DONT_ADD_TO_DB;
     load_langmaps= 0;
     load_spells= 0;
   }
-  else if (cmd != UDM_IND_INDEX)
+  else if (cmd != UDM_IND_CRAWL)
   {
     add_servers=0;
     load_langmaps=0;
@@ -1621,8 +1950,8 @@ int main(int argc, char **argv)
   if ((argc>1) || (help))
   {
     usage(help);
-    UdmEnvFree(&Conf);
-    return 1;
+    rc= 1;
+    goto ex;
   }
   
   if (!*cname)
@@ -1639,109 +1968,18 @@ int main(int argc, char **argv)
       cname[sizeof(cname)-1]='\0';
     }
   }
-     
-  if (UDM_OK != UdmIndexerEnvLoad(&Main, cname,
-                                  add_servers + load_langmaps + load_spells +
-                                  add_server_urls + load_for_dump))
+  
+  
+  if (UDM_OK != (rc= UdmIndexerEnvLoad(&Main, cname,
+                                       add_servers + load_langmaps +
+                                       load_spells +
+                                       add_server_urls + load_for_dump)))
   {
     fprintf(logfile, "%s\n", UdmEnvErrMsg(&Conf));
-    UdmEnvFree(&Conf);
-    return 1;
+    goto ex;
   }
 
- 
-  if (cmd == UDM_IND_DUMPCONF)
-  {
-    if (UDM_OK != (rc= UdmEnvSave(&Main, "-", 0)))
-      fprintf(logfile, "%s\n", UdmEnvErrMsg(&Conf));
-    UdmEnvFree(&Conf);
-    return rc;
-  }
-     
-  if (cmd == UDM_IND_CHECKCONF)
-    return 0;
-  
-#ifndef WIN32
-   UdmSetLogLevel(NULL, have_loglevel ? loglevel :
-                  UdmVarListFindInt(&Main.Conf->Vars, "LogLevel", UDM_LOG_INFO));
-   UdmOpenLog("indexer",&Conf, log2stderr);
-#endif
-   
-  if (REQUEST_METHOD)
-  {
-    UdmBitrixIndex(&Main);
-    return 0;
-  }
-  
-  if (cmd == UDM_IND_HASHSPELL)
-  {
-    char errmsg[256];
-    if (UDM_OK != (rc= UdmSpellListListLoad(&Conf.Spells, errmsg, sizeof(errmsg)))||
-        UDM_OK != (rc= UdmSpellListListWriteHash(&Conf.Spells, errmsg, sizeof(errmsg))))
-    {
-      fprintf(stderr, "error: %s\n", errmsg);
-      return 1;
-    }
-    return 0;
-  }
-  
-  if (cmd == UDM_IND_DUMPSPELL)
-  {
-    char errmsg[256];
-    int spflags= UDM_SPELL_NOPREFIX;
-    if (UDM_OK != (rc= UdmSpellListListLoad(&Conf.Spells,
-                                            errmsg, sizeof(errmsg)))||
-        UDM_OK != (rc= UdmAffixListListLoad(&Conf.Affixes, spflags,
-                                            errmsg, sizeof(errmsg)))||
-        UDM_OK != (rc= UdmSpellDump(&Conf.Spells, &Conf.Affixes,
-                                    errmsg, sizeof(errmsg))))
-    {
-      fprintf(stderr, "error: %s\n", errmsg);
-      return 1;
-    }
-    return 0;
-  }
-  
-  if (cmd==UDM_IND_SQLMON)
-  {
-    UDM_SQLMON_PARAM prm;
-    bzero((void*)&prm,sizeof(prm));
-    prm.infile= stdin;
-    prm.outfile= stdout;
-    prm.flags= UDM_SQLMON_DISPLAY_FIELDS;
-    prm.gets= sqlmongets;
-    prm.display= UdmDisplaySQLQuery;
-    prm.prompt= sqlmonprompt;
-    UdmSQLMonitor(&Main, &Conf, &prm);
-    return 0;
-  }
 
-  if (cmd == UDM_IND_MULTI2BLOB)
-  {
-    rc= UdmMulti2Blob(&Main);
-    return rc;
-  }
-  else if (cmd == UDM_IND_EXPORT)
-  {
-    rc= UdmExport(&Main);
-    return rc;
-  }
-  else if (cmd == UDM_IND_WRDSTAT)
-  {
-    rc= UdmURLAction(&Main, NULL, UDM_URL_ACTION_WRDSTAT);
-    return rc;
-  }
-  else if (cmd == UDM_IND_REWRITEURL)
-  {
-    rc= UdmRewriteURL(&Main);
-    return rc;
-  }     
-  else if (cmd == UDM_IND_REWRITELIMITS)
-  {
-    rc= UdmRewriteLimits(&Main);
-    return rc;
-  }
-     
   if (url_filename && strcmp(url_filename,"-"))
   {
     /* Make sure URL file is readable if not STDIN */
@@ -1754,121 +1992,75 @@ int main(int argc, char **argv)
     fclose(url_file);
   }
   
-  if (insert && url_filename)
+  
+  
+  if (cmd == UDM_IND_DUMPCONF)
   {
-    if (strcmp(url_filename,"-"))
-    {
-      /* Make sure all URLs to be inserted are OK */
-      if (UDM_OK!=UdmURLFile(&Main, url_filename,UDM_URL_FILE_PARSE))
-      {
-        UdmLog(&Main,UDM_LOG_ERROR,"Error: Invalid URL in '%s'",url_filename);
-        goto ex;
-      }
-    }      
-    if (UDM_OK != UdmURLFile(&Main,url_filename,UDM_URL_FILE_INSERT))
-    {
-      UdmLog(&Main,UDM_LOG_ERROR,"Error: '%s'",UdmEnvErrMsg(Main.Conf));
-      goto ex;
-    }
-  }
-     
-  if (expire)
-  {
-    int  res;
-          
-    if (url_filename)
-    {
-      res= UdmURLFile(&Main,url_filename,UDM_URL_FILE_REINDEX);
-    }
-    else
-    {
-      res= UdmURLAction(&Main, NULL, UDM_URL_ACTION_EXPIRE);
-    }
-    if (res!=UDM_OK)
-    {
-      UdmLog(&Main,UDM_LOG_ERROR,"Error: '%s'",UdmEnvErrMsg(Main.Conf));
-      goto ex;
-    }
-  }
-     
-  if (affix||dictionary)
-  {
-    if (!language)
-    {
-      UdmLog(&Main,UDM_LOG_ERROR,"Error: Language is not specified for import!");
-    }
-    else if (strlen(language)!=2)
-    {
-      UdmLog(&Main,UDM_LOG_ERROR,"Error: Language should be 2 letters!");
-    }
+    if (UDM_OK != (rc= UdmEnvSave(&Main, "-", 0)))
+      fprintf(logfile, "%s\n", UdmEnvErrMsg(&Conf));
     goto ex;
   }
+  
+  
+  if (cmd == UDM_IND_CHECKCONF)
+  {
+    rc= 0;
+    goto ex;
+  }
+  
+  
+#ifndef WIN32
+   UdmSetLogLevel(NULL, have_loglevel ? loglevel :
+                  UdmVarListFindInt(&Main.Conf->Vars, "LogLevel", UDM_LOG_INFO));
+   UdmOpenLog("indexer",&Conf, log2stderr);
+#endif
 
-  switch(cmd)
+  if (insert && url_filename)
   {
-    case UDM_IND_DELETE:
-      if (UDM_OK != UdmClear(&Main,url_filename))
-      {
-        UdmLog(&Main,UDM_LOG_ERROR,"Error: '%s'",UdmEnvErrMsg(Main.Conf));
-      }
-      break;
-    case UDM_IND_STAT:
-      if (UDM_OK != ShowStatistics(&Main))
-      {
-        UdmLog(&Main,UDM_LOG_ERROR,"Error: '%s'",UdmEnvErrMsg(Main.Conf));
-      }
-      break;
-    case UDM_IND_REFERERS:
-      if (UDM_OK != ShowReferers(&Main))
-      {
-        UdmLog(&Main,UDM_LOG_ERROR,"Error: '%s'",UdmEnvErrMsg(Main.Conf));
-      }
-      break;
-    case UDM_IND_CREATE:
-    case UDM_IND_DROP:
-      if (UDM_OK != CreateOrDrop(&Main,cmd))
-      {
-        UdmLog(&Main,UDM_LOG_ERROR,"Error: '%s'",UdmEnvErrMsg(Main.Conf));
-      }
-      break;
-    default:
-      {
-        if (block)
-        {
-          /* Check that another instance isn't running and create PID file. */
-          const char *vardir= UdmVarListFindStr(&Conf.Vars,"VarDir",UDM_VAR_DIR);
-          sprintf(pidname,"%s/%s", vardir ,"indexer.pid");
-          pid_fd = open(pidname,O_CREAT|O_EXCL|O_WRONLY,0644);
-          if (pid_fd < 0)
-          {
-            fprintf(stderr,"%s Can't create '%s': %s\n", time_pid_info(), pidname, strerror(errno));
-            if (errno == EEXIST)
-            {
-              fprintf(stderr,"It seems that another indexer is already running!\n");
-              fprintf(stderr,"Remove '%s' if it is not true.\n",pidname);
-            }
-            goto ex;
-          }
-          sprintf(pidbuf,"%d\n",(int)getpid());
-          write(pid_fd,&pidbuf,strlen(pidbuf));
-#ifdef HAVE_ATEXIT
-          atexit(&exitproc);
-#endif
-        }
-        UdmLog(&Main,UDM_LOG_ERROR, "indexer from %s-%s-%s started with '%s'", PACKAGE, VERSION, UDM_DBTYPE, cname);
-        UdmStoreHrefs(&Main);    /**< store hrefs from config and command line */
-        UdmSigHandlersInit(&Main);
-        rc= UdmIndex(&Main);
-      }
+    if (UDM_OK != UdmIndInsertFromFile(&Main))
+    {
+      UdmLog(&Main,UDM_LOG_ERROR,"Error: '%s'",UdmEnvErrMsg(Main.Conf));
+      goto ex;
+    }
   }
-     
-#ifdef HAVE_SQL
-  if (pop_rank)
+  
+  
+  if (REQUEST_METHOD)
   {
-    UdmSrvAction(&Main, NULL, UDM_SRV_ACTION_POPRANK);
+    rc= UdmBitrixIndex(&Main);
+    goto ex;
   }
-#endif
-     
+  
+  
+  switch (cmd)
+  {
+    case UDM_IND_HASHSPELL:     rc= UdmIndHashSpell(&Main);     goto ex;
+    case UDM_IND_DUMPSPELL:     rc= UdmIndDumpSpell(&Main);     goto ex;
+    case UDM_IND_DUMPDATA:      rc= UdmDumpDocuments(&Main);    goto ex;
+    case UDM_IND_RESTOREDATA:   rc= UdmRestoreDocuments(&Main); goto ex;
+    case UDM_IND_SQLMON:        rc= UdmIndSQLMonitor(&Main);    goto ex;
+    case UDM_IND_MULTI2BLOB:    rc= UdmMulti2Blob(&Main);       goto ex;
+    case UDM_IND_EXPORT:        rc= UdmExport(&Main);           goto ex;
+    case UDM_IND_WRDSTAT:       rc= UdmIndWordStat(&Main);      goto ex;
+    case UDM_IND_REWRITEURL:    rc= UdmRewriteURL(&Main);       goto ex;
+    case UDM_IND_REWRITELIMITS: rc= UdmRewriteLimits(&Main);    goto ex;
+    case UDM_IND_STAT:          rc= ShowStatistics(&Main);      goto ex;
+    case UDM_IND_REFERERS:      rc= ShowReferers(&Main);        goto ex;
+    case UDM_IND_CREATE:        rc= UdmIndCreate(&Main);        goto ex;
+    case UDM_IND_DROP:          rc= UdmIndDrop(&Main);          goto ex;
+    case UDM_IND_DELETE:        rc= UdmIndDelete(&Main);        goto ex;
+    case UDM_IND_CRAWL:         rc= UdmIndCrawl(&Main);         goto ex;
+    case UDM_IND_INDEX:         rc= UdmIndCrawl(&Main);         goto ex;
+
+    case UDM_IND_AMBIGUOUS:
+    case UDM_IND_UNKNOWN:
+    case UDM_IND_CHECKCONF:
+    case UDM_IND_CONVERT:
+    case UDM_IND_DUMPCONF:
+      break;
+  }
+  
+  
 ex:
   total_threads= 0;
   UdmAgentFree(&Main);
