@@ -860,7 +860,7 @@ int UdmHTMLParseTag(UDM_HTMLTOK * tag,UDM_DOCUMENT * Doc)
       char *p;
       if((p=strstr(metacont,"charset=")))
       {
-        const char *cs = UdmCharsetCanonicalName(UdmTrim(p + 8, " \t"));
+        const char *cs = UdmCharsetCanonicalName(UdmTrim(p + 8, "' \t"));
         UdmVarListReplaceStr(&Doc->Sections, "Meta-Charset", cs ? cs : p + 8);
       }
     }
