@@ -512,7 +512,8 @@ static int endElement(UDM_XML_PARSER *parser, const char *name, size_t l)
   {
     char *prevname= UdmStrndup(name, l);
     const char *val;
-    if ((val= UdmVarListFindStr(&D->Indexer->Conf->XMLLeaveHooks,
+    if (D->Href.url &&
+        (val= UdmVarListFindStr(&D->Indexer->Conf->XMLLeaveHooks,
                                 prevname, NULL)))
     {
       UDM_DOCUMENT *Doc= D->Doc;
@@ -574,7 +575,7 @@ static int Text (UDM_XML_PARSER *parser, const char *s, size_t len)
   else if(D->XMLDefaultSection)
   {
     Item.section= D->body_sec;
-    Item.section_name= D->XMLDefaultSection;
+    Item.section_name= (char*) D->XMLDefaultSection;
   }
   else
   {
