@@ -31,6 +31,7 @@
 #include "udm_parsexml.h"
 #include "udm_vars.h"
 
+#include "udm_db.h" /* for UdmURLAction */
 
 
 UDM_RESULT *UdmResultInit(UDM_RESULT *Res)
@@ -196,205 +197,9 @@ int UdmResultToTextBuf(UDM_RESULT *R,char *buf,size_t len){
 
 /*****************************************************************/
 
-#define UDM_DF_SIZE 64
-
-typedef struct
-{
-  int state;
-  UDM_WIDEWORD WW;
-  UDM_DOCUMENT Doc;
-  UDM_RESULT *Res;
-  UDM_CHARSET *cs;
-  uint4       score;
-  uint4       per_site;
-  urlid_t     site_id;
-  char date_format[UDM_DF_SIZE];
-} RES_PARSER_DATA;
-
-
-#define RES_MISC 0
-
-struct udm_res_section_st
-{
-  int        state;
-  size_t     length;
-  const char *str;
-};
-
-#define UDM_RESSEC_TOTAL_RESULTS       100
-
-#define UDM_RESSEC_WORD                1000
-#define UDM_RESSEC_WORD_ID             1001
-#define UDM_RESSEC_WORD_ORDER          1002
-#define UDM_RESSEC_WORD_COUNT          1003
-#define UDM_RESSEC_WORD_ORIGIN         1004
-#define UDM_RESSEC_WORD_WEIGHT         1005
-#define UDM_RESSEC_WORD_MATCH          1006
-#define UDM_RESSEC_WORD_SECNO          1007
-#define UDM_RESSEC_WORD_PHRLEN         1008
-#define UDM_RESSEC_WORD_PHRPOS         1009
-#define UDM_RESSEC_WORD_WORD           1010
-
-#define UDM_RESSEC_ITEM                2000
-#define UDM_RESSEC_ITEM_TITLE          2001
-#define UDM_RESSEC_ITEM_DESCR          2002
-#define UDM_RESSEC_ITEM_LINK           2003
-#define UDM_RESSEC_ITEM_ID             2004
-#define UDM_RESSEC_ITEM_SCORE          2005
-#define UDM_RESSEC_ITEM_UPDATED        2006
-#define UDM_RESSEC_ITEM_CONTENT_LENGTH 2007
-#define UDM_RESSEC_ITEM_CONTENT_TYPE   2008
-#define UDM_RESSEC_ITEM_CACHED_HREF    2009
-#define UDM_RESSEC_ITEM_CACHED_CONT    2010
-#define UDM_RESSEC_ITEM_PERSITE        2011
-#define UDM_RESSEC_ITEM_SITE_ID        2012
-#define UDM_RESSEC_ITEM_TITLE_B        2013
-#define UDM_RESSEC_ITEM_DESCR_B        2014
-#define UDM_RESSEC_ITEM_TAG            2015
-
-#define WSL "rss.channel.mnoGoSearch:WordStatList"
-#define WSI "rss.channel.mnoGoSearch:WordStatList.mnoGoSearch:WordStatItem"
-#define WSI_LEN 61
-
-static struct udm_res_section_st res_sec[]=
-{
-  {UDM_RESSEC_WORD,                  20, "result.wordinfo.word"},
-  {UDM_RESSEC_WORD_ID,               23, "result.wordinfo.word.id"},
-  {UDM_RESSEC_WORD_ORDER,            26, "result.wordinfo.word.order"},
-  {UDM_RESSEC_WORD_COUNT,            26, "result.wordinfo.word.count"},
-  {UDM_RESSEC_WORD_ORIGIN,           27, "result.wordinfo.word.origin"},
-  {UDM_RESSEC_WORD_WEIGHT,           27, "result.wordinfo.word.weight"},
-  {UDM_RESSEC_WORD_MATCH,            26, "result.wordinfo.word.match"},
-  {UDM_RESSEC_WORD_SECNO,            26, "result.wordinfo.word.secno"},
-  {UDM_RESSEC_WORD_PHRLEN,           27, "result.wordinfo.word.phrlen"},
-  {UDM_RESSEC_WORD_PHRPOS,           27, "result.wordinfo.word.phrpos"},
-  {UDM_RESSEC_TOTAL_RESULTS,         19, "result.totalResults"},
-
-  {UDM_RESSEC_WORD,                  WSI_LEN,     WSI},
-  {UDM_RESSEC_WORD_ID,               WSI_LEN + 3, WSI ".id"},
-  {UDM_RESSEC_WORD_WORD,             WSI_LEN + 5, WSI ".word"},
-  {UDM_RESSEC_WORD_ORDER,            WSI_LEN + 6, WSI ".order"},
-  {UDM_RESSEC_WORD_COUNT,            WSI_LEN + 6, WSI ".count"},
-  {UDM_RESSEC_WORD_ORIGIN,           WSI_LEN + 7, WSI ".origin"},
-  {UDM_RESSEC_WORD_WEIGHT,           WSI_LEN + 7, WSI ".weight"},
-  {UDM_RESSEC_WORD_MATCH,            WSI_LEN + 6, WSI ".match"},
-  {UDM_RESSEC_WORD_SECNO,            WSI_LEN + 6, WSI ".secno"},
-  {UDM_RESSEC_WORD_PHRLEN,           WSI_LEN + 6, WSI ".phrlen"},
-  {UDM_RESSEC_WORD_PHRPOS,           WSI_LEN + 6, WSI ".phrpos"},
-
-  {UDM_RESSEC_ITEM,                  16, "rss.channel.item"},
-  {UDM_RESSEC_ITEM_TITLE,            22, "rss.channel.item.title"},
-  {UDM_RESSEC_ITEM_TITLE_B,          24, "rss.channel.item.title.b"},
-  {UDM_RESSEC_ITEM_DESCR,            28, "rss.channel.item.description"},
-  {UDM_RESSEC_ITEM_DESCR_B,          30, "rss.channel.item.description.b"},
-  {UDM_RESSEC_ITEM_LINK,             21, "rss.channel.item.link"},
-  {UDM_RESSEC_ITEM_ID,               19, "rss.channel.item.id"},
-  {UDM_RESSEC_ITEM_SCORE,            22, "rss.channel.item.score"},
-  {UDM_RESSEC_ITEM_UPDATED,          24, "rss.channel.item.updated"},
-  {UDM_RESSEC_ITEM_UPDATED,          24, "rss.channel.item.pubDate"},
-  {UDM_RESSEC_ITEM_CONTENT_LENGTH,   31, "rss.channel.item.content-length"},
-  {UDM_RESSEC_ITEM_CONTENT_TYPE,     29, "rss.channel.item.content-type"},
-  {UDM_RESSEC_ITEM_CACHED_HREF,      23, "rss.channel.item.cached"},
-  {UDM_RESSEC_ITEM_PERSITE,          24, "rss.channel.item.persite"},
-  {UDM_RESSEC_ITEM_SITE_ID,          23, "rss.channel.item.siteid"},
-  {UDM_RESSEC_ITEM_CACHED_CONT,      31, "rss.channel.item.cached-content"},
-  {UDM_RESSEC_ITEM_TAG,              20, "rss.channel.item.tag"},
-
-  {UDM_RESSEC_TOTAL_RESULTS,         35, "rss.channel.openSearch:totalResults"},
-
-  {0, 0, NULL}
-};
-
-
-static struct udm_res_section_st *res_sec_find(const char *attr, size_t len)
-{
-  struct udm_res_section_st *s;
-  for (s= res_sec; s->str; s++)
-  {
-    if (len == s->length && !strncasecmp(attr, s->str, len))
-      return s;
-  }
-  return NULL;
-}
-
-                    
-static int ResFromXMLEnter(UDM_XML_PARSER *parser, const char *name, size_t l)
-{
-  RES_PARSER_DATA *D = parser->user_data;
-  struct udm_res_section_st *st= res_sec_find(parser->attr,
-                                              parser->attrend - parser->attr);
-  D->state= st ? st->state : 0;
-  if (D->state == UDM_RESSEC_WORD)
-  {
-    UdmWideWordInit(&D->WW);
-    D->WW.origin= UDM_WORD_ORIGIN_QUERY;
-  }
-  if (D->state == UDM_RESSEC_ITEM)
-  {
-    char dbuf[128];
-    UdmDocInit(&D->Doc);
-    snprintf(dbuf, 128, "%.5f", (float) 0);
-    UdmVarListReplaceStr(&D->Doc.Sections, "Pop_Rank", dbuf);
-  }
-  return(UDM_XML_OK);
-}
-
-static int ResFromXMLLeave(UDM_XML_PARSER *parser, const char *name, size_t l)
-{
-
-  RES_PARSER_DATA *D = parser->user_data;
-  struct udm_res_section_st *st= res_sec_find(parser->attr,
-                                              parser->attrend - parser->attr);
-  D->state= st ? st->state : 0;
-
-  if (D->state == UDM_RESSEC_WORD)
-  {
-    if (!D->WW.word)
-    {
-      D->WW.word= UdmStrdup("<empty>");
-      D->WW.len= 7;
-    }
-    UdmWideWordListAddForStat(&D->Res->WWList, &D->WW);
-    UdmWideWordFree(&D->WW);
-  }
-  if (D->state == UDM_RESSEC_ITEM)
-  {
-    size_t nbytes;
-    D->Res->URLData.nitems++;
-    D->Res->num_rows++;
-    nbytes= D->Res->num_rows * sizeof(UDM_DOCUMENT);
-    D->Res->Doc= (UDM_DOCUMENT*) UdmRealloc(D->Res->Doc, nbytes);
-    D->Res->Doc[D->Res->num_rows-1]= D->Doc;
-    bzero((void*)&D->Doc, sizeof(UDM_DOCUMENT));
-    
-    nbytes= D->Res->num_rows * sizeof(UDM_URLDATA);
-    D->Res->URLData.Item= (UDM_URLDATA*) UdmRealloc(D->Res->URLData.Item, nbytes);
-    bzero((void*)&D->Res->URLData.Item[D->Res->num_rows-1], sizeof(UDM_URLDATA));
-    D->Res->URLData.Item[D->Res->num_rows-1].url_id= D->Res->URLData.nitems-1;
-    D->Res->URLData.Item[D->Res->num_rows-1].score= D->score;
-    D->Res->URLData.Item[D->Res->num_rows-1].per_site= D->per_site;
-    D->Res->URLData.Item[D->Res->num_rows-1].site_id= D->site_id;
-    
-    D->score= 0;
-    D->per_site= 0;
-    D->site_id= 0;
-  }
-  /* fprintf(stderr, "leave: len=%d '%s'\n", l, name);*/
-  return(UDM_XML_OK);
-}
-
-
-static int UdmVarListReplaceStrn(UDM_VARLIST *vars,
-                                 const char *name, const char *val, size_t len)
-{
-  int rc;
-  char *v= UdmStrndup(val, len);
-  rc= UdmVarListReplaceStr(vars, name, v);
-  UdmFree(v);
-  return rc;
-}
-
-
+/*
+  This function should probably be moved to var.c
+*/
 static int
 UdmVarListReplaceOrAppendStrn(UDM_VARLIST *vars, const char *name,
                               const char *val, size_t len, int hl)
@@ -422,6 +227,305 @@ UdmVarListReplaceOrAppendStrn(UDM_VARLIST *vars, const char *name,
 }
 
 
+/*
+  This function should probably be moved to vars.c
+*/
+static int
+UdmVarListReplaceStrn(UDM_VARLIST *vars,
+                      const char *name,
+                      const char *val, size_t len)
+{
+  int rc;
+  char *v= UdmStrndup(val, len);
+  rc= UdmVarListReplaceStr(vars, name, v);
+  UdmFree(v);
+  return rc;
+}
+
+
+/*****************************************************************/
+#define UDM_DF_SIZE 64
+
+typedef struct
+{
+  int state;
+  UDM_AGENT *Agent;
+  UDM_WIDEWORD WW;
+  UDM_DOCUMENT Doc;
+  UDM_RESULT *Res;
+  UDM_CHARSET *cs;
+  uint4       score;
+  uint4       per_site;
+  urlid_t     site_id;
+  char date_format[UDM_DF_SIZE];
+} RES_PARSER_DATA;
+
+
+static int
+PD_ReplaceOrAppendStrn_hl0(RES_PARSER_DATA *D,
+                        const char *name,
+                        const char *val, size_t len)
+{
+  return UdmVarListReplaceOrAppendStrn(&D->Doc.Sections, name, val, len, 0);
+}
+
+
+static int
+PD_ReplaceOrAppendStrn_hl1(RES_PARSER_DATA *D,
+                           const char *name,
+                           const char *val, size_t len)
+{
+  return UdmVarListReplaceOrAppendStrn(&D->Doc.Sections, name, val, len, 1);
+}
+
+
+static int
+PD_ReplaceStrn(RES_PARSER_DATA *D,
+               const char *name,
+               const char *val, size_t len)
+{
+  return UdmVarListReplaceStrn(&D->Doc.Sections, name, val, len);
+}
+
+
+static int
+PD_ReplaceLastModified(RES_PARSER_DATA *D,
+                       const char *name,
+                       const char *val, size_t len)
+{
+  time_t last_mod_time;
+  char tmp[64];
+  len= len >= sizeof(tmp) ? sizeof(tmp) - 1 : len;
+  memcpy(tmp, val, len);
+  tmp[len]= '\0';
+  last_mod_time= UdmHttpDate2Time_t(tmp);
+  UdmVarListReplaceInt(&D->Doc.Sections, "Last-Modified-Timestamp", (int) last_mod_time);
+  if ((len= strftime(tmp, sizeof(tmp)-1, D->date_format, localtime(&last_mod_time))))
+    tmp[len]= '\0';
+  else
+    UdmTime_t2HttpStr(last_mod_time, tmp);
+  UdmVarListReplaceStr(&D->Doc.Sections, "Last-Modified", tmp);
+  return UDM_OK;
+}
+
+
+typedef int
+(*section_handler)(RES_PARSER_DATA *D,
+                   const char *name,
+                   const char *val, size_t len);
+
+
+
+
+#define RES_MISC 0
+
+struct udm_res_section_st
+{
+  int        state;
+  size_t     length;
+  const char *str;
+  const char *section_name;
+  section_handler handler;
+};
+
+#define UDM_RESSEC_TOTAL_RESULTS       100
+
+#define UDM_RESSEC_WORD                1000
+#define UDM_RESSEC_WORD_ID             1001
+#define UDM_RESSEC_WORD_ORDER          1002
+#define UDM_RESSEC_WORD_COUNT          1003
+#define UDM_RESSEC_WORD_ORIGIN         1004
+#define UDM_RESSEC_WORD_WEIGHT         1005
+#define UDM_RESSEC_WORD_MATCH          1006
+#define UDM_RESSEC_WORD_SECNO          1007
+#define UDM_RESSEC_WORD_PHRLEN         1008
+#define UDM_RESSEC_WORD_PHRPOS         1009
+#define UDM_RESSEC_WORD_WORD           1010
+
+#define UDM_RESSEC_ITEM                2000
+#define UDM_RESSEC_ITEM_DESCR          2002
+#define UDM_RESSEC_ITEM_SCORE          2005
+
+#define UDM_RESSEC_ITEM_PERSITE        2011
+#define UDM_RESSEC_ITEM_SITE_ID        2012
+#define UDM_RESSEC_ITEM_DESCR_B        2014
+
+#define WSL "rss.channel.mnoGoSearch:WordStatList"
+#define WSI "rss.channel.mnoGoSearch:WordStatList.mnoGoSearch:WordStatItem"
+#define WSI_LEN 61
+#define RCI(x) ("rss.channel.item." x)
+
+static struct udm_res_section_st res_sec[]=
+{
+  {UDM_RESSEC_WORD,                  20, "result.wordinfo.word",        NULL, NULL},
+  {UDM_RESSEC_WORD_ID,               23, "result.wordinfo.word.id",     NULL, NULL},
+  {UDM_RESSEC_WORD_ORDER,            26, "result.wordinfo.word.order",  NULL, NULL},
+  {UDM_RESSEC_WORD_COUNT,            26, "result.wordinfo.word.count",  NULL, NULL},
+  {UDM_RESSEC_WORD_ORIGIN,           27, "result.wordinfo.word.origin", NULL, NULL},
+  {UDM_RESSEC_WORD_WEIGHT,           27, "result.wordinfo.word.weight", NULL, NULL},
+  {UDM_RESSEC_WORD_MATCH,            26, "result.wordinfo.word.match",  NULL, NULL},
+  {UDM_RESSEC_WORD_SECNO,            26, "result.wordinfo.word.secno",  NULL, NULL},
+  {UDM_RESSEC_WORD_PHRLEN,           27, "result.wordinfo.word.phrlen", NULL, NULL},
+  {UDM_RESSEC_WORD_PHRPOS,           27, "result.wordinfo.word.phrpos", NULL, NULL},
+  {UDM_RESSEC_TOTAL_RESULTS,         19, "result.totalResults",         NULL, NULL},
+  
+  {UDM_RESSEC_WORD,                  WSI_LEN,     WSI,            NULL, NULL},
+  {UDM_RESSEC_WORD_ID,               WSI_LEN + 3, WSI ".id",      NULL, NULL},
+  {UDM_RESSEC_WORD_WORD,             WSI_LEN + 5, WSI ".word",    NULL, NULL},
+  {UDM_RESSEC_WORD_ORDER,            WSI_LEN + 6, WSI ".order",   NULL, NULL},
+  {UDM_RESSEC_WORD_COUNT,            WSI_LEN + 6, WSI ".count",   NULL, NULL},
+  {UDM_RESSEC_WORD_ORIGIN,           WSI_LEN + 7, WSI ".origin",  NULL, NULL},
+  {UDM_RESSEC_WORD_WEIGHT,           WSI_LEN + 7, WSI ".weight",  NULL, NULL},
+  {UDM_RESSEC_WORD_MATCH,            WSI_LEN + 6, WSI ".match",   NULL, NULL},
+  {UDM_RESSEC_WORD_SECNO,            WSI_LEN + 6, WSI ".secno",   NULL, NULL},
+  {UDM_RESSEC_WORD_PHRLEN,           WSI_LEN + 6, WSI ".phrlen",  NULL, NULL},
+  {UDM_RESSEC_WORD_PHRPOS,           WSI_LEN + 6, WSI ".phrpos",  NULL, NULL},
+
+  {UDM_RESSEC_TOTAL_RESULTS,         35, "rss.channel.openSearch:totalResults"},
+  
+  {UDM_RESSEC_ITEM,                  16, "rss.channel.item",                NULL, NULL},
+
+  {UDM_RESSEC_ITEM_SCORE,            22, "rss.channel.item.score",          NULL, NULL},
+  {UDM_RESSEC_ITEM_PERSITE,          24, "rss.channel.item.persite",        NULL, NULL},
+  {UDM_RESSEC_ITEM_SITE_ID,          23, "rss.channel.item.siteid",         NULL, NULL},
+
+  /* Duplicate: */
+  {0, 24, RCI("updated"),         "Last-Modified",    PD_ReplaceLastModified},
+  {0, 24, RCI("pubDate"),         "Last-Modified",    PD_ReplaceLastModified},
+  {0, 30, RCI("last-modified"),   "Last-Modified",    PD_ReplaceLastModified},
+
+  {0, 22, RCI("title"),           "title",            PD_ReplaceOrAppendStrn_hl0},
+  {0, 24, RCI("title.b"),         "title",            PD_ReplaceOrAppendStrn_hl1},
+  
+  /* Duplicate: */
+  {0, 21, RCI("body"),            "body",             PD_ReplaceOrAppendStrn_hl0},
+  {0, 28, RCI("description"),     "body",             PD_ReplaceOrAppendStrn_hl0},
+  
+  {0, 30, RCI("description.b"),   "body",             PD_ReplaceOrAppendStrn_hl1},
+  
+  /* Duplicate: */
+  {0, 21, RCI("link"),            "url",              PD_ReplaceStrn},
+  {0, 20, RCI("url"),             "url",              PD_ReplaceStrn},
+  
+  {0, 19, RCI("id"),              "id",               PD_ReplaceStrn}, 
+  {0, 31, RCI("content-length"),  "Content-Length",   PD_ReplaceStrn},
+  {0, 29, RCI("content-type"),    "Content-Type",     PD_ReplaceStrn},
+  {0, 23, RCI("cached"),          "stored_href",      PD_ReplaceStrn},
+  
+  /* Duplicate: */
+  {0, 33, RCI("CachedCopyBase64"),"CachedCopyBase64", PD_ReplaceStrn},
+  {0, 31, RCI("cached-content"),  "CachedCopyBase64", PD_ReplaceStrn}, 
+  
+  {0, 20, RCI("tag"),             "tag",              PD_ReplaceStrn},
+  {0, 22, RCI("crc32"),           "crc32",            PD_ReplaceStrn},
+  {0, 24, RCI("charset"),         "charset",          PD_ReplaceStrn},
+  {0, 23, RCI("status"),          "status",           PD_ReplaceStrn},
+
+  {0, 0, NULL, NULL, NULL}
+};
+
+
+static struct udm_res_section_st *res_sec_find(const char *attr, size_t len)
+{
+  struct udm_res_section_st *s;
+  for (s= res_sec; s->str; s++)
+  {
+    if (len == s->length && !strncasecmp(attr, s->str, len))
+      return s;
+  }
+  return NULL;
+}
+
+                    
+static int
+ResFromXMLEnter(UDM_XML_PARSER *parser, const char *name, size_t l)
+{
+  RES_PARSER_DATA *D = parser->user_data;
+  struct udm_res_section_st *st= res_sec_find(parser->attr,
+                                              parser->attrend - parser->attr);
+  D->state= st ? st->state : 0;
+  if (D->state == UDM_RESSEC_WORD)
+  {
+    UdmWideWordInit(&D->WW);
+    D->WW.origin= UDM_WORD_ORIGIN_QUERY;
+  }
+  if (D->state == UDM_RESSEC_ITEM)
+  {
+    char dbuf[128];
+    UdmDocInit(&D->Doc);
+    snprintf(dbuf, 128, "%.5f", (float) 0);
+    UdmVarListReplaceStr(&D->Doc.Sections, "Pop_Rank", dbuf);
+  }
+  return(UDM_XML_OK);
+}
+
+
+static int
+ResFromXMLAddDocHook(UDM_XML_PARSER *parser)
+{
+  RES_PARSER_DATA *D= parser->user_data;
+  size_t nbytes;
+  D->Res->URLData.nitems++;
+  D->Res->num_rows++;
+  nbytes= D->Res->num_rows * sizeof(UDM_DOCUMENT);
+  D->Res->Doc= (UDM_DOCUMENT*) UdmRealloc(D->Res->Doc, nbytes);
+  D->Res->Doc[D->Res->num_rows-1]= D->Doc;
+  bzero((void*)&D->Doc, sizeof(UDM_DOCUMENT));
+  
+  nbytes= D->Res->num_rows * sizeof(UDM_URLDATA);
+  D->Res->URLData.Item= (UDM_URLDATA*) UdmRealloc(D->Res->URLData.Item, nbytes);
+  bzero((void*)&D->Res->URLData.Item[D->Res->num_rows-1], sizeof(UDM_URLDATA));
+  D->Res->URLData.Item[D->Res->num_rows-1].url_id= D->Res->URLData.nitems-1;
+  D->Res->URLData.Item[D->Res->num_rows-1].score= D->score;
+  D->Res->URLData.Item[D->Res->num_rows-1].per_site= D->per_site;
+  D->Res->URLData.Item[D->Res->num_rows-1].site_id= D->site_id;
+  
+  D->score= 0;
+  D->per_site= 0;
+  D->site_id= 0;
+  return UDM_OK;
+}
+
+static int
+ResFromXMLAddDocHookImport(UDM_XML_PARSER *parser)
+{
+  RES_PARSER_DATA *Data= parser->user_data;
+  UDM_DOCUMENT *D= &Data->Doc;
+  UdmURLAction(Data->Agent, D, UDM_URL_ACTION_RESTOREDATA);
+  UdmVarListFree(&D->Sections);
+  return UDM_OK;
+}
+
+static int
+ResFromXMLLeave(UDM_XML_PARSER *parser, const char *name, size_t l)
+{
+  RES_PARSER_DATA *D= parser->user_data;
+  struct udm_res_section_st *st= res_sec_find(parser->attr,
+                                              parser->attrend - parser->attr);
+  D->state= st ? st->state : 0;
+
+  if (D->state == UDM_RESSEC_WORD)
+  {
+    if (!D->WW.word)
+    {
+      D->WW.word= UdmStrdup("<empty>");
+      D->WW.len= 7;
+    }
+    UdmWideWordListAddForStat(&D->Res->WWList, &D->WW);
+    UdmWideWordFree(&D->WW);
+  }
+  if (D->state == UDM_RESSEC_ITEM)
+  {
+    if (D->Res)
+      ResFromXMLAddDocHook(parser);
+    else
+      ResFromXMLAddDocHookImport(parser);
+  }
+  /* fprintf(stderr, "leave: len=%d '%s'\n", l, name);*/
+  return(UDM_XML_OK);
+}
+
+
 static double
 udm_strntod(const char *s, size_t len)
 {
@@ -438,7 +542,15 @@ static int ResFromXMLValue(UDM_XML_PARSER *parser, const char *s, size_t len)
   RES_PARSER_DATA *D= parser->user_data;
   struct udm_res_section_st *st= res_sec_find(parser->attr,
                                               parser->attrend - parser->attr);
-  D->state= st ? st->state : 0;
+  if (!st)
+    return UDM_XML_OK;
+  
+  if (st->handler)
+  {
+    st->handler(D, st->section_name, s, len);
+    return UDM_XML_OK;
+  }
+  D->state= st->state;
   switch (D->state)
   {
     case  UDM_RESSEC_WORD         :
@@ -477,36 +589,6 @@ static int ResFromXMLValue(UDM_XML_PARSER *parser, const char *s, size_t len)
     case  UDM_RESSEC_WORD_PHRPOS  :
       D->WW.phrpos= atoi(s);
       break;
-    case UDM_RESSEC_ITEM_TITLE:
-      UdmVarListReplaceOrAppendStrn(&D->Doc.Sections, "title", s, len, 0);
-      break;
-    case UDM_RESSEC_ITEM_TITLE_B:
-      UdmVarListReplaceOrAppendStrn(&D->Doc.Sections, "title", s, len, 1);
-      break;
-    case UDM_RESSEC_ITEM_DESCR:
-      UdmVarListReplaceOrAppendStrn(&D->Doc.Sections, "body", s, len, 0);
-      break;
-    case UDM_RESSEC_ITEM_DESCR_B:
-      UdmVarListReplaceOrAppendStrn(&D->Doc.Sections, "body", s, len, 1);
-      break;
-    case UDM_RESSEC_ITEM_LINK:
-      UdmVarListReplaceStrn(&D->Doc.Sections, "url", s, len);
-      break;
-    case UDM_RESSEC_ITEM_ID:
-      UdmVarListReplaceStrn(&D->Doc.Sections, "id", s, len);
-      break;
-    case UDM_RESSEC_ITEM_CONTENT_LENGTH:
-      UdmVarListReplaceStrn(&D->Doc.Sections, "Content-Length", s, len);
-      break;
-    case UDM_RESSEC_ITEM_CONTENT_TYPE:
-      UdmVarListReplaceStrn(&D->Doc.Sections, "Content-Type", s, len);
-      break;
-    case UDM_RESSEC_ITEM_CACHED_HREF:
-      UdmVarListReplaceStrn(&D->Doc.Sections, "stored_href", s, len);
-      break;
-    case UDM_RESSEC_ITEM_CACHED_CONT:
-      UdmVarListReplaceStrn(&D->Doc.Sections, "CachedCopyBase64", s, len);
-      break;
     case UDM_RESSEC_ITEM_PERSITE:
       D->per_site= udm_strntod(s, len);
       break;
@@ -514,27 +596,8 @@ static int ResFromXMLValue(UDM_XML_PARSER *parser, const char *s, size_t len)
       D->site_id= udm_strntod(s, len);
       UdmVarListReplaceStrn(&D->Doc.Sections, "Site_id", s, len);
       break;
-    case UDM_RESSEC_ITEM_UPDATED:
-      {
-        time_t last_mod_time;
-        char tmp[64];
-        len= len >= sizeof(tmp) ? sizeof(tmp) - 1 : len;
-        memcpy(tmp, s, len);
-        tmp[len]= '\0';
-        last_mod_time= UdmHttpDate2Time_t(tmp);
-        UdmVarListReplaceInt(&D->Doc.Sections, "Last-Modified-Timestamp", (int) last_mod_time);
-        if ((len= strftime(tmp, sizeof(tmp)-1, D->date_format, localtime(&last_mod_time))))
-          tmp[len]= '\0';
-        else
-          UdmTime_t2HttpStr(last_mod_time, tmp);
-        UdmVarListReplaceStr(&D->Doc.Sections, "Last-Modified", tmp);
-      }
-      break;
     case UDM_RESSEC_ITEM_SCORE:
-      D->score= udm_strntod(s, len) * 1000;
-      break;
-    case UDM_RESSEC_ITEM_TAG:
-      UdmVarListReplaceStrn(&D->Doc.Sections, "Tag", s, len);
+      D->score= udm_strntod(s, len) * 1000 + 0.5;
       break;
     case UDM_RESSEC_TOTAL_RESULTS:
       D->Res->total_found= atoi(s);
@@ -557,6 +620,7 @@ UdmResultFromXML(UDM_AGENT *A, UDM_RESULT *Res,
   UdmXMLParserCreate(&parser);
   parser.flags |= UDM_XML_SKIP_TEXT_NORMALIZATION;
   bzero(&Data, sizeof(Data));
+  Data.Agent= A;
   Data.Res= Res;
   Data.cs= cs;
   udm_snprintf(Data.date_format, UDM_DF_SIZE, "%s", date_format);
