@@ -150,6 +150,7 @@ UdmAddNodeQueryString(UDM_VARLIST *Vars)
   size_t dstlen;
   size_t ps= UdmVarListFindInt(Vars, "ps", 10);
   size_t np= UdmVarListFindInt(Vars, "np", 0);
+  size_t offs= UdmVarListFindInt(Vars, "offs", 0);
   char *dst, *d;
   const char *query_string, *s;
   int group_by_site= UdmVarListFindBool(Vars, "GroupBySite", 0) &&
@@ -171,6 +172,9 @@ UdmAddNodeQueryString(UDM_VARLIST *Vars)
     else if (!strncasecmp(s, "np=", 3))
     {
     }
+    else if (!strncasecmp(s, "offs=", 5))
+    {
+    }
     else
     {
       size_t toklen= amp - s;
@@ -183,7 +187,7 @@ UdmAddNodeQueryString(UDM_VARLIST *Vars)
       break;
     s= amp + 1;
   }
-  sprintf(d, "&ps=%d", ps * (np + 1) * group_by_site_factor);
+  sprintf(d, "&ps=%d", (offs + ps * (np + 1)) * group_by_site_factor);
   UdmVarListReplaceStr(Vars, "NODE_QUERY_STRING", dst);
   UdmFree(dst);
 }
@@ -236,8 +240,7 @@ int __UDMCALL UdmFindWordsSearchd(UDM_AGENT *A,UDM_RESULT *Res, UDM_DB *db)
   
   ticks= UdmStartTimer();
   rc= UdmGetURL(A, &Inc);
-  ticks= UdmStartTimer() - ticks;
-  UdmLog(A, UDM_LOG_DEBUG, "Received searchd response: %.2f", (float) ticks/1000);
+  UdmLog(A, UDM_LOG_DEBUG, "Received searchd response: %.2f", UdmStopTimer(&ticks));
   
   if (UDM_OK != rc)
     return rc;
@@ -252,8 +255,7 @@ int __UDMCALL UdmFindWordsSearchd(UDM_AGENT *A,UDM_RESULT *Res, UDM_DB *db)
   length= Inc.Buf.size - (Inc.Buf.content - Inc.Buf.buf);
   UdmResultFromXML(A, Res, Inc.Buf.content, length, A->Conf->lcs);
   UdmDocFree(&Inc);
-  ticks= UdmStartTimer() - ticks;
-  UdmLog(A, UDM_LOG_DEBUG, "Stop parsing results: %.2f", (float) ticks/1000);
+  UdmLog(A, UDM_LOG_DEBUG, "Stop parsing results: %.2f", UdmStopTimer(&ticks));
   UdmLog(A, UDM_LOG_DEBUG, "searchd: %d rows, %d totalResults", Res->num_rows, Res->total_found);
   return UDM_OK;
 }
