@@ -69,6 +69,7 @@ typedef struct udm_stack_parser_state_st
   int dehyphenate;
   int strip_accents;
   int phrpos;
+  int user_weight;
   size_t SubstringMatchMinWordLength;
   const char *lang;
 } UDM_STACK_PARSER_STATE;
@@ -159,7 +160,10 @@ UdmWordMatchApplyLength(UDM_STACK_PARSER_STATE *S, size_t wlen)
 static int
 UdmStackItemListAddWord(UDM_AGENT *query, UDM_RESULT *Res,
                         UDM_STACK_PARSER_STATE *state,
-                        size_t length, int *lt, char *wrd)
+                        size_t length,
+                        /* Pointers to the next token and the query end*/
+                        int *lt, int *ltend, 
+                        char *wrd)
 {
   UDM_WIDEWORD OWord;
   UDM_WIDEWORDLIST Forms;
@@ -169,6 +173,18 @@ UdmStackItemListAddWord(UDM_AGENT *query, UDM_RESULT *Res,
   int end_of_phrase= 0;
   int final_word_match= UdmWordMatchApplyLength(state, length);
 
+  /*
+    Ignore AutoPhraseChar in case when we have a sequence:
+    - a word
+    - followed by a AutoPhraseChar
+    - followed by a separator
+    For example: B. Abama
+    Notice space character after dot.
+  */
+  if (&lt[1] < ltend &&
+      UdmUniCType(udm_unidata_default, lt[1]) == UDM_UNI_SEPAR)
+    lt_auto_phrase= 0;
+
   if(Res->WWList.nuniq >= UDM_MAXWORDPERQUERY-1)
     return UDM_OK;
 
@@ -227,6 +243,7 @@ UdmStackItemListAddWord(UDM_AGENT *query, UDM_RESULT *Res,
       W->phrlen= phrlen;
   }
 
+  /* Add the original word form */
   bzero((void*) &OWord, sizeof(OWord));
   OWord.len= strlen(wrd);
   OWord.order= Res->WWList.nuniq;
@@ -238,14 +255,17 @@ UdmStackItemListAddWord(UDM_AGENT *query, UDM_RESULT *Res,
   OWord.match= final_word_match;
   OWord.secno= state->secno;
   OWord.phrwidth= state->dehyphenate ? 1 : 0;
+  OWord.user_weight= state->user_weight;
   
   UdmWideWordListAdd(&Res->WWList, &OWord);
   if (state->nphrasecmd % 2)
     state->phrpos++;
-      
+  state->user_weight= UDM_DEFAULT_USER_WORD_WEIGHT;
+  
   if (origin == UDM_WORD_ORIGIN_STOP)
     return UDM_OK;
-      
+  
+  /* Add generated word forms: synonyms, stemming, etc. */
   UdmWideWordListInit(&Forms);
   if (UDM_OK != (rc= UdmAllForms(query,&Forms,&OWord)))
     return rc;
@@ -265,6 +285,7 @@ UdmStackItemListAddWord(UDM_AGENT *query, UDM_RESULT *Res,
       FWord.origin = Forms.Word[frm].origin;
       FWord.match= state->next_word_match_type;
       FWord.secno= state->secno;
+      FWord.user_weight= OWord.user_weight;
       UdmWideWordListAdd(&Res->WWList,&FWord);
 /*    UdmLog(query, UDM_LOG_DEBUG, "Word form: [%d] %s", FWord.origin, wrd);*/
     }
@@ -279,6 +300,7 @@ UdmStackItemListAddWord(UDM_AGENT *query, UDM_RESULT *Res,
 
 int UdmPrepare(UDM_AGENT * query,UDM_RESULT *Res)
 {
+  UDM_VARLIST *Vars= &query->Conf->Vars;
   UDM_CHARSET * browser_cs, * local_cs, *sys_int;
   int  ctype, rc;
   int *ustr, *uend, *lt, *lex;
@@ -308,6 +330,7 @@ int UdmPrepare(UDM_AGENT * query,UDM_RESULT *Res)
   state.lang= UdmVarListFindStr(&query->Conf->Vars, "g", NULL);
   state.auto_phrase= 0;
   state.phrpos= 0;
+  state.user_weight= UDM_DEFAULT_USER_WORD_WEIGHT;
   state.SubstringMatchMinWordLength=
     (size_t) UdmVarListFindInt(&query->Conf->Vars, "SubstringMatchMinWordLength", 0);
   UniGetSepToken= state.dehyphenate ? UdmUniGetSepToken2 :UdmUniGetSepToken;
@@ -409,10 +432,16 @@ int UdmPrepare(UDM_AGENT * query,UDM_RESULT *Res)
           state.secno_match_type= lt[0];
           continue;
         }
+        else if (wlen > 10 && !strncmp(wrd, "importance", 10) &&
+                 wrd[10] >= '0' && wrd[10] <= '9')
+        {
+          state.user_weight= atoi(wrd + 10);
+          continue;
+        }
       }
       
       if (UDM_OK != (rc= UdmStackItemListAddWord(query, Res, &state,
-                                                 wlen, lt, wrd)))
+                                                 wlen, lt, uend, wrd)))
         return rc;
     }
   }
@@ -428,64 +457,16 @@ int UdmPrepare(UDM_AGENT * query,UDM_RESULT *Res)
   UDM_FREE(ustr);
   UDM_FREE(uwrd);
   UDM_FREE(wrd);
+  
+  if (UdmVarListFindBool(Vars, "ComplexSynonyms", 0))
+    UdmComplexSynonyms(query, &Res->WWList);
+  
   Res->WWList.wm= state.word_match;
   Res->WWList.strip_noaccents= state.strip_accents;
   return UDM_OK;
 }
 
 
-void UdmURLDataGroupBySite(UDM_URLDATALIST *List)
-{
-  UDM_URLDATA *src= List->Item + 1;
-  UDM_URLDATA *dst= List->Item;
-  UDM_URLDATA *end= List->Item + List->nitems;
-  uint4 count;
-  
-  if (!List->nitems)
-    return;
-
-  for(count= List->Item[0].per_site; src < end; src++)
-  {
-    /* Group by site_id */
-    if(dst->site_id == src->site_id)
-    {
-      count+= src->per_site;
-      if (dst->score > src->score)
-      {
-        continue;
-      }
-      else if (dst->score == src->score)
-      {
-        if (dst->pop_rank > src->pop_rank)
-        {
-          continue;
-        }
-        else if (dst->pop_rank == src->pop_rank)
-        {
-          if (dst->url_id < src->url_id)
-            continue;
-        }
-      }
-      dst->url_id=        src->url_id;
-      dst->score=         src->score;
-      dst->last_mod_time= src->last_mod_time;
-      dst->pop_rank=      src->pop_rank;
-      dst->url=           src->url;
-      dst->section=       src->section;
-    }
-    else
-    {
-      /* Next site */
-      dst->per_site= count;
-      *++dst= *src;
-      count= src->per_site;
-    }
-  }
-  dst->per_site= count;
-  List->nitems= dst - List->Item + 1;
-  return;
-}
-
 
 /******** Convert category string into 32 bit number *************/
 
@@ -563,3 +544,50 @@ int UdmParseQueryString(UDM_AGENT * Agent,
   UDM_FREE(qs);
   return 0;
 }
+
+
+/*
+  Applying fast limit to UDLCRDLIST.
+  Used in:
+  - UdmFindAlwaysFoundWord()
+  - UdmFindMultiWord()
+  - UdmFindWordSingle()
+  - UdmFindCrossWord()
+  - UdmMultiAddCoords()
+*/
+int
+UdmApplyFastLimit(UDM_URLCRDLIST *Coord, UDM_URLID_LIST *urls)
+{
+  UDM_URL_CRD *dst= Coord->Coords;
+  UDM_URL_CRD *src= Coord->Coords;
+  UDM_URL_CRD *srcend= Coord->Coords + Coord->ncoords;
+  
+  if (urls->exclude)
+  {
+    for ( ; src < srcend; src++)
+    {
+      if (!UdmBSearch(&src->url_id,
+                      urls->urls, urls->nurls, sizeof(urlid_t),
+                      (udm_qsort_cmp)UdmCmpURLID))
+      {
+        *dst= *src;
+        dst++;
+      }
+    }
+  }
+  else
+  {
+    for ( ; src < srcend; src++)
+    {
+      if (UdmBSearch(&src->url_id,
+                     urls->urls, urls->nurls, sizeof(urlid_t),
+                     (udm_qsort_cmp)UdmCmpURLID))
+      {
+        *dst= *src;
+        dst++;
+      }
+    }
+  }
+  Coord->ncoords= dst- Coord->Coords;
+  return UDM_OK;
+}
