@@ -59,6 +59,45 @@ cmpspell_word_and_flag(const void *a, const void *b)
 }
 
 
+static int
+detect_hashed_ispell_dict(UDM_SPELLLIST *L, int fd)
+{
+  char buf[1024*8];
+  size_t i, nbytes, nwords;
+  
+  /* QQ: Why is that? */
+  if (strlen(L->fname) + 4 >= UDM_SPELL_FILELEN)
+    return 0;
+  
+  if (!(nbytes= read(fd, buf, sizeof(buf))))
+    return 0;
+  
+  for (i= 0; i < nbytes; i++)
+  {
+    if (buf[i]== '\n')
+    {
+      L->wordlen= i + 1;
+      break;
+    }
+  }
+
+  if (L->wordlen <= 0 || L->wordlen >= 64)
+    return 0;
+
+  /* Number of words in the buffer */
+  nwords= nbytes / (L->wordlen + 1) - 1;
+
+  for (i= 1; i < nwords; i++)
+  {
+    char ch= buf[i * L->wordlen - 1];
+    if (ch != '\n')
+      return 0;
+  }
+
+  return 1;
+}
+
+
 static int
 UdmSpellListLoad(UDM_SPELLLIST *L, char *err, size_t errlen)
 {
@@ -98,29 +137,15 @@ UdmSpellListLoad(UDM_SPELLLIST *L, char *err, size_t errlen)
   }
 
   /* Detect *.msp hash file */
-  if (sizeof(tolowermap) == read(fd, tolowermap, sizeof(tolowermap)))
+  if (detect_hashed_ispell_dict(L, fd))
   {
-    for (i= 0; i < sizeof(tolowermap); i++)
-    {
-      if (tolowermap[i]== '\n')
-      {
-        L->wordlen= i + 1;
-        break;
-      }
-    }
-    if (L->wordlen > 0 && L->wordlen < 64 &&
-        tolowermap[L->wordlen*2-1] == '\n' &&
-        tolowermap[L->wordlen*3-1] == '\n' &&
-        tolowermap[L->wordlen*4-1] == '\n' &&
-        strlen(L->fname) + 4 < UDM_SPELL_FILELEN)
-    {
-      L->fmt= UDM_SPELL_FMT_HASH;
-      L->nitems= sb.st_size / L->wordlen;
-      close(fd);
-      return UDM_OK;
-    }
+    L->fmt= UDM_SPELL_FMT_HASH;
+    L->nitems= sb.st_size / L->wordlen;
+    close(fd);
+    return UDM_OK;
   }
-  
+
+
   lseek(fd, 0, SEEK_SET);
 
   if (!(L->fbody = (char*)UdmMalloc(sb.st_size+1)))
