@@ -301,26 +301,26 @@ ex1:
   }
 
   if (DBUser != NULL &&
-      CS_SUCCEED != (retcode= ct_con_props(ct->conn, CS_SET, CS_USERNAME, DBUser, CS_NULLTERM, NULL)))
+      CS_SUCCEED != (retcode= ct_con_props(ct->conn, CS_SET, CS_USERNAME, (char*) DBUser, CS_NULLTERM, NULL)))
   {
     DisplayError(db, "ct_con_props(username)");
     goto ex2;
   }
 
   if (DBPass != NULL &&
-      CS_SUCCEED != (retcode = ct_con_props(ct->conn, CS_SET, CS_PASSWORD, DBPass, CS_NULLTERM, NULL)))
+      CS_SUCCEED != (retcode = ct_con_props(ct->conn, CS_SET, CS_PASSWORD, (char*) DBPass, CS_NULLTERM, NULL)))
   {
     DisplayError(db, "ct_con_props(password)");
     goto ex2;
   }
 
-  if (CS_SUCCEED != (retcode = ct_con_props(ct->conn, CS_SET, CS_APPNAME, "indexer", CS_NULLTERM, NULL)))
+  if (CS_SUCCEED != (retcode = ct_con_props(ct->conn, CS_SET, CS_APPNAME, (char*) "indexer", CS_NULLTERM, NULL)))
   {
     DisplayError(db, "ct_con_props(appname)");
     goto ex2;
   }
 
-  if (CS_SUCCEED != (retcode = ct_connect(ct->conn, DBHost, DBHost ? CS_NULLTERM : 0)))
+  if (CS_SUCCEED != (retcode = ct_connect(ct->conn, (char*) DBHost, DBHost ? CS_NULLTERM : 0)))
   {
     DisplayError(db, "ct_connect(appname)");
     goto ex2;
@@ -662,7 +662,7 @@ UdmCTLIBQuery(UDM_DB *db, UDM_SQLRES *res, const char * query)
     goto unlock_ret;
   }
   
-  retcode = ct_command(cmd, CS_LANG_CMD, query, CS_NULLTERM, CS_UNUSED);
+  retcode = ct_command(cmd, CS_LANG_CMD, (char*) query, CS_NULLTERM, CS_UNUSED);
   if (retcode != CS_SUCCEED)
   {
     DisplayError(db, "ct_command");
@@ -768,6 +768,25 @@ UdmCTLibCommit(UDM_DB *db)
 }
 
 
+static int
+UdmCTLibRenameTable(UDM_DB *db, const char *from, const char *to)
+{
+  char buf[256];
+  udm_snprintf(buf, sizeof(buf), "sp_rename %s, %s", from, to);
+  return UdmSQLExecDirect(db, NULL, buf);
+}
+
+
+static int
+UdmCTLibCopyStructure(UDM_DB *db, const char *from, const char *to)
+{
+  char buf[256];
+  udm_snprintf(buf, sizeof(buf),
+               "SELECT * INTO %s FROM %s WHERE 1=0", to, from);
+  return UdmSQLExecDirect(db, NULL, buf);
+}
+
+
 UDM_SQLDB_HANDLER udm_sqldb_ctlib_handler =
 {
   UdmSQLEscStrGeneric,
@@ -783,7 +802,9 @@ UDM_SQLDB_HANDLER udm_sqldb_ctlib_handler =
   UdmSQLFetchRowSimple,
   UdmSQLStoreResultSimple,
   UdmSQLFreeResultSimple,
-  UdmCTLIBQuery
+  UdmCTLIBQuery,
+  UdmCTLibRenameTable,
+  UdmCTLibCopyStructure
 };
 
 #endif /* HAVE_CTLIB */
