@@ -742,7 +742,7 @@ static int short flag0= 0;
 
 
 static int
-UdmIBBind(UDM_DB *db, int position, void *data, int size, int type)
+UdmIBBind(UDM_DB *db, int position, const void *data, int size, int type)
 {
   UDM_IB *ib= (UDM_IB*) db->specific;
   XSQLVAR *var= &ib->in_sqlda.sqlvar[position - 1];
@@ -775,7 +775,9 @@ UDM_SQLDB_HANDLER udm_sqldb_ibase_handler =
   UdmSQLFetchRowSimple,
   UdmSQLStoreResultSimple,
   UdmSQLFreeResultSimple,
-  UdmIBaseQuery
+  UdmIBaseQuery,
+  NULL, /* RenameTable    */
+  NULL  /* CopyStrucuture */
 };
 
 #endif
