@@ -93,7 +93,9 @@ UdmMySQLConnect(UDM_DB *db)
   int DBPort= UdmVarListFindInt(&db->Vars, "DBPort", 0);
   const char* setnames=  UdmVarListFindStr(&db->Vars,"setnames", "");
   int opt_compress= UdmVarListFindBool(&db->Vars, "compress", 0);
+  int opt_client_multi_statements= UdmVarListFindBool(&db->Vars, "ClientMultiStatements", 0);
   UDM_MYSQL *mydb= (UDM_MYSQL*) UdmMalloc(sizeof(UDM_MYSQL));
+  unsigned long client_flag= 0;
   
   if (!(db->specific= mydb))
   {
@@ -116,10 +118,14 @@ UdmMySQLConnect(UDM_DB *db)
   
   if (opt_compress)
     mysql_options(&mydb->mysql, MYSQL_OPT_COMPRESS, 0);
-  
+
+#ifdef CLIENT_MULTI_STATEMENTS
+  client_flag |= opt_client_multi_statements ? CLIENT_MULTI_STATEMENTS : 0;
+#endif
+
   if(!(mysql_real_connect(&mydb->mysql, DBHost, DBUser, DBPass,
   			  db->DBName?db->DBName:"mnogosearch",
-  			  (unsigned)DBPort, DBSock, 0)))
+  			  (unsigned)DBPort, DBSock, client_flag)))
   {
     db->errcode=1;
     sprintf(db->errstr, "MySQL driver: #%d: %s",
@@ -152,6 +158,20 @@ UdmMySQLConnect(UDM_DB *db)
       }
     }
   }
+  else
+  {
+#ifdef HAVE_MYSQL_PS
+    /* Switch from native to general PS API */
+    if (db->sql->SQLPrepare)
+    {
+      db->sql->SQLPrepare=  UdmSQLPrepareGeneric;
+      db->sql->SQLBind=     UdmSQLBindGeneric;
+      db->sql->SQLExec=     UdmSQLExecGeneric;
+      db->sql->SQLStmtFree= UdmSQLStmtFreeGeneric;
+    }
+#endif
+  }
+  
   if (DBLog)
   {
     char qbuf[64];
@@ -311,9 +331,31 @@ UdmMySQLQuery(UDM_DB *db,UDM_SQLRES *R,const char *query)
   int rc= UdmMySQLExecDirect(db, R, query);
   if (rc != UDM_OK)
     return rc;
-  if (R->specific)
-    return UdmMySQLStoreResult(db, R);
-  return UDM_OK;
+
+  if (!R->specific)
+    return UDM_OK;
+  
+  rc= UdmMySQLStoreResult(db, R);
+
+#if (MYSQL_VERSION_ID > 40100)
+  {
+    UDM_MYSQL *mydb= (UDM_MYSQL*) db->specific;
+    while (mysql_next_result(db->specific) == 0)
+    {
+      MYSQL_RES *res;
+      if ((res= mysql_store_result(&mydb->mysql)))
+      {
+        mysql_free_result(res);
+      }
+      else
+      {
+        /* Non-select query */
+      }
+    }
+  }
+#endif
+
+  return rc;
 }
 
 
@@ -342,11 +384,10 @@ UdmMySQLCommit(UDM_DB *db)
 }
 
 
-static char*
+static size_t
 UdmMySQLEscStr(UDM_DB *db, char *to, const char *from, size_t len)
 {
-  mysql_escape_string(to, from, len);
-  return(to);
+  return mysql_escape_string(to, from, len);
 }
 
 
@@ -414,7 +455,7 @@ UdmSQLType2MySQLType(int type)
 
 
 static int
-UdmMySQLBind(UDM_DB *db, int position, void *data, int size, int type)
+UdmMySQLBind(UDM_DB *db, int position, const void *data, int size, int type)
 {
   UDM_MYSQL *mydb= (UDM_MYSQL*) db->specific;
   MYSQL_BIND *b= &mydb->bind[position - 1];
@@ -471,6 +512,26 @@ UdmMySQLExec(UDM_DB *db)
 #endif
 
 
+static int
+UdmMySQLRenameTable(UDM_DB *db, const char *from, const char *to)
+{
+  char buf[256];
+  udm_snprintf(buf, sizeof(buf), "ALTER TABLE %s RENAME TO %s", from, to);
+  return UdmSQLExecDirect(db, NULL, buf);
+}
+
+
+static int
+UdmMySQLCopyStructure(UDM_DB *db, const char *from, const char *to)
+{
+  char buf[256];
+  udm_snprintf(buf, sizeof(buf),
+              "CREATE TABLE %s MAX_ROWS=300000000 AVG_ROW_LENGTH=512 "
+              "SELECT * FROM %s LIMIT 0", to, from);
+  return UdmSQLExecDirect(db, NULL, buf);
+}
+
+
 UDM_SQLDB_HANDLER udm_sqldb_mysql_handler =
 {
   UdmMySQLEscStr,
@@ -493,6 +554,8 @@ UDM_SQLDB_HANDLER udm_sqldb_mysql_handler =
   UdmMySQLFetchRow,
   UdmSQLStoreResultSimple,
   UdmMySQLFreeResult,
-  UdmMySQLExecDirect
+  UdmMySQLExecDirect,
+  UdmMySQLRenameTable,
+  UdmMySQLCopyStructure
 };
 #endif
