@@ -410,7 +410,7 @@ UdmODBCConnect(UDM_DB *db)
   if (SQL_SUCCESS != (db->errcode= SQLAllocEnv(&sdb->hEnv)) ||
       SQL_SUCCESS != (db->errcode= SQLAllocConnect(sdb->hEnv, &sdb->hDbc)) ||
       SQL_SUCCESS != (db->errcode= SQLSetConnectOption(sdb->hDbc, SQL_AUTOCOMMIT, SQL_AUTOCOMMIT_ON)) ||
-      !SQL_OK(db->errcode = SQLConnect(sdb->hDbc, (SQLCHAR *)DSN, SQL_NTS, DBUser, SQL_NTS, DBPass, SQL_NTS)))
+      !SQL_OK(db->errcode = SQLConnect(sdb->hDbc, (SQLCHAR *)DSN, SQL_NTS, (SQLCHAR*) DBUser, SQL_NTS, (SQLCHAR*) DBPass, SQL_NTS)))
     return UDM_ERROR;
 
   db->errcode= 0;
@@ -467,7 +467,7 @@ ret:
 
 
 static int
-UdmODBCQuery(UDM_DB *db,UDM_SQLRES *res, const char *qbuf)
+UdmODBCExecDirect(UDM_DB *db,UDM_SQLRES *res, const char *qbuf)
 {
   int rc;
    
@@ -558,6 +558,7 @@ UdmODBCBegin(UDM_DB *db)
   db->errcode= SQLSetConnectOption(sdb->hDbc, SQL_AUTOCOMMIT, SQL_AUTOCOMMIT_OFF);
   if(SQL_SUCCESS != db->errcode)
   {
+    sdb->hstmt= SQL_NULL_HSTMT;
     UdmODBCDisplayError(db,"SQLSetConnectOption(SQL_AUTOCOMMIT_OFF) failed");
     return UDM_ERROR;
   }
@@ -570,11 +571,16 @@ UdmODBCCommit(UDM_DB *db)
 {
   UDM_ODBC_CONN *sdb= (UDM_ODBC_CONN*) db->specific;
   int rc;
-  if (UDM_OK!=(rc=UdmODBCQuery(db,NULL,"COMMIT")))
+  if (UDM_OK!=(rc=UdmODBCExecDirect(db,NULL,"COMMIT")))
     return UDM_ERROR;
-  db->errcode= SQLSetConnectOption(sdb->hDbc, SQL_AUTOCOMMIT, SQL_AUTOCOMMIT_ON);
-  if(SQL_SUCCESS != db->errcode)
+  if (!SQL_OK(db->errcode= SQLSetConnectOption(sdb->hDbc,
+                                               SQL_AUTOCOMMIT,
+                                               SQL_AUTOCOMMIT_ON)))
+  {
+    sdb->hstmt= SQL_NULL_HSTMT;
+    UdmODBCDisplayError(db,"SQLSetConnectOption(SQL_AUTOCOMMIT_ON) failed");
     return UDM_ERROR;
+  }
   return UDM_OK;
 }
 
@@ -594,7 +600,7 @@ UdmODBCPrepare(UDM_DB *db, const char *query)
     UdmODBCDisplayError(db,"UdmODBCPrepare: ");
     return(UDM_ERROR);
   }
-  rc= SQLPrepare(sdb->hstmt, query, SQL_NTS);
+  rc= SQLPrepare(sdb->hstmt, (SQLCHAR*) query, SQL_NTS);
   if (!SQL_OK(rc))
   {
     db->errcode = rc;
@@ -634,12 +640,12 @@ UdmSQLParamLen2ODBC(int udm_len)
 
 
 static int
-UdmODBCBind(UDM_DB *db, int position, void *data, int size, int type)
+UdmODBCBind(UDM_DB *db, int position, const void *data, int size, int type)
 {
   UDM_ODBC_CONN *sdb= (UDM_ODBC_CONN*) db->specific;
   int rc;
   BindBuf[position].size= UdmSQLParamLen2ODBC(size);
-  BindBuf[position].data= data;
+  BindBuf[position].data= (char*) data;
   
   type= UdmSQLType2ODBCType(type);
   db->errcode = 0;
@@ -647,7 +653,7 @@ UdmODBCBind(UDM_DB *db, int position, void *data, int size, int type)
 
   rc= SQLBindParameter(sdb->hstmt, position, SQL_PARAM_INPUT, SQL_C_DEFAULT,
                        type, 0 /* Column size */, 0 /* Decimal digits */,
-                       /*&BindBuf[position]*/ data, size < 0 ? 0 : size,
+                       /*&BindBuf[position]*/ (char*) data, size < 0 ? 0 : size,
                        &BindBuf[position].size);
 
 #ifdef WIN32  
@@ -711,10 +717,68 @@ check_err:
 }
 
 
+static int
+UdmODBCRenameTable(UDM_DB *db, const char *from, const char *to)
+{
+  if (db->DBType == UDM_DB_MYSQL ||
+      db->DBType == UDM_DB_PGSQL ||
+      db->DBType == UDM_DB_ORACLE8)
+  {
+    char buf[256];
+    udm_snprintf(buf, sizeof(buf), "ALTER TABLE %s RENAME TO %s", from, to);
+    return UdmSQLExecDirect(db, NULL, buf);
+  }
+  else if (db->DBType == UDM_DB_DB2)
+  {
+    char buf[256];
+    udm_snprintf(buf, sizeof(buf), "RENAME TABLE %s TO %s", from, to);
+    return UdmSQLExecDirect(db, NULL, buf);
+  }
+  sprintf(db->errstr, "This database type does not support RENAME TABLE");
+  return UDM_ERROR;
+}
+
+
+/*
+  Copy table structure from another table, without indexes.
+*/
+static int
+UdmODBCCopyStructure(UDM_DB *db, const char *from, const char *to)
+{
+  char buf[256];
+  switch (db->DBType)
+  {
+    case UDM_DB_MYSQL:
+      udm_snprintf(buf, sizeof(buf),
+                   "CREATE TABLE %s MAX_ROWS=300000000 AVG_ROW_LENGTH=512 "
+                   "SELECT * FROM %s LIMIT 0", to, from);
+      return UdmSQLExecDirect(db, NULL, buf);
+
+    case UDM_DB_PGSQL:
+      udm_snprintf(buf, sizeof(buf), "CREATE TABLE %s (LIKE %s)", to, from);
+      return UdmSQLExecDirect(db, NULL, buf);
+
+    case UDM_DB_ORACLE8:
+      udm_snprintf(buf, sizeof(buf),
+                   "CREATE TABLE %s AS SELECT * FROM %s WHERE 1=0", to, from);
+      return UdmSQLExecDirect(db, NULL, buf);
+
+    case UDM_DB_DB2:
+      udm_snprintf(buf, sizeof(buf), "CREATE TABLE %s LIKE %s", to, from);
+      return UdmSQLExecDirect(db, NULL, buf);
+
+    default: break;
+  }
+  sprintf(db->errstr, "This database type does not support"
+          "<CREATE TABLE name LIKE> or <CREATE TABLE name AS SELECT>");
+  return UDM_ERROR;
+}
+
+
 UDM_SQLDB_HANDLER udm_sqldb_odbc_handler =
 {
   UdmSQLEscStrGeneric,
-  UdmODBCQuery,
+  UdmODBCExecDirect,
   UdmODBCConnect,
   UdmODBCClose,
   UdmODBCBegin,
@@ -726,7 +790,9 @@ UDM_SQLDB_HANDLER udm_sqldb_odbc_handler =
   UdmSQLFetchRowSimple,
   UdmSQLStoreResultSimple,
   UdmSQLFreeResultSimple,
-  UdmODBCQuery
+  UdmODBCExecDirect,
+  UdmODBCRenameTable,
+  UdmODBCCopyStructure
 };
 
 #endif /* HAVE_ODBC */
