@@ -526,7 +526,7 @@ stmt_free:
 
 
 static int
-UdmOraQuery(UDM_DB *db, UDM_SQLRES *res, const char *qbuf)
+UdmOraExecDirect(UDM_DB *db, UDM_SQLRES *res, const char *qbuf)
 {
   int rc;
   
@@ -608,7 +608,7 @@ UdmOraPrepare(UDM_DB *db, const char *query)
 
 
 static int
-UdmOraBind(UDM_DB *db, int position, void *data, int size, int type)
+UdmOraBind(UDM_DB *db, int position, const void *data, int size, int type)
 {
   int rc;
   UDM_ORACLE_DB *ob= (UDM_ORACLE_DB *)db->specific;
@@ -660,7 +660,7 @@ UdmOraExec(UDM_DB *db)
 static int
 UdmOraBegin(UDM_DB *db)
 {
-  int rc= UdmOraQuery(db,NULL,"COMMIT");
+  int rc= UdmOraExecDirect(db,NULL,"COMMIT");
   db->commit_fl= 1;
   return rc;
 }
@@ -669,16 +669,36 @@ UdmOraBegin(UDM_DB *db)
 static int
 UdmOraCommit(UDM_DB *db)
 {
-  int rc= UdmOraQuery(db,NULL,"COMMIT");
+  int rc= UdmOraExecDirect(db,NULL,"COMMIT");
   db->commit_fl= 0;
   return rc;
 }
 
 
+static int
+UdmOraRenameTable(UDM_DB *db, const char *from, const char *to)
+{
+  char buf[256];
+  /* "RENAME t1 TO t2" also works */
+  udm_snprintf(buf, sizeof(buf), "ALTER TABLE %s RENAME TO %s", from, to);
+  return UdmSQLExecDirect(db, NULL, buf);
+}
+
+
+static int
+UdmOraCopyStructure(UDM_DB *db, const char *from, const char *to)
+{
+  char buf[256];
+  udm_snprintf(buf, sizeof(buf),
+               "CREATE TABLE %s AS SELECT * FROM %s WHERE 1=0", to, from);
+  return UdmSQLExecDirect(db, NULL, buf);
+}
+
+
 UDM_SQLDB_HANDLER udm_sqldb_oracle_handler =
 {
   UdmSQLEscStrGeneric,
-  UdmOraQuery,
+  UdmOraExecDirect,
   UdmOraConnect,
   UdmOraClose,
   UdmOraBegin,
@@ -690,7 +710,9 @@ UDM_SQLDB_HANDLER udm_sqldb_oracle_handler =
   UdmSQLFetchRowSimple,
   UdmSQLStoreResultSimple,
   UdmSQLFreeResultSimple,
-  UdmOraQuery
+  UdmOraExecDirect,
+  UdmOraRenameTable,
+  UdmOraCopyStructure
 };
 
 #endif
