@@ -97,7 +97,7 @@
 static char udm_hex_digits[]= "0123456789ABCDEF";
 
 
-void DecodeHexStr (char *src, UDM_PSTR *dst, size_t size)
+void DecodeHexStr (const char *src, UDM_PSTR *dst, size_t size)
 {
   char p1;
   char p2;
@@ -183,7 +183,7 @@ UdmSQLDropTableIfExists(UDM_DB *db, const char *name)
 {
   char qbuf[128];
   int rc;
-  int have_if_exists= (db->DBType == UDM_DB_MYSQL);
+  int have_if_exists= (db->flags & UDM_SQL_HAVE_DROP_IF_EXISTS);
   const char *if_exists= have_if_exists ? "IF EXISTS " : "";
   if (!have_if_exists)
     db->flags|= UDM_SQL_IGNORE_ERROR;
@@ -194,8 +194,6 @@ UdmSQLDropTableIfExists(UDM_DB *db, const char *name)
 
   /*
     TODO:
-    PostgreSQL:
-      DROP TABLE IF EXISTS t1; -- starting from Pg-8.2 (checked with 8.2.4)
 
     MSSQL:
     IF OBJECT_ID(N'tempdb..#temptable', N'U') IS NOT NULL 
@@ -246,7 +244,7 @@ UdmSQLTableTruncateOrDelete(UDM_DB *db, const char *name)
 }
 
 
-char *
+size_t
 UdmSQLEscStrGeneric(UDM_DB *db, char *to,const char *from,size_t len)
 {
   int DBType= db->DBType;
@@ -309,18 +307,51 @@ UdmSQLEscStrGeneric(UDM_DB *db, char *to,const char *from,size_t len)
       to++;from++;
     }
   }
-  *to=0;return(s);
+  *to=0;
+  return to - s;
+}
+
+
+size_t
+UdmSQLEscStr(UDM_DB *db, char *to, const char *from, size_t len)
+{
+  UDM_ASSERT(from);
+  UDM_ASSERT(to);
+  return db->sql->SQLEscStr(db, to, from, len);
+}
+
+
+static int
+UdmSQLEscStrLength(UDM_DB *db, size_t srclen)
+{
+  return ((db->DBType == UDM_DB_PGSQL) ? 4 : 2) * srclen + 1;
 }
 
 
 char *
-UdmSQLEscStr(UDM_DB *db, char *to, const char *from, size_t len)
+UdmSQLEscStrAlloc(UDM_DB *db, const char *src, size_t srclen)
 {
-  size_t multiply= db->DBType == UDM_DB_PGSQL ? 4 : 2;
-  if(!from)return(NULL);
-  if(!to)to=(char*)UdmMalloc(len * multiply + 1);
-  if(!to)return(NULL);
-  return db->sql->SQLEscStr(db, to, from, len);
+  char *dst;
+  if(!src ||
+     !(dst= (char*) UdmMalloc(UdmSQLEscStrLength(db, srclen))))
+    return NULL;
+  db->sql->SQLEscStr(db, dst, src, srclen);
+  return dst;
+}
+
+
+/*
+  SQL-Escape string to DSTR
+  dstr - a pointer to a initialized DSTR
+*/
+int
+UdmSQLEscDSTR(UDM_DB *db, UDM_DSTR *dstr, const char *src, size_t srclen)
+{
+  int rc;
+  if (UDM_OK != (rc= UdmDSTRAlloc(dstr, UdmSQLEscStrLength(db, srclen))))
+    return rc;
+  dstr->size_data= db->sql->SQLEscStr(db, dstr->data, src, srclen);
+  return rc;
 }
 
 
@@ -398,7 +429,7 @@ UdmSQLBinEscCharForPg(char *dst, unsigned int ch)
 
 
 size_t
-UdmSQLBinEscStr(UDM_DB *db, char *dst, const char *src0, size_t len)
+UdmSQLBinEscStr(UDM_DB *db, char *dst, size_t dstlen, const char *src0, size_t len)
 {
   const unsigned char *src= (const unsigned char*) src0;
   UDM_ASSERT(dst != NULL);
@@ -420,17 +451,23 @@ UdmSQLBinEscStr(UDM_DB *db, char *dst, const char *src0, size_t len)
 int
 UdmSQLBeginDebug(UDM_DB *db, const char *file, const int line)
 {
+  udm_timer_t ticks= UdmStartTimer(); 
+  int rc= db->sql->SQLBegin(db);
   if (db->flags & UDM_SQL_DEBUG_QUERY)
-    fprintf(stderr, "[%d] SQLBegin {%s:%d}\n", getpid(), file, line);
-  return db->sql->SQLBegin(db);
+    fprintf(stderr, "[%d] %.2f SQLBegin {%s:%d}\n",
+            getpid(), UdmStopTimer(&ticks)/1000, file, line);
+  return rc;
 }
 
 int
 UdmSQLCommitDebug(UDM_DB *db, const char *file, const int line)
 {
+  udm_timer_t ticks= UdmStartTimer();
+  int rc= db->sql->SQLCommit(db);
   if (db->flags & UDM_SQL_DEBUG_QUERY)
-    fprintf(stderr, "[%d] SQLCommit {%s:%d}\n", getpid(), file, line);
-  return db->sql->SQLCommit(db);
+    fprintf(stderr, "[%d] %.2f SQLCommit {%s:%d}\n",
+            getpid(), UdmStopTimer(&ticks)/1000, file, line);
+  return rc;
 }
 int
 UdmSQLExecDirectDebug(UDM_DB *db, UDM_SQLRES *R, const char *query,
@@ -439,11 +476,10 @@ UdmSQLExecDirectDebug(UDM_DB *db, UDM_SQLRES *R, const char *query,
   if (db->flags & UDM_SQL_DEBUG_QUERY)
   {
     int rc;
-    unsigned long long ticks= UdmStartTimer();
+    udm_timer_t ticks= UdmStartTimer();
     rc= db->sql->SQLExecDirect(db, R, query);
-    ticks= UdmStartTimer() - ticks;
-    fprintf(stderr,"[%d] %.2fs SQL {%s:%d}: %s\n",
-            getpid(), (float)ticks/1000, file, line, query);
+    fprintf(stderr,"[%d] %.2f SQL {%s:%d}: %s\n",
+            getpid(), UdmStopTimer(&ticks), file, line, query);
     return rc;
   }
   else
@@ -465,10 +501,11 @@ int
 UdmSQLExecuteDebug(UDM_DB *db,
                    const char* file, const int line)
 {
-  int rc;
-  rc= db->sql->SQLExec(db);
+  udm_timer_t ticks= UdmStartTimer();
+  int rc= db->sql->SQLExec(db);
   if (db->flags & UDM_SQL_DEBUG_QUERY)
-    fprintf(stderr, "[%d] Execute {%s:%d}\n", getpid(), file, line);
+    fprintf(stderr, "[%d] %.2f Execute {%s:%d}\n",
+            getpid(), UdmStopTimer(&ticks), file, line);
   return rc;
 }
 
@@ -501,9 +538,9 @@ UdmSQLTypeToStr(int type)
 
 
 static void
-UdmSQLPrintParameter(FILE *file, void *data, int size)
+UdmSQLPrintParameter(FILE *file, const void *data, int size)
 {
-  const unsigned char *s= (unsigned char*) data;
+  const unsigned char *s= (const unsigned char*) data;
   const unsigned char *e= s + size;
   fprintf(stderr, "'");
   for ( ; s < e; s++)
@@ -518,7 +555,7 @@ UdmSQLPrintParameter(FILE *file, void *data, int size)
 
 int
 UdmSQLBindParameterDebug(UDM_DB *db,
-                         int pos, void *data, int size, int type,
+                         int pos, const void *data, int size, int type,
                          const char *file, const int line)
 {
   if (db->flags & UDM_SQL_DEBUG_QUERY)
@@ -530,7 +567,32 @@ UdmSQLBindParameterDebug(UDM_DB *db,
   }
   return db->sql->SQLBind(db, pos, data, size, type);
 }
-#else
+
+int
+UdmSQLRenameTableDebug(UDM_DB *db, const char *from, const char *to,
+                       const char *file, const int line)
+{
+  if (db->flags & UDM_SQL_DEBUG_QUERY)
+  {
+    fprintf(stderr, "[%d] RenameTable {%s:%d} '%s' to '%s'\n",
+            getpid(), file, line, from, to);
+  }
+  return db->sql->SQLRenameTable(db, from, to);
+}
+
+int
+UdmSQLCopyStructureDebug(UDM_DB *db, const char *from, const char *to,
+                         const char *file, const int line)
+{
+  if (db->flags & UDM_SQL_DEBUG_QUERY)
+  {
+    fprintf(stderr, "[%d] CopyStructure {%s:%d} '%s' to '%s'\n",
+            getpid(), file, line, from, to);
+  }
+  return db->sql->SQLCopyStructure(db, from, to);
+}
+
+#else /* not HAVE_DEBUG */
 int
 UdmSQLBegin(UDM_DB *db)
 {
@@ -568,11 +630,23 @@ UdmSQLStmtFree(UDM_DB *db)
 }
 
 int
-UdmSQLBindParameter(UDM_DB *db, int pos, void *data, int size, int type)
+UdmSQLBindParameter(UDM_DB *db, int pos, const void *data, int size, int type)
 {
   return db->sql->SQLBind(db, pos, data, size, type);
 }
-#endif
+
+int
+UdmSQLRenameTable(UDM_DB *db, const char *from, const char *to)
+{
+  return db->sql->SQLRenameTable(db, from, to);
+}
+
+int
+UdmSQLCopyStructure(UDM_DB *db, const char *from, const char *to)
+{
+  return db->sql->SQLCopyStructure(db, from, to);
+}
+#endif /* HAVE_DEBUG */
 
 
 int __UDMCALL
@@ -582,7 +656,7 @@ _UdmSQLQuery(UDM_DB *db, UDM_SQLRES *SQLRes, const char * query,
   UDM_SQLRES res;
   
 #ifdef HAVE_DEBUG
-  unsigned long ticks;
+  udm_timer_t ticks;
   if (db->flags & UDM_SQL_DEBUG_QUERY)
   {
     ticks= UdmStartTimer();
@@ -597,8 +671,8 @@ _UdmSQLQuery(UDM_DB *db, UDM_SQLRES *SQLRes, const char * query,
 #ifdef HAVE_DEBUG
   if (db->flags & UDM_SQL_DEBUG_QUERY)
   {
-    ticks= UdmStartTimer() - ticks;
-    fprintf(stderr,"[%d] %.2fs SQL {%s:%d}: %s\n", getpid(), (float)ticks/1000, file, line, query);
+    fprintf(stderr,"[%d] %.2f SQL {%s:%d}: %s\n",
+            getpid(), UdmStopTimer(&ticks), file, line, query);
   }
 #endif
   
@@ -883,7 +957,7 @@ UdmSQLPrepareGeneric(UDM_DB *db, const char *query)
 
 
 int
-UdmSQLBindGeneric(UDM_DB *db, int pos, void *data, int size, int type)
+UdmSQLBindGeneric(UDM_DB *db, int pos, const void *data, int size, int type)
 {
   UDM_PS *ps= (UDM_PS*) db->ps;
   UDM_ASSERT(ps->sql);
@@ -928,7 +1002,7 @@ prepared_query_length(const char *sql, UDM_PS *param)
 
 
 static size_t
-prepared_query_add_params(UDM_DB *db, char *dst,
+prepared_query_add_params(UDM_DB *db, char *dst, size_t dstlen,
                           UDM_PS *param, size_t param_num)
 {
   char *dst0= dst;
@@ -998,7 +1072,7 @@ prepared_query_add_params(UDM_DB *db, char *dst,
           *dst++= 'E';
         *dst++= '\'';
         if (is_bin)
-          UdmSQLBinEscStr(db, dst, src, srclen); /* TODO: get rid of strlen below*/
+          UdmSQLBinEscStr(db, dst, dstlen, src, srclen); /* TODO: get rid of strlen below*/
         else
           UdmSQLEscStr(db, dst, src, srclen);
         dst+= strlen(dst);
@@ -1038,7 +1112,7 @@ UdmSQLExecGeneric(UDM_DB *db)
   {
     if (*src == '?')
     {
-      size_t len= prepared_query_add_params(db, dst, ps, param_num);
+      size_t len= prepared_query_add_params(db, dst, qlen, ps, param_num);
       param_num++;
       dst+= len;
     }
