@@ -196,7 +196,7 @@ static int
   UDM_CHARSET *dcs= UdmGetCharSet(UdmVarListFindStr(&Doc->Sections, "Charset", "iso-8859-1"));
   const char *CachedCopy= UdmVarListFindStr(&Doc->Sections, "CachedCopy", NULL);
 #ifdef EXTRA_DEBUG
-  unsigned long long ticks UdmStartTimer();
+  udm_timer_t ticks UdmStartTimer();
 #endif
   
   if (!CachedCopy || !dcs || !(Source = UdmMalloc(UDM_MAXDOCSIZE)))
@@ -229,8 +229,7 @@ static int
   UdmFree(in_buf);
 
 #ifdef EXTRA_DEBUG
-  ticks= UdmStartTimer() - ticks;
-  UdmLog(Agent, UDM_LOG_DEBUG, "Inflate: %.5f", (float) ticks/1000);
+  UdmLog(Agent, UDM_LOG_DEBUG, "Inflate: %.5f", UdmStopTimer(&ticks));
 #endif
 
 #ifdef EXTRA_DEBUG
@@ -246,8 +245,7 @@ static int
   UdmFree(Source);
 
 #ifdef EXTRA_DEBUG
-  ticks= UdmStartTimer() - ticks;
-  UdmLog(Agent, UDM_LOG_DEBUG, "Convert: %.5f", (float) ticks/1000);
+  UdmLog(Agent, UDM_LOG_DEBUG, "Convert: %.5f", UdmStopTimer(&ticks));
 #endif
   
   return (int*) dst;
@@ -408,8 +406,7 @@ UdmExcerptDoc (UDM_AGENT *Agent,
   ul = j;
 
 #ifdef EXTRA_DEBUG
-  ticks= UdmStartTimer() - ticks;
-  UdmLog(Agent, UDM_LOG_ERROR, "GetSour: %.5f ul=%d", (float) ticks / 1000, ul);
+  UdmLog(Agent, UDM_LOG_ERROR, "GetSour: %.5f ul=%d", UdmStopTimer(&ticks), ul);
 #endif  
 
 #ifdef EXTRA_DEBUG
@@ -517,8 +514,7 @@ UdmExcerptDoc (UDM_AGENT *Agent,
   res[l]= 0;
 
 #ifdef EXTRA_DEBUG
-  ticks= UdmStartTimer() - ticks;
-  UdmLog(Agent, UDM_LOG_ERROR, "Excerpt: %.5f l=%d", (float) ticks / 1000, l);
+  UdmLog(Agent, UDM_LOG_ERROR, "Excerpt: %.5f l=%d", UdmStopTimer(&ticks), l);
 #endif  
 
   return res;
