@@ -38,12 +38,38 @@ void UdmSynonymListInit(UDM_SYNONYMLIST * List)
 }
 
 
+/*
+  Returns the number of separators in a complex word.
+  Needed for complex synonyms.
+  
+  TODO: currently doesn't work with extra spaces: "aaa   bbb"
+*/
+size_t
+UdmMultiWordPhraseLength(const char *s)
+{
+  size_t res;
+  for (res= 0; *s; s++)
+  {
+    if (*s == ' ')
+      res++;
+  }
+  return res;
+}
+
 static void
-AddSynonym(UDM_SYNONYMLIST *Syn, char *first, char *second)
+AddSynonym(UDM_SYNONYMLIST *Syn, char *first, char *second, int origin)
 {
   UDM_SYNONYM *trg= &Syn->Synonym[Syn->nsynonyms];
+  size_t phrase_length= UdmMultiWordPhraseLength(first);
+  if (Syn->max_phrase_length < phrase_length)
+    Syn->max_phrase_length= phrase_length;
   trg->p= strdup(first);
   trg->s= strdup(second);
+  /*
+    We don't allow multi-word synonyms to get into loop
+    at this point - for performance purposes.
+  */
+  trg->origin= phrase_length ? UDM_WORD_ORIGIN_SYNONYM_FINAL : origin;
   Syn->nsynonyms++;
 }
 
@@ -61,8 +87,10 @@ __C_LINK int __UDMCALL UdmSynonymListLoad(UDM_ENV * Env,const char * filename)
   UDM_CHARSET  *cs=NULL;
   UDM_CONV     file_lcs;
   int          mode= UDM_SYN_MODE_ROUNDTRIP;
+  int          origin= UDM_WORD_ORIGIN_SYNONYM;
   UDM_UNIDATA  *unidata= Env->unidata;
   UDM_SYNONYMLIST Synonyms;
+  size_t       lineno= 0;
   
   UdmSynonymListInit(&Synonyms);
   
@@ -72,7 +100,7 @@ __C_LINK int __UDMCALL UdmSynonymListLoad(UDM_ENV * Env,const char * filename)
     return UDM_ERROR;
   }
   
-  while(fgets(str,sizeof(str),f))
+  for (lineno= 1; fgets(str,sizeof(str),f); lineno++)
   {
     if(str[0]=='#'||str[0]==' '||str[0]=='\t'||str[0]=='\r'||str[0]=='\n')continue;
     
@@ -102,7 +130,8 @@ __C_LINK int __UDMCALL UdmSynonymListLoad(UDM_ENV * Env,const char * filename)
     else if (!strncasecmp(str, "Mode:", 5))
     {
       char *lasttok, *l;
-      if ((l= udm_strtok_r(str + 5, " \t\n\r", &lasttok)))
+      for (l= udm_strtok_r(str + 5, " ,\t\n\r", &lasttok) ; l;
+           l= udm_strtok_r(NULL, " ,\t\n\r", &lasttok))
       {
         if (!strcasecmp(l, "oneway"))
         {
@@ -119,11 +148,25 @@ __C_LINK int __UDMCALL UdmSynonymListLoad(UDM_ENV * Env,const char * filename)
           mode= UDM_SYN_MODE_RETURN;
           continue;
         }
+        else if (!strcasecmp(l, "recursive"))
+        {
+          origin= UDM_WORD_ORIGIN_SYNONYM;
+          continue;
+        }
+        else if (!strcasecmp(l, "final"))
+        {
+          origin= UDM_WORD_ORIGIN_SYNONYM_FINAL;
+          continue;
+        }
+        else
+        {
+          udm_snprintf(Env->errstr, sizeof(Env->errstr),
+                       "Bad Mode command in synonym file %s:%d",
+                       filename, lineno);
+          fclose(f);
+          return UDM_ERROR;
+        }
       }
-      udm_snprintf(Env->errstr, sizeof(Env->errstr),
-                   "Bad Mode command in synonym file '%s'", filename);
-      fclose(f);
-      return UDM_ERROR;
     }
     else
     {
@@ -167,11 +210,11 @@ __C_LINK int __UDMCALL UdmSynonymListLoad(UDM_ENV * Env,const char * filename)
 
           /* Add direct order */
           if (mode == UDM_SYN_MODE_ONEWAY || mode == UDM_SYN_MODE_ROUNDTRIP)
-            AddSynonym(&Synonyms, av[i], av[j]);
+            AddSynonym(&Synonyms, av[i], av[j], origin);
           
           /* Add reverse order */
           if (mode == UDM_SYN_MODE_RETURN || mode == UDM_SYN_MODE_ROUNDTRIP)
-            AddSynonym(&Synonyms, av[j], av[i]);
+            AddSynonym(&Synonyms, av[j], av[i], origin);
         }
       }
     }
@@ -210,13 +253,54 @@ __C_LINK void __UDMCALL UdmSynonymListSort(UDM_SYNONYMLIST * List)
 }
 
 
-static UDM_WIDEWORDLIST *
-UdmSynonymListFind(UDM_WIDEWORDLIST *Res,
-                   const UDM_SYNONYMLIST *List, UDM_WIDEWORD *wword)
+static void
+UdmWideWordListAddSynonym(UDM_WIDEWORDLIST *Res,
+                          const UDM_WIDEWORD *pattern,
+                          const UDM_SYNONYM *syn)
 {
-  UDM_SYNONYM syn, *res, *first, *last;
+  UDM_WIDEWORD ww= *pattern;
+  ww.origin= syn->origin;
+  UdmWideWordListAddLike(Res, &ww, syn->s);
+}
+
+
+
+/*
+  Traverse through a synonym list starting
+  from "medium" towards the beginning or the
+  end of the list and adding all synonyms
+  equal to "word".
+  "medium" must be a valid pointer to some word
+  in the list.
+*/
+static void
+UdmWideWordListAddSynonymIterate(UDM_WIDEWORDLIST *Res,
+                                 const UDM_SYNONYMLIST *List,
+                                 const UDM_SYNONYM *medium,
+                                 const UDM_WIDEWORD *pattern,
+                                 const char *word,
+                                 int direction)
+{
+  UDM_SYNONYM *first= List->Synonym;
+  UDM_SYNONYM *last= List->Synonym + List->nsynonyms;
+  for ( ; medium >= first && medium < last; medium+= direction)
+  {
+    if (strcmp(word, medium->p))
+      break;
+    UdmWideWordListAddSynonym(Res, pattern, medium);
+  }
+}
+
+
+UDM_WIDEWORDLIST *
+UdmSynonymListFind(UDM_WIDEWORDLIST *Res,
+                   const UDM_SYNONYMLIST *List,
+                   const UDM_WIDEWORD *wword)
+{
+  UDM_SYNONYM syn, *res;
   
-  if(!List->nsynonyms)
+  /* Quickly skip empty lists, and skip final synonyms */
+  if (!List->nsynonyms || wword->origin == UDM_WORD_ORIGIN_SYNONYM_FINAL)
     return NULL;
 
   syn.p= wword->word;
@@ -224,47 +308,28 @@ UdmSynonymListFind(UDM_WIDEWORDLIST *Res,
   if((res= UdmBSearch(&syn,List->Synonym,List->nsynonyms,sizeof(UDM_SYNONYM),&cmpsyn)))
   {
     size_t nnorm,i;
-    UDM_WIDEWORD ww= wword[0];
-    ww.origin= UDM_WORD_ORIGIN_SYNONYM;
     
     /* Find first and last synonym */
-    for(first= res;
-        first >= List->Synonym && !strcmp(wword->word, first->p);
-        first--)
-    {
-      UdmWideWordListAddLike(Res, &ww, first->s);
-    }
-    
-    for(last= res + 1;
-        last < List->Synonym+List->nsynonyms && !strcmp(wword->word, last->p) ;
-        last++)
-    {
-      UdmWideWordListAddLike(Res, &ww, last->s);
-    }
+    UdmWideWordListAddSynonymIterate(Res, List, res, wword, wword->word, -1);
+    UdmWideWordListAddSynonymIterate(Res, List, res, wword, wword->word, +1);
     
     /* Now find each of them in reverse order */
     nnorm=Res->nwords;
     for(i=0; i < nnorm; i++)
     {
-      syn.p= Res->Word[i].word;
+      UDM_WIDEWORD *ww= &Res->Word[i];
+      /* Skip final synonyms */
+      if (ww->origin == UDM_WORD_ORIGIN_SYNONYM_FINAL)
+        continue;
+      syn.p= ww->word;
       res= UdmBSearch(&syn,List->Synonym,List->nsynonyms,sizeof(UDM_SYNONYM),&cmpsyn);
       
       if(res)
       {
         /* Find first and last synonym */
-        for(first= res;
-            first > List->Synonym && !strcmp(syn.p, first->p) ;
-            first--)
-        {
-          UdmWideWordListAddLike(Res, &ww, first->s);
-        }
-        
-        for(last= res+1;
-            last<List->Synonym+List->nsynonyms && !strcmp(syn.p, last->p);
-            last++)
-        {
-          UdmWideWordListAddLike(Res, &ww, last->s);
-        }
+        UdmWideWordListAddSynonymIterate(Res, List, res, wword, syn.p, -1);
+        /* Note, "ww" is not valid here anymore, realloc could happen */
+        UdmWideWordListAddSynonymIterate(Res, List, res, wword, syn.p, +1);
       }
     }
   }
@@ -314,7 +379,10 @@ UdmSynonymListListFind(const UDM_SYNONYMLISTLIST *SLL, UDM_WIDEWORD *word)
   size_t i;
   UDM_WIDEWORDLIST *Res= (UDM_WIDEWORDLIST *) UdmMalloc(sizeof(*Res));
   UdmWideWordListInit(Res);
-  
+
+  UDM_ASSERT(word->origin >= UDM_WORD_ORIGIN_QUERY &&
+             word->origin <= UDM_WORD_ORIGIN_COLLATION);
+
   for (i= 0; i < SLL->nitems; i++)
     UdmSynonymListFind(Res, &SLL->Item[i], word);
   
