@@ -1144,6 +1144,7 @@ static UDM_CHARSET_ALIAS alias[]={
 	{"utf8",		UDM_CHARSET_UTF8},
 	{"viscii",		UDM_CHARSET_VISCII},
 	{"viscii1.1-1",		UDM_CHARSET_VISCII},
+	{"win-1251",	        UDM_CHARSET_CP1251},
 	{"win1251",	        UDM_CHARSET_CP1251},
 	{"winbaltrim",		UDM_CHARSET_CP1257},
 	{"windows-1250",	UDM_CHARSET_CP1250},
@@ -1156,6 +1157,7 @@ static UDM_CHARSET_ALIAS alias[]={
 	{"windows-1257",	UDM_CHARSET_CP1257},
 	{"windows-1258",	UDM_CHARSET_CP1258},
 	{"windows-874",		UDM_CHARSET_CP874},
+	{"windows-utf-8",	UDM_CHARSET_UTF8},
 	{"x-euc-jp",		UDM_CHARSET_EUC_JP},
 	{"x-mac-cyrillic",	UDM_CHARSET_MACCYRILLIC},
 	{"x-sjis",		UDM_CHARSET_SJIS},
