@@ -55,10 +55,6 @@
 #include "udm_signals.h"
 #include "udm_utils.h"
 
-#ifdef WIN32
-#include <assert.h>
-#endif
-
 
 char udm_null_char= 0;
 
@@ -661,7 +657,7 @@ UdmWriteLock(int fd)
 
   filelen_low= GetFileSize((HANDLE)fd, &filelen_high);
   lock_res= LockFileEx((HANDLE)fd, LOCKFILE_EXCLUSIVE_LOCK, 0, filelen_low, filelen_high, &overlap_str);
-  assert(lock_res);
+  UDM_ASSERT(lock_res);
 #endif
 }
 
@@ -680,7 +676,7 @@ UdmUnLock(int fd) /* ulock an entire file */
 
   filelen_low= GetFileSize((HANDLE)fd, &filelen_high);
   lock_res= UnlockFile((HANDLE)fd, 0, 0, filelen_low, filelen_high);
-  assert(lock_res);
+  UDM_ASSERT(lock_res);
 #endif
 
 }
@@ -704,7 +700,7 @@ UdmReadLock(int fd)
 
   filelen_low= GetFileSize((HANDLE)fd, &filelen_high);
   lock_res= LockFileEx((HANDLE)fd, 0, 0, filelen_low, filelen_high, &overlap_str);
-  assert(lock_res);
+  UDM_ASSERT(lock_res);
 #endif
 }
 
@@ -793,6 +789,20 @@ udm_zint4_write_bits(UDM_ZINT4_STATE *state,
 }
 
 
+/*
+  Encoding components:
+  Bits Bit mask                                         Range
+  ==== ================================================ ==========
+  5           0 xxxx                                    0x0000000F
+  10         10 xxxx xxxx                               0x000000FF
+  15        110 xxxx xxxx xxxx                          0x00000FFF
+  20       1110 xxxx xxxx xxxx xxxx                     0x0000FFFF
+  25      11110 xxxx xxxx xxxx xxxx xxxx                0x000FFFFF
+  30     111110 xxxx xxxx xxxx xxxx xxxx xxxx           0x00FFFFFF
+  35    1111110 xxxx xxxx xxxx xxxx xxxx xxxx xxxx      0x0FFFFFFF
+  40   11111110 xxxx xxxx xxxx xxxx xxxx xxxx xxxx xxxx 0xFFFFFFFF
+*/
+
 void
 udm_zint4(UDM_ZINT4_STATE *state, int next)
 {
