@@ -388,9 +388,10 @@ static int UdmVarListMethodEnvSQLQuery(UDM_VARLIST *Vars, UDM_VAR *Var,
     if (Agent.Conf->dbl.nitems > 0)
     {
       UDM_VAR *New;
+      UDM_DB *db= &Agent.Conf->dbl.db[0];
       char prmname[64];
       Res= UdmMalloc(sizeof(UDM_SQLRES));
-      UdmSQLQuery(&Agent.Conf->dbl.db[0], Res, args[0]->val);
+      UdmSQLQuery(db, Res, args[0]->val);
       UdmVarListDel(Vars, resname);
       UdmVarListAdd(Vars, NULL);
       New= &Vars->Var[Vars->nvars-1];
@@ -400,6 +401,8 @@ static int UdmVarListMethodEnvSQLQuery(UDM_VARLIST *Vars, UDM_VAR *Var,
       UdmVarListSort(Vars);
       udm_snprintf(prmname, sizeof(prmname), "%s.num_rows", resname);
       UdmVarListReplaceUnsigned(Vars, prmname, UdmSQLNumRows(Res));
+      udm_snprintf(prmname, sizeof(prmname), "%s.errstr", resname);
+      UdmVarListReplaceStr(Vars, prmname, db->errstr);
     }
     UdmAgentFree(&Agent);
   }
@@ -544,7 +547,7 @@ UDM_VARHANDLER ResultVar=
   UdmVarCreateResult,
   UdmVarFreeResult,
   UdmVarCopyValueResult,
-  &ResultMethods
+  ResultMethods
 };
 
 static UDM_VARMETHOD SQLResultMethods[]=
@@ -560,7 +563,7 @@ UDM_VARHANDLER SQLResultVar=
   UdmVarCreateSQLResult,
   UdmVarFreeSQLResult,
   UdmVarCopyValueSQLResult,
-  &SQLResultMethods
+  SQLResultMethods
 };
 
 /********************************************************/
@@ -1010,6 +1013,12 @@ UDM_VAR * UdmVarListFindWithValue(UDM_VARLIST * vars,
   return(NULL);
 }
 
+int UdmVarIntVal(UDM_VAR *var)
+{
+  return var->val ? atoi(var->val) : 0;
+}
+
+
 int UdmVarListFindInt(UDM_VARLIST * vars,const char * name,int defval)
 {
   UDM_VAR * var;
